Package: BayesFM
Title: Bayesian Inference for Factor Modeling
Type: Package
Version: 0.1.7
Authors@R: person(
    given = "Rémi", 
    family = "Piatek", 
    role = c("aut", "cre"),
    email = "remi.piatek@gmail.com",
    comment = c(ORCID = "0000-0002-3474-1304"))
Description: Collection of procedures to perform Bayesian analysis on a variety
    of factor models. Currently, it includes: "Bayesian Exploratory Factor 
    Analysis" (befa) from G. Conti, S. Frühwirth-Schnatter, J.J. Heckman, 
    R. Piatek (2014) <doi:10.1016/j.jeconom.2014.06.008>, an approach to 
    dedicated factor analysis with stochastic search on the structure of the 
    factor loading matrix. The number of latent factors, as well as the 
    allocation of the manifest variables to the factors, are not fixed a priori 
    but determined during MCMC sampling.
Depends: R (>= 3.0.0)
Imports: checkmate (>= 1.8.0), coda, ggplot2 (>= 2.1.0), gridExtra,
        plyr (>= 1.8.0)
SystemRequirements: gfortran (>= 4.6.3)
License: GPL-3
NeedsCompilation: yes
Encoding: UTF-8
RoxygenNote: 7.3.1
Packaged: 2024-06-09 07:16:09 UTC; remek
Author: Rémi Piatek [aut, cre] (<https://orcid.org/0000-0002-3474-1304>)
Maintainer: Rémi Piatek <remi.piatek@gmail.com>
Repository: CRAN
Date/Publication: 2024-06-13 21:50:16 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 23:09:06 UTC; unix
Archs: BayesFM.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(plot,befa)
S3method(plot,simul.R.prior)
S3method(plot,simul.nfac.prior)
S3method(print,befa)
S3method(print,simul.R.prior)
S3method(print,simul.nfac.prior)
S3method(print,summary.befa)
S3method(print,summary.simul.R.prior)
S3method(print,summary.simul.nfac.prior)
S3method(summary,befa)
S3method(summary,simul.R.prior)
S3method(summary,simul.nfac.prior)
export(befa)
export(post.column.switch)
export(post.sign.switch)
export(simul.R.prior)
export(simul.dedic.facmod)
export(simul.nfac.prior)
import(checkmate)
importFrom(coda,HPDinterval)
importFrom(coda,as.mcmc)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_rug)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggplot_gtable)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,theme)
importFrom(ggplot2,unit)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(plyr,count)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cov2cor)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,rWishart)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
useDynLib(BayesFM, .registration = TRUE, .fixes = "F_")

# bayesfm - Bayesian Fama-MacBeth

Implementation of ["Bayesian Fama-MacBeth Regressions"](https://papers.ssrn.com/sol3/papers.cfm?abstract_id=4989615) from Bryzgalova, Huang and Julliard (2024).
As presented by the authors, this methodology provides reliable risk premia estimates for both tradable and nontradable factors, 
detects those weakly identified, delivers valid credible intervals for all objects of interest, and is intuitive, fast and simple to implement.

## Installation
```bash
pip install bayesfm
```

## Usage
There is a self-contained [example file](https://github.com/gusamarante/bayesfm/blob/main/example.py) using the Fama-French 25 sorted portfolios and their 5 factors.

There are 3 classes available:
- `BFM`: Bayesian Fama-MacBeth
- `BFMGLS`: Bayesian Fama-MacBeth with the GLS precision matrix for the cross-sectional step
- `BFMOMIT`: Bayesian Fama-MacBeth with omitted factors
  - As noted by the authors, the use of this model requires us to include a sufficient number of latent factors in the cross-sectional step, which is chosen with the `p` argument of this class

All three class save the draws of all elements of interest as attributes, and have a method called `plot_lambda`, which plots the posteriors of the risk premia parameters.
This method outputs the chart below, where the blue density are the posterior draws and the orange lines are the canonical Fama-MacBeth two-pass OLS regression estiamtes.

<p align="center">
  <img src="https://github.com/gusamarante/bayesfm/blob/main/images/bfm_lambda_posterior.png?raw=true" alt="Risk Premia Posterior"/>
</p>

BayesFM Package

Installation notes

- This package requires a Fortran compiler and a C compiler
- GNU Fortran version 4.6.3 or later is recommended
- G95 and earlier versions of GNU Fortran may not work because
  of unsupported Fortran 2003 features used in this package

Windows users:
- It is recommended to install Rtools to compile this package from source,
  see R for Windows FAQ https://cran.r-project.org/bin/windows/base/rw-FAQ.html

#'
#' Bayesian Exploratory Factor Analysis
#'
#' This function implements the Bayesian Exploratory Factor Analysis
#' (\code{befa}) approach developed in Conti et al. (CFSHP, 2014). It runs a
#' MCMC sampler for a factor model with dedicated factors, where each manifest
#' variable is allowed to load on at most one latent factor. The allocation of
#' the manifest variables to the latent factors is not fixed \emph{a priori} but
#' determined stochastically during sampling. The minimum number of variables
#' dedicated to each factor can be controlled by the user to achieve the desired
#' level of identification. The manifest variables can be continuous or
#' dichotomous, and control variables can be introduced as covariates.
#'
#' @param model
#'        This argument specifies the manifest variables and the covariates used
#'        in the model (if any). It can be specified in two different ways:
#'        \itemize{
#'        \item A numeric matrix or a data frame containing the manifest
#'          variables. This corresponds to a model without covariates,
#'          and the argument \code{data} is not required.
#'        \item A list of model formulas. Each element of the list specifies
#'          a manifest variable and its corresponding control variables (e.g.,
#'          '\code{Y1 ~ X1 + X2}' to use \code{X1} and \code{X2} as control
#'          variables for \code{Y1}).
#'          If a formula has no left-hand side variable, covariates on the
#'          right-hand side are included in all equations (e.g., '\code{~ X3}'
#'          means that \code{X3} is used as a control variable for all the
#'          manifest variables). Argument \code{data} can be passed to the
#'          function in that case, otherwise parent data frame is used.
#'        }
#'        Binary manifest variables should be specified as logical vectors in
#'        the data frame to be treated as dichotomous. \code{NA} values are
#'        accepted in manifest variables only.
#' @param data
#'        Data frame. If missing, parent data frame if used.
#' @param burnin
#'        Burn-in period of the MCMC sampler.
#' @param iter
#'        Number of MCMC iterations saved for posterior inference (after
#'        burn-in).
#' @param Nid
#'        Minimum number of manifest variables dedicated to each latent factor
#'        for identification.
#' @param Kmax
#'        Maximum number of latent factors. If missing, the maximum number of
#'        factors that satisfies the identification condition determined by
#'        \code{Nid} and the Ledermann bound is specified (see CFSHP,
#'        section 2.2).
#' @param A0
#'        Scaling parameters of the variance of the Normal prior on the nonzero
#'        factor loadings. Either a scalar or a numeric vector of length equal
#'        to the number of manifest variables.
#' @param B0
#'        Variances of the Normal prior on the regression coefficients. Either a
#'        scalar or a numeric vector of length equal to the number of
#'        manifest variables.
#' @param c0
#'        Shape parameters of the Inverse-Gamma prior on the idiosyncratic
#'        variances. Either a scalar or a numeric vector of length equal to the
#'        number of manifest variables.
#' @param C0
#'        Scale parameters of the Inverse-Gamma prior on the idiosyncratic
#'        variances. Either a scalar or a numeric vector of length equal to the
#'        number of manifest variables.
#' @param HW.prior
#'        If \code{TRUE}, implement Huang-Wand (2013) prior on the covariance
#'        matrix of the factors in the expanded model, otherwise use an
#'        Inverse-Wishart prior if \code{FALSE}, see CFSHP section 2.3.5.
#' @param nu0
#'        Degrees of freedom of the Inverse-Wishart prior on the covariance
#'        matrix of the latent factors in the expanded model.
#' @param S0
#'        Scale parameters of the Inverse-Wishart prior on the covariance matrix
#'        of latent factors in the expanded model:
#'        \itemize{
#'          \item if \code{HW.prior = TRUE}, scale parameter of the Gamma
#'          hyperprior distribution on the individual scales of the
#'          Inverse-Wishart prior.
#'          \item if \code{HW.prior = FALSE}, diagonal elements of the scale
#'          matrix of the Inverse-Wishart prior on the covariance matrix of
#'          the latent factors in the expanded model.
#'        }
#'        Either a scalar or a numeric vector of length equal to \code{Kmax}.
#' @param kappa0
#'        First shape parameter of the Beta prior distribution on the
#'        probability \eqn{\tau_0} that a manifest variable does not load on any
#'        factor.
#' @param xi0
#'        Second shape parameter of the Beta prior distribution on the
#'        probability \eqn{\tau_0} that a manifest variable does not load on any
#'        factor.
#' @param kappa
#'        Concentration parameters of the Dirichlet prior distribution on the
#'        indicators. Either a scalar or a numeric vector of length equal to
#'        \code{Kmax}.
#' @param indp.tau0
#'        If \code{TRUE}, specify the alternative prior specification with
#'        independent parameters \eqn{\tau_{0m}}{\tau_0m} across manifest
#'        variables \eqn{m = 1, ..., M}, otherwise use a common parameter
#'        \eqn{\tau_0} if \code{FALSE}.
#' @param rnd.step
#'        If \code{TRUE}, select randomly the number of intermediate steps in
#'        non-identified models at each MCMC iteration, otherwise use a fixed
#'        number of steps if \code{FALSE}.
#' @param n.step
#'        Controls the number of intermediate steps in non-identified models:
#'        \itemize{
#'          \item if \code{rnd.step = TRUE}, average number of steps. The number
#'          of steps is sampled at each MCMC iteration from
#'          1+Poisson(\code{n.step}-1).
#'          \item if \code{rnd.step = FALSE}, fixed number of steps.
#'        }
#' @param search.delay
#'        Number of MCMC iterations run with fixed indicator matrix (specified
#'        in \code{dedic.start}) at beginning of MCMC sampling.
#' @param R.delay
#'        Number of MCMC iterations run with fixed correlation matrix (specified
#'        in \code{dedic.start}) at beginning of MCMC sampling.
#' @param alpha.start
#'        Starting values for the factor loadings. Numeric vector of length
#'        equal to the number of manifest variables. If missing, sampled from a
#'        Normal distribution with zero mean and variance \code{A0}.
#' @param dedic.start
#'        Starting values for the indicators. Vector of integers of length equal
#'        to the number of manifest variables. Each element takes a value among
#'        0, 1, ..., \code{Kmax}. If missing, random allocation of the manifest
#'        variables to the maximum number of factors \code{Kmax}, with a minimum
#'        of \code{Nid} manifest variables loading on each factor.
#' @param sigma.start
#'        Starting values for the idiosyncratic variances. Numeric vector of
#'        length equal to the number of manifest variables. Sampled from prior
#'        if missing.
#' @param beta.start
#'        Starting values for the regression coefficients. Numeric vector of
#'        length equal to the total number of regression coefficients,
#'        concatenated for all the manifest variables. Sampled from prior if
#'        missing.
#' @param R.start
#'        Starting values for the correlation matrix of the latent factors.
#'        Numeric matrix with \code{Kmax} rows and columns, and unit diagonal
#'        elements. If missing, identity matrix is used.
#' @param verbose
#'        If \code{TRUE}, display information on the progress of the function.
#'
#' @details \strong{Model specification.} The model is specified as follows, for
#' each observation \eqn{i = 1, ..., N}:
#'   \deqn{Y^*_i = \beta X_i + \alpha \theta_i + \epsilon_i}{
#'         Y*_i = \beta X_i + \alpha \theta_i + \epsilon_i}
#'   \deqn{\theta_i \sim \mathcal{N}(0, R)}{\theta_i ~ N(0, R)}
#'   \deqn{\epsilon_i \sim \mathcal{N}(0, \Sigma)}{\epsilon_i ~ N(0, \Sigma)}
#'   \deqn{\Sigma = diag(\sigma^2_1, ..., \sigma^2_M)}
#' where \eqn{Y^*_i}{Y*_i} is the \eqn{M}-vector containing the latent
#' variables underlying the corresponding \eqn{M} manifest variables
#' \eqn{Y_i}, which can be continuous such that
#' \eqn{Y_{im} = Y^*_{im}}{Y_im = Y*_im}, or binary with
#' \eqn{Y_{im} = 1[Y^*_{im} > 0]}{Y_im = 1[Y*_im > 0]}, for \eqn{m = 1, ..., M}.
#' The \eqn{K}-vector \eqn{\theta_i} contains the latent factors, and
#' \eqn{\alpha} is the \eqn{(M \times K)}{(M*K)}-matrix of factor loadings.
#' The \eqn{M}-vector \eqn{\epsilon_i} is the vector of error terms.
#' Covariates can be included in the \eqn{Q}-vector \eqn{X_i} and are related to
#' the manifest variables through the \eqn{(M \times Q)}{(M*Q)}-matrix of
#' regression coefficients \eqn{\beta}. Intercept terms are automatically
#' included, but can be omitted in some or all equations using the usual syntax
#' for R formulae (e.g., 'Y1 ~ X1 - 1' specifies that that Y1 is regressed on X1
#' and no intercept is included in the corresponding equation).
#'
#' The number of latent factors \eqn{K} is specified as \code{Kmax}. However,
#' during MCMC sampling the stochastic search process on the matrix \eqn{\alpha}
#' may produce zero columns, thereby reducing the number of active factors.
#'
#' The covariance matrix \eqn{R} of the latent factors is assumed to be a
#' correlation matrix for identification.
#'
#' Each row of the factor loading matrix \eqn{\alpha} contains at most one
#' nonzero element (dedicated factor model). The allocation of the manifest
#' variables to the latent factors is indicated by the binary matrix
#' \eqn{\Delta} with same dimensions as \eqn{\alpha}, such that each row
#' \eqn{\Delta_m} indicates which factor loading is different from zero, e.g.:
#' \deqn{\Delta_m = (0, .., 0, 1, 0, ..., 0) \equiv e_k}{
#'       \Delta_m = (0, .., 0, 1, 0, ..., 0) = e_k}
#' indicates that variable \eqn{m} loads on the \eqn{k}th factor, where
#' \eqn{e_k} is a \eqn{K}-vector that contains only zeros, besides its \eqn{k}th
#' element that equals 1.
#'
#' \strong{Identification.} The function verifies that the maximum number of
#' latent factors \code{Kmax} does not exceed the Ledermann bound. It also
#' checks that \code{Kmax} is consistent with the identification restriction
#' specified with \code{Nid} (enough variables should be available to load on
#' the factors when \code{Kmax} is reached). The default value for \code{Kmax}
#' is the minimum between the Ledermann bound and the maximum number of factors
#' that can be loaded by \code{Nid} variables. The user is free to select the
#' level of identification, see CFSHP section 2.2 (non-identified models are
#' allowed with \code{Nid = 1}).
#'
#' Note that identification is achieved only with respect to the scale of the
#' latent factors. Non-identifiability problems may affect the posterior sample
#' because of column switching and sign switching of the factor loadings.
#' These issues can be addressed \emph{a posteriori} with the functions
#' \code{\link{post.column.switch}} and \code{\link{post.sign.switch}}.
#'
#' \strong{Prior specification.}
#' The indicators are assumed to have the following probabilities,
#' for \eqn{k = 1, ..., K}:
#' \deqn{Prob(\Delta_m = e_k \mid \tau_k) = \tau_k}{
#'       Prob(\Delta_m = e_k | \tau_k) = \tau_k}
#' \deqn{\tau = (\tau_0, \tau_1, ..., \tau_K)}
#' If \code{indp.tau0 = FALSE}, the probabilities are specified as:
#' \deqn{\tau = [\tau_0, (1-\tau_0)\tau^*_1, ..., (1-\tau_0)\tau^*_K]}{
#'       \tau = [\tau_0, (1-\tau_0)\tau*_1, ..., (1-\tau_0)\tau*_K]}
#' \deqn{\tau_0 \sim \mathcal{B}eta(\kappa_0, \xi_0)}{
#'       \tau_0 ~ Beta(\kappa_0, \xi_0)}
#' \deqn{\tau^* = (\tau^*_1, ..., \tau^*_K) \sim \mathcal{D}ir(\kappa)}{
#'       \tau* = (\tau*_1, ..., \tau*_K) ~ Dir(\kappa)}
#' with \eqn{\kappa_0} = \code{kappa0}, \eqn{\xi_0} = \code{xi0} and
#' \eqn{\kappa} = \code{kappa}.
#' Alternatively, if \code{indp.tau0 = TRUE}, the probabilities are specified
#' as:
#' \deqn{\tau_m = [\tau_{0m}, (1-\tau_{0m})\tau^*_1, ...,
#'         (1-\tau_{0m})\tau^*_K]}{
#'       \tau_m = [\tau_0m, (1-\tau_0m)\tau*_1, ..., (1-\tau_0m)\tau*_K]}
#' \deqn{\tau_{0m} \sim \mathcal{B}eta(\kappa_0, \xi_0)}{
#'       \tau_0m ~ Beta(\kappa_0, \xi_0)}
#' for each manifest variable \eqn{m = 1, ..., M}.
#'
#' A normal-inverse-Gamma prior distribution is assumed on the nonzero factor
#' loadings and on the idiosyncratic variances:
#' \deqn{\sigma^2_m \sim \mathcal{I}nv-\mathcal{G}amma(c_{0m}, C_{0m})}{
#'       \sigma^2_m ~ Inv-Gamma(c0_m, C0_m)}
#' \deqn{\alpha_m^\Delta \mid \sigma^2_m \sim \mathcal{N}(0, A_{0m}\sigma^2_m)}{
#'       \alpha_m^\Delta | \sigma^2_m ~ N(0, A0_m * \sigma^2_m)}
#' where \eqn{\alpha_m^\Delta} denotes the only nonzero loading in the \eqn{m}th
#' row of \eqn{\alpha}.
#'
#' For the regression coefficients, a multivariate Normal prior distribution is
#' assumed on each row \eqn{m = 1, ..., M} of \eqn{\beta}:
#' \deqn{\beta_m \sim \mathcal{N}(0, B_0 I_Q)}{\beta_m ~ N(0, B_0 I_Q)}
#' The covariates can be different across manifest variables, implying zero
#' restrictions on the matrix \eqn{\beta}. To specify covariates, use a list
#' of formulas as \code{model} (see example below). Intercept terms can be
#' introduced using
#'
#' To sample the correlation matrix \eqn{R} of the latent factors, marginal data
#' augmentation is implemented (van Dyk and Meng, 2001), see CFSHP section 2.2.
#' Using the transformation \eqn{\Omega = \Lambda^{1/2} R \Lambda^{1/2}}, the
#' parameters \eqn{\Lambda = diag(\lambda_1, ..., \lambda_K)} are used as
#' \emph{working parameters}. These parameters correspond to the variances of
#' the latent factors in an expanded version of the model where the factors do
#' not have unit variances. Two prior distributions can be specified on the
#' covariance matrix \eqn{\Omega} in the expanded model:
#' \itemize{
#'   \item If \code{HW.prior = FALSE}, inverse-Wishart distribution:
#'   \deqn{\Omega \sim \mathcal{I}nv-\mathcal{W}ishart(\nu_0, diag(S_0))}{
#'         \Omega ~ Inv-Wishart(\nu_0, diag(S0))}
#'   with \eqn{\nu_0} = \code{nu0} and \eqn{S_0} = \code{S0}.
#'   \item If \code{HW.prior = TRUE}, Huang-Wand (2013) prior:
#'   \deqn{\Omega \sim \mathcal{I}nv-\mathcal{W}ishart(\nu_0, W), \qquad
#'         W = diag(w_1, ..., w_K)}{
#'         \Omega ~ Inv-Wishart(nu0, W), W = diag(w_1, ..., w_K)}
#'   \deqn{w_k \sim \mathcal{G}amma\left(\frac{1}{2},
#'         \frac{1}{2\nu^*S_{0k}}\right)}{w_k ~ Gamma(1/2, 1/(2\nu*S0_k))}
#'   with \eqn{\nu^*}{\nu*} = \code{nu0} - \code{Kmax} + 1, and the shape and
#'   rate parameters are specified such that the mean of the gamma distribution
#'   is equal to \eqn{\nu^* S_{0k}}{\nu* S0_k}, for each \eqn{k = 1, ..., K}.
#' }
#'
#' \strong{Missing values.} Missing values (\code{NA}) are allowed in the
#' manifest variables. They are drawn from their corresponding conditional
#' distributions during MCMC sampling. Control variables with missing values
#' can be passed to the function. However, all the observations with at least
#' one missing value in the covariates are discarded from the sample (a warning
#' message is issued in that case).
#'
#' @return The function returns an object of class '\code{befa}' containing the
#' MCMC draws of the model parameters saved in the following matrices (each
#' matrix has '\code{iter}' rows):
#' \itemize{
#'   \item \code{alpha}: Factor loadings.
#'   \item \code{sigma}: Idiosyncratic variances.
#'   \item \code{R}: Correlation matrix of the latent factors (off-diagonal
#'         elements only).
#'   \item \code{beta}: regression coefficients (if any).
#'   \item \code{dedic}: indicators (integers indicating on which factors the
#'         manifest variable load).
#'   }
#' The returned object also contains:
#' \itemize{
#'   \item \code{nfac}: Vector of number of 'active' factors across MCMC
#'         iterations (i.e., factors loaded by at least \code{Nid} manifest
#'         variables).
#'   \item \code{MHacc}: Logical vector indicating accepted proposals of
#'         Metropolis-Hastings algorithm.
#' }
#' The parameters \code{Kmax} and \code{Nid} are saved as object attributes, as
#' well as the function call and the number of mcmc iterations (\code{burnin}
#' and \code{iter}), and two logical variables indicating if the returned object
#' has been post processed to address the column switching problem
#' (\code{post.column.switch}) and the sign switching problem
#' (\code{post.sign.switch}).
#'
#' @author Rémi Piatek \email{remi.piatek@@gmail.com}
#'
#' @references G. Conti, S. Frühwirth-Schnatter, J.J. Heckman, R. Piatek (2014):
#' ``Bayesian Exploratory Factor Analysis'', \emph{Journal of Econometrics},
#' 183(1), pages 31-57, \url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.
#'
#' @references A. Huang, M.P. Wand (2013):
#' ``Simple Marginally Noninformative Prior Distributions for Covariance
#' Matrices'', \emph{Bayesian Analysis}, 8(2), pages 439-452,
#' \url{http://dx.doi.org/10.1214/13-BA815}.
#'
#' @references D.A. van Dyk, X.-L. Meng (2001):
#' ``The Art of Data Augmentation'',
#' \emph{Journal of Computational and Graphical Statistics}, 10(1), pages 1-50,
#' \url{http://dx.doi.org/10.1198/10618600152418584}.
#'
#' @seealso \code{\link{post.column.switch}} and \code{\link{post.sign.switch}}
#' for column switching and sign switching of the factor loading matrix and of
#' the correlation matrix of the latent factors to restore identification
#' \emph{a posteriori}.
#'
#' @seealso \code{\link{summary.befa}} and \code{\link{plot.befa}} to summarize
#' and plot the posterior results.
#'
#' @seealso \code{\link{simul.R.prior}} and \code{\link{simul.nfac.prior}} to
#' simulate the prior distribution of the correlation matrix of the factors and
#' the prior distribution of the indicator matrix, respectively. This is useful
#' to perform prior sensitivity analysis and to understand the role of the
#' corresponding parameters in the factor search.
#'
#' @examples
#' #### model without covariates
#'
#' set.seed(6)
#'
#' # generate fake data with 15 manifest variables and 3 factors
#' N <- 100    # number of observations
#' Y <- simul.dedic.facmod(N, dedic = rep(1:3, each = 5))
#'
#' # run MCMC sampler
#' # notice: 1000 MCMC iterations for illustration purposes only,
#' #  increase this number to obtain reliable posterior results!
#' mcmc <- befa(Y, Kmax = 5, iter = 1000)
#'
#' # post process MCMC draws to restore identification
#' mcmc <- post.column.switch(mcmc)
#' mcmc <- post.sign.switch(mcmc)
#' \donttest{
#' summary(mcmc)  # summarize posterior results
#' plot(mcmc)     # plot posterior results
#'
#' # summarize highest posterior probability (HPP) model
#' summary(mcmc, what = 'hppm')
#'
#' #### model with covariates
#'
#' # generate covariates and regression coefficients
#' Xcov <- cbind(1, matrix(rnorm(4*N), ncol = 4))
#' colnames(Xcov) <- c('(Intercept)', paste0('X', 1:4))
#' beta <- rbind(rnorm(15), rnorm(15), diag(3) %x% t(rnorm(5)))
#'
#' # add covariates to previous model
#' Y <- Y + Xcov %*% beta
#'
#' # specify model
#' model <- c('~ X1',                        # X1 covariate in all equations
#'            paste0('Y', 1:5,   ' ~ X2'),   # X2 covariate for Y1-Y5 only
#'            paste0('Y', 6:10,  ' ~ X3'),   # X3 covariate for Y6-Y10 only
#'            paste0('Y', 11:15, ' ~ X4'))   # X4 covariate for Y11-Y15 only
#' model <- lapply(model, as.formula)        # make list of formulas
#'
#' # run MCMC sampler, post process and summarize
#' mcmc <- befa(model, data = data.frame(Y, Xcov), Kmax = 5, iter = 1000)
#' mcmc <- post.column.switch(mcmc)
#' mcmc <- post.sign.switch(mcmc)
#' mcmc.sum <- summary(mcmc)
#' mcmc.sum
#'
#' # compare posterior mean of regression coefficients to true values
#' beta.comp <- cbind(beta[beta != 0], mcmc.sum$beta[, 'mean'])
#' colnames(beta.comp) <- c('true', 'mcmc')
#' print(beta.comp, digits = 3)
#' }
#'
#' @export befa
#' @import checkmate
#' @importFrom stats rnorm runif
#' @useDynLib BayesFM, .registration = TRUE, .fixes = "F_"

befa <- function(model, data, burnin = 1000, iter = 10000, Nid = 3, Kmax,
                 A0 = 10, B0 = 10, c0 = 2, C0 = 1, HW.prior = TRUE,
                 nu0 = Kmax + 1, S0 = 1, kappa0 = 2, xi0 = 1, kappa = 1/Kmax,
                 indp.tau0 = TRUE, rnd.step = TRUE, n.step = 5,
                 search.delay = min(burnin, 10), R.delay = min(burnin, 100),
                 dedic.start, alpha.start, sigma.start, beta.start, R.start,
                 verbose = TRUE)
{

  checkArgs <- makeAssertCollection()

  ##############################################################################
  ## data and model specification

  if (missing(data))
    data <- parent.frame()
  else
    assertDataFrame(data)

  if (is.matrix(model))
    model <- as.data.frame(model)

  if (is.data.frame(model)) {

    assertDataFrame(model, types = c('double', 'logical'), all.missing = FALSE)
    Ytype <- sapply(model, typeof)
    Yobs  <- as.matrix(model)
    Xobs  <- nX <- 0
    YXloc <- FALSE

  } else if (is.list(model) &
             all(sapply(model, is.formula))) {

    tmp <- extract.data(model, data)
    if (!is.null(tmp$errmsg)) {
      for (w in tmp$errmsg) checkArgs$push(w)
    } else {
      for (w in tmp$warnmsg) warning(w, immediate. = TRUE)
      Yobs  <- tmp$Yobs
      Ytype <- tmp$Ytype
      Xobs  <- tmp$Xobs
      Xlab  <- colnames(Xobs)
      YXloc <- tmp$YXloc
      nX    <- ncol(Xobs)
    }

  } else {

    checkArgs$push('Y should be a matrix, a data frame or a list of formulas.')

  }

  # if any errors, report and stop
  reportAssertions(checkArgs)

  # check manifest variables are either continuous or dichotomous
  Ycat <- rep(0, length(Ytype))
  Ycat[Ytype == 'logical'] <- 2
  Yind <- Ytype %in% c('double', 'numeric', 'logical')
  if (any(!Yind)) {
    checkArgs$push(paste('following variables not continuous nor dichotomous:',
                   paste0(Ylab[!Yind], collapse = ', ')))
  }

  Ylab  <- colnames(Yobs)
  nobs  <- nrow(Yobs)
  nvar  <- ncol(Yobs)
  nbeta <- sum(YXloc)
  Ymiss <- is.na(Yobs)
  Yobs[Ymiss] <- -99999  # flag for NA (not used in Fortran subroutine)


  ##############################################################################
  ## number of latent factors and identification restrictions

  # minimum number of dedicated variables per factor
  assertInt(Nid, lower = 1, upper = nvar, add = checkArgs)

  # check maximum number of latent factors and Ledermann bound
  Ledermann.bound <- 0.5 * (2 * nvar + 1 - sqrt(8 * nvar + 1))
  if (missing(Kmax)) {
    Kmax <- floor(min(nvar/Nid, Ledermann.bound))
  } else {
    assertInt(Kmax, lower = 1, upper = nvar, add = checkArgs)
  }
  if (Kmax > Ledermann.bound) {
    warning("Check identification! (Kmax exceeds Ledermann bound)",
            immediate. = TRUE)
  }

  # check consistency of Nid and Kmax
  if (Kmax > floor(nvar/Nid)) {
    msg <- paste("Too many latent factors specified given identification",
                 "restriction. Check arguments Nid and Kmax.")
    checkArgs$push(msg)
  }

  # throw warning in case of single-factor model
  if (Kmax == 1) {
    checkArgs$push("Single-factor model not allowed.")
  }

  reportAssertions(checkArgs)


  ##############################################################################
  ## prior parameters

  # function checking prior parameter values
  check.prior <- function(x, n, min, name) {
    if (length(x) == 1)
      x <- rep(x, n)
    assertNumeric(x, len = n, lower = min, finite = TRUE, any.missing = FALSE,
      .var.name = name, add = checkArgs)
    return(x)
  }

  tiny <- 10^-9
  A0 <- check.prior(A0, nvar, tiny, "A0")
  B0 <- check.prior(B0, nvar, tiny, "B0")
  c0 <- check.prior(c0, nvar, tiny, "c0")
  C0 <- check.prior(C0, nvar, tiny, "C0")
  S0 <- check.prior(S0, Kmax, tiny, "S0")
  nu0 <- check.prior(nu0, 1, Kmax, "nu0")
  xi0 <- check.prior(xi0, 1, tiny, "xi0")
  kappa <- check.prior(kappa, Kmax, tiny, "kappa")
  kappa0 <- check.prior(kappa0, 1, tiny, "kappa0")

  # use Huang-Wand (2013) prior?
  assertFlag(HW.prior, add = checkArgs)

  # use specific tau0 parameters for each manifest variable? [see CFSHP, p.36]
  assertFlag(indp.tau0, add = checkArgs)

  # if any errors, stop here
  reportAssertions(checkArgs)

  # prior values for Fortran subroutine
  prior.idiovar <- cbind(c0, C0)
  prior.loading <- 1/A0  # precision passed to Fortran subroutine
  prior.beta <- 1/B0  # precision passed to Fortran subroutine
  prior.dedic <- c(indp.tau0, 1/A0, c0, C0, xi0, kappa0, kappa)
  prior.facdist <- c(HW.prior, nu0, S0)


  ##############################################################################
  ## starting values

  ### idiosyncratic variances
  if (missing(sigma.start)) {
    sigma.start <- 1/rgamma(nvar, shape = c0, rate = C0)
  } else {
    assertNumeric(sigma.start, len = nvar, lower = tiny, any.missing = FALSE,
      add = checkArgs)
  }
  sigma.start[Ycat > 0] <- 1  # fix variance to 1 for binary variables

  ### factor loadings
  if (missing(alpha.start)) {
    alpha.start <- rnorm(nvar, mean = 0, sd = sqrt(A0))
  } else {
    assertNumeric(alpha.start, len = nvar, any.missing = FALSE,
                  add = checkArgs)
  }

  ### regression coefficients
  if (missing(beta.start)) {
    if (nbeta > 0) {
      beta.start <- rnorm(nbeta, mean = 0, sd = rep(sqrt(B0), rowSums(YXloc)))
    } else {
      beta.start <- double()
    }
  }
  assertNumeric(beta.start, len = nbeta, any.missing = FALSE, add = checkArgs)
  # prepare matrix to be passed to Fortran subroutine
  beta.start.1 <- matrix(-99999, nX, nvar)
  if (length(beta.start) == nbeta) {
    beta.start.1[t(YXloc)] <- beta.start
    beta.start.1 <- t(beta.start.1)
  }

  ### correlation matrix of latent factors
  if (missing(R.start)) {
    R.start <- diag(Kmax)
  }
  # check matrix is a correlation matrix, positive semi-definite, and invertible
  assertMatrix(R.start, mode = "double", nrows = Kmax, ncols = Kmax,
               any.missing = FALSE, add = checkArgs)
  if (!all(diag(R.start) == 1)) {
    checkArgs$push("R.start should be a correlation matrix.")
  }
  if (!is.pos.semidefinite.matrix(R.start)) {
    checkArgs$push("R.start should be a positive semi-definite matrix.")
  }
  if (!is.invertible.matrix(R.start)) {
    checkArgs$push("R.start is not invertible (singular matrix).")
  }

  ### indicators - default: maximum number of factors, random allocation
  if (missing(dedic.start)) {
    dedic.start <- rep(0, nvar)
    ind <- matrix(sample(Nid * Kmax), ncol = Kmax)
    for (k in 1:Kmax) dedic.start[ind[, k]] <- k
    dedic.start[dedic.start == 0] <- sample(Kmax, nvar - Nid * Kmax,
                                            replace = TRUE)
  }
  assertIntegerish(dedic.start, len = nvar, lower = 0, upper = Kmax,
                   any.missing = FALSE, add = checkArgs)
  # check identification constraint
  if (any(table(dedic.start[dedic.start != 0]) < Nid)) {
    checkArgs$push("dedic.start does not correspond to an identified model.")
  }

  ### latent factors
  start.factor <- replicate(Kmax, rnorm(nobs))


  ##############################################################################
  ## MCMC tuning

  assertCount(iter, positive = TRUE, add = checkArgs)
  assertCount(burnin, positive = FALSE, add = checkArgs)
  assertInt(search.delay, lower = 0, upper = burnin + iter)
  assertInt(R.delay, lower = 0, upper = burnin + iter)
  assertFlag(rnd.step, add = checkArgs)
  if (rnd.step) {
    assertNumber(n.step, lower = 1.1, add = checkArgs)
    n.step <- n.step - 1  # so that number of steps ~ 1 + Poisson(n.step-1)
  } else {
    assertCount(n.step, positive = TRUE, add = checkArgs)
  }


  ##############################################################################
  ## if any errors in arguments, report and stop execution

  assertFlag(verbose, add = checkArgs)
  reportAssertions(checkArgs)


  ##############################################################################
  ## MCMC sampling

  # total number of model parameters
  npar <- c(nvar, nvar, Kmax*(Kmax - 1)/2, nbeta)
  npar.all <- sum(npar)

  # seed for RNG in Fortran subroutine
  seed <- round(runif(1) * 10^9)

  # call Fortran subroutine
  if (verbose) cat("starting MCMC sampling...\n")
  mcmc <- .Fortran(F_befa,
                   as.integer(nvar),
                   as.integer(nobs),
                   as.integer(Kmax),
                   as.integer(Nid),
                   as.double(Yobs),
                   as.integer(Ycat),
                   as.logical(Ymiss),
                   as.integer(nX),
                   as.double(Xobs),
                   as.logical(YXloc),
                   as.integer(burnin + iter),
                   as.integer(burnin),
                   as.integer(search.delay),
                   as.integer(R.delay),
                   as.logical(rnd.step),
                   as.double(n.step),
                   as.integer(seed),
                   as.double(cbind(prior.loading, prior.idiovar)),
                   as.double(prior.beta),
                   as.double(prior.dedic),
                   as.double(prior.facdist),
                   as.double(cbind(alpha.start, sigma.start)),
                   as.double(beta.start.1),
                   as.integer(dedic.start),
                   as.double(start.factor),
                   as.double(R.start),
                   as.logical(verbose),
                   as.integer(npar.all),
                   MCMCdraws = double(iter * npar.all),
                   MCMCdedic = integer(iter * nvar),
                   MHacc = logical(iter))
  if (verbose) cat("done with sampling!\n")


  ##############################################################################
  ## label MCMC draws and return output

  # extract MCMC draws
  par.mcmc <- split(mcmc$MCMCdraws, rep(1:4, times = iter * npar))
  par.mcmc <- lapply(par.mcmc, matrix, nrow = iter)

  # label parameters
  names(par.mcmc)[1:3] <- c("alpha", "sigma", "R")
  colnames(par.mcmc$R) <- paste("R", rep(1:(Kmax-1), times = (Kmax-1):1),
                                 unlist(mapply(seq, 2:Kmax, Kmax)), sep = ":")
  colnames(par.mcmc$alpha) <- paste0("alpha:", Ylab)
  colnames(par.mcmc$sigma) <- paste0("sigma:", Ylab)

  iter.lab <- burnin + 1:iter
  rownames(par.mcmc$alpha) <- iter.lab
  rownames(par.mcmc$sigma) <- iter.lab
  rownames(par.mcmc$R)     <- iter.lab

  if (nbeta > 0) {
    names(par.mcmc)[4] <- "beta"
    beta.lab <- c()
    for (i in 1:nvar) {
      if (!any(YXloc[i, ])) next
      beta.lab <- c(beta.lab, paste(Ylab[i], Xlab[YXloc[i, ]], sep = ":"))
    }
    colnames(par.mcmc$beta) <- beta.lab
    rownames(par.mcmc$beta) <- iter.lab
  }

  # indicators
  dedic.mcmc <- as.integer(mcmc$MCMCdedic)
  dedic.mcmc <- matrix(dedic.mcmc, nrow = iter, ncol = nvar)
  colnames(dedic.mcmc) <- paste0("dedic:", Ylab)
  rownames(dedic.mcmc) <- iter.lab

  # number of active latent factors across MCMC iterations
  nfac.mcmc <- apply(dedic.mcmc, 1, count.unique.nonzero)

  # prepare and return output
  output       <- par.mcmc
  output$dedic <- dedic.mcmc
  output$nfac  <- nfac.mcmc
  output$MHacc <- mcmc$MHacc
  attr(output, "call")   <- match.call()
  attr(output, "title")  <- "BEFA posterior sample"
  attr(output, "Kmax")   <- Kmax
  attr(output, "Nid")    <- Nid
  attr(output, "iter")   <- iter
  attr(output, "burnin") <- burnin
  attr(output, "post.column.switch") <- FALSE
  attr(output, "post.sign.switch")   <- FALSE
  class(output) <- "befa"
  return(output)

}

module measurement_class

  use global
  use probability, only : rnorm, rtnorm
  implicit none


  private
  public :: measurement
  public :: measurement_cont
  public :: measurement_bin


  type :: measurement_cont
    integer               :: nobs
    real(r8), allocatable :: Y(:)
    logical,  allocatable :: Ymiss(:)
    ! back up
    real(r8), allocatable :: Y_bak(:)
  contains
    procedure, public :: init    => init_measurement
    procedure, public :: update  => update_measurement_cont
    procedure, public :: backup  => backup_measurement
    procedure, public :: restore => restore_measurement
  end type measurement_cont


  type, extends(measurement_cont) :: measurement_bin
    logical, allocatable :: Ybin(:)
  contains
    procedure, public :: update => update_measurement_bin
  end type measurement_bin


  type :: measurement
    class(measurement_cont), allocatable :: p
  end type measurement


contains


  !-----------------------------------------------------------------------------

  subroutine init_measurement(this, nobs, Y, Ymiss)
    implicit none
    class(measurement_cont)        :: this
    integer,  intent(in)           :: nobs
    real(r8), intent(in)           :: Y(nobs)
    logical,  intent(in), optional :: Ymiss(nobs)
    integer                        :: i

    this%nobs = nobs
    allocate(this%Y(nobs))

    if(present(Ymiss)) then
      if(any(Ymiss)) then
        allocate(this%Ymiss(nobs))
        this%Ymiss = Ymiss
      end if
    end if

    select type (this)

      type is (measurement_cont)
        this%Y = Y
        if(allocated(this%Ymiss)) then
          allocate(this%Y_bak(nobs))
          do i = 1, nobs
            if(.not.this%Ymiss(i)) cycle
            this%Y(i) = rnorm()
          end do
        end if

      type is (measurement_bin)
        allocate(this%Ybin(nobs))
        allocate(this%Y_bak(nobs))
        this%Ybin = int(Y) == 1
        do i = 1, nobs
          this%Y(i) = abs(rnorm())
        end do
        if(allocated(this%Ymiss)) then
          do i = 1, nobs
            if(.not.this%Ymiss(i) .and. .not.this%Ybin(i)) then
              this%Y(i) = -this%Y(i)
            end if
          end do
        else
          do i = 1, nobs
            if(.not.this%Ybin(i)) then
              this%Y(i) = -this%Y(i)
            end if
          end do
        end if

    end select

    if(allocated(this%Y_bak)) this%Y_bak = this%Y

  end subroutine init_measurement


  !-----------------------------------------------------------------------------

  subroutine update_measurement_cont(this, mean, var)
    implicit none
    class(measurement_cont) :: this
    real(r8), intent(in)    :: mean(this%nobs)
    real(r8), intent(in)    :: var
    integer                 :: i

    if(.not.allocated(this%Ymiss)) return

    do i = 1, this%nobs
      if(this%Ymiss(i)) then
        this%Y(i) = rnorm(mean(i), var)
      end if
    end do

  end subroutine update_measurement_cont


  !-----------------------------------------------------------------------------

  subroutine update_measurement_bin(this, mean, var)
    implicit none
    class(measurement_bin) :: this
    real(r8), intent(in)   :: mean(this%nobs)
    real(r8), intent(in)   :: var
    integer                :: i

    if(allocated(this%Ymiss)) then

      do i = 1, this%nobs
        if(this%Ymiss(i)) then
          this%Y(i) = rnorm(mean(i), var)
        else
          this%Y(i) = rtnorm(mean(i), var, 0._r8, this%Ybin(i))
        end if
      end do

    else

      do i = 1, this%nobs
        this%Y(i) = rtnorm(mean(i), var, 0._r8, this%Ybin(i))
      end do

    end if

  end subroutine update_measurement_bin


  !-----------------------------------------------------------------------------

  subroutine backup_measurement(this)
    implicit none
    class(measurement_cont) :: this

    if(.not.allocated(this%Y_bak)) return

    this%Y_bak = this%Y

  end subroutine backup_measurement


  !-----------------------------------------------------------------------------

  subroutine restore_measurement(this)
    implicit none
    class(measurement_cont) :: this

    if(.not.allocated(this%Y_bak)) return

    this%Y = this%Y_bak

  end subroutine restore_measurement


end module measurement_class

#' @export
#' @importFrom ggplot2 ggplot aes_string geom_bar labs theme element_blank

plot.simul.nfac.prior <- function(x, ...)
{
  nfac <- summary(x)$nfac
  dat  <- data.frame(freq = unlist(nfac),
                     nfac = c(as.integer(sapply(nfac, names))),
                     lab  = rep(names(nfac), sapply(nfac, length)))

  ggplot(dat, aes_string(x = 'nfac', y = 'freq')) +
    geom_bar(aes_string(fill = 'lab'), position = 'dodge', stat = 'identity') +
    labs(x = 'number of factors', y = 'prior probability') +
    theme(legend.position  = 'bottom',
          legend.direction = 'horizontal',
          legend.title     = element_blank())
}

#' @export

print.summary.simul.R.prior <- function(x, ...)
{

  cat('prior distribution of correlation matrix of latent factors\n\n')

  cat('maximum correlation (in absolute value):\n\n')
  print(x$maxcor$stat,  print.gap = 3)
  cat('\n')
  print(x$maxcor$quant, print.gap = 3)
  cat('\n')

  cat('minimum eigenvalue of correlation matrix:\n\n')
  print(x$mineig$stat,  print.gap = 3)
  cat('\n')
  print(x$mineig$quant, print.gap = 3)

}

#' @export

print.summary.simul.nfac.prior <- function(x, ...)
{
  cat('prior probabilities of numbers of factors:\n')
  out <- cbind(do.call(rbind, x$nfac), acc = unlist(x$acc))
  out <- round(out, digits = 3)
  print(out, print.gap = 3)
}

module covariates_class

  use global
  use probability, only : rnorm
  use matrix,      only : chol, solvl, solvu
  implicit none

  private
  public :: covariates
  public :: get_all_covariates

  type :: covariates
    integer               :: nobs
    integer               :: npar
    real(r8), allocatable :: beta(:)
    real(r8), allocatable :: X(:,:)
    real(r8), allocatable :: XX(:,:)
    real(r8), allocatable :: Xbeta(:)
    real(r8)              :: prec0
    ! back up
    real(r8), allocatable :: beta_bak(:)
    real(r8), allocatable :: Xbeta_bak(:)
  contains
    procedure, public :: init    => init_covariates
    procedure, public :: update  => update_covariates
    procedure, public :: backup  => backup_covariates
    procedure, public :: restore => restore_covariates
  end type covariates


contains


  !-----------------------------------------------------------------------------

  subroutine init_covariates(this, nobs, npar, X, prior, start, mask)
    implicit none
    class(covariates)              :: this
    integer,  intent(in)           :: nobs
    integer,  intent(in)           :: npar
    real(r8), intent(in)           :: X(nobs, npar)
    real(r8), intent(in)           :: prior
    real(r8), intent(in)           :: start(npar)
    logical,  intent(in), optional :: mask(npar)
    integer                        :: i

    this%nobs = nobs

    if(present(mask)) then
      this%npar = count(mask)
    else
      this%npar = npar
    end if

    allocate(this%Xbeta(nobs))
    this%Xbeta = 0._r8

    if(this%npar == 0) return    ! nothing more to allocate if no covariates

    allocate(this%beta(this%npar))
    allocate(this%X(nobs, this%npar))
    allocate(this%XX(this%npar, this%npar))
    allocate(this%beta_bak(this%npar))
    allocate(this%Xbeta_bak(nobs))

    if(present(mask)) then
      forall(i = 1:nobs)
        this%X(i,:) = pack(X(i,:), mask)
      end forall
      this%beta = pack(start, mask)
    else
      this%X    = X
      this%beta = start
    end if

    this%XX    = matmul(transpose(this%X), this%X)
    this%Xbeta = matmul(this%X, this%beta)
    this%prec0 = prior

    this%beta_bak  = this%beta
    this%Xbeta_bak = this%Xbeta

  end subroutine init_covariates


  !-----------------------------------------------------------------------------

  subroutine update_covariates(this, Y, prec)
    implicit none
    class(covariates)                          :: this
    real(r8), dimension(this%nobs), intent(in) :: Y
    real(r8),                       intent(in) :: prec
    real(r8), dimension(this%npar, this%npar)  :: beta_B, beta_L
    real(r8), dimension(this%npar)             :: beta_m, beta_z, beta_e
    integer                                    :: i

    if(this%npar == 0) return

    beta_m = prec * matmul(transpose(this%X), Y)
    beta_B = prec * this%XX
    forall(i = 1:this%npar) beta_B(i,i) = beta_B(i,i) + this%prec0
    beta_L = chol(beta_B)
    beta_z = solvl(beta_L, beta_m)

    do i = 1, this%npar
      beta_e(i) = rnorm()
    end do
    this%beta  = solvu(transpose(beta_L), beta_e + beta_z)
    this%Xbeta = matmul(this%X, this%beta)

  end subroutine update_covariates


  !-----------------------------------------------------------------------------

  subroutine backup_covariates(this)
    implicit none
    class(covariates) :: this

    if(this%npar == 0) return

    this%beta_bak  = this%beta
    this%Xbeta_bak = this%Xbeta

  end subroutine backup_covariates


  !-----------------------------------------------------------------------------

  subroutine restore_covariates(this)
    implicit none
    class(covariates) :: this

    if(this%npar == 0) return

    this%beta  = this%beta_bak
    this%Xbeta = this%Xbeta_bak

  end subroutine restore_covariates


  !-----------------------------------------------------------------------------

  function get_all_covariates(X) result(par)
    implicit none
    type(covariates) :: X(:)
    real(r8)         :: par(sum(X%npar))
    integer          :: i, j

    i = 0
    do j = 1, size(X)
      if(X(j)%npar == 0) cycle
      par(i+1:i+X(j)%npar) = X(j)%beta
      i = i + X(j)%npar
    end do

  end function get_all_covariates


end module covariates_class

#'
#' Simulate prior distribution of factor correlation matrix
#'
#' This function produces a sample of correlation matrices drawn from their
#' prior distribution induced in the identified version of the factor model,
#' given the prior distribution specified on the corresponding covariance
#' matrices of the factors in the expanded model.
#'
#' @inheritParams befa
#' @param Kmax Maximum number of latent factors.
#' @param nrep Number of Monte Carlo replications.
#'
#' @details Covariance matrices are sampled from the prior distribution in the
#' expanded model, and transformed to produce the corresponding correlation
#' matrices. See section 2.3.5 of CFSHP (p.36-37), as well as the details of
#' the function \code{\link{befa}}.
#'
#' To compare several prior specifications, different values of the parameters
#' \code{nu0} and \code{S0} can be specified. The function then simulates for
#' each pair of these parameters. \code{nu0} and \code{S0} should therefore be
#' scalars or vectors of same length.
#'
#' @return A list of length equal to the number of pairs of parameters
#' \code{nu0} and \code{S0}, where each element of the list is an array of
#' dimension (\code{Kmax}, \code{Kmax}, \code{nrep}) that contains the
#' correlation matrices of the latent factors drawn from the prior.
#'
#' @author Rémi Piatek \email{remi.piatek@@gmail.com}
#'
#' @references G. Conti, S. Frühwirth-Schnatter, J.J. Heckman, R. Piatek (2014):
#' ``Bayesian Exploratory Factor Analysis'', \emph{Journal of Econometrics},
#' 183(1), pages 31-57, \url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.
#'
#' @examples
#' # partial reproduction of figure 1 in CFSHP (p.38)
#' # note: use larger number of replications nrep to increase smoothness
#' Kmax <- 10
#' Rsim <- simul.R.prior(Kmax, nu0 = Kmax + c(1, 2, 5), S0 = .5, nrep = 1000)
#' summary(Rsim)
#' plot(Rsim)
#'
#' @export simul.R.prior
#' @import checkmate
#' @importFrom stats rWishart rgamma

simul.R.prior <- function(Kmax, nu0 = Kmax + 1, S0 = 1, HW.prior = TRUE,
                          nrep = 10^5, verbose = TRUE) {

  # sanity checks
  checkArgs <- makeAssertCollection()
  assertInt(Kmax, lower = 1, add = checkArgs)
  assertNumeric(nu0, lower = Kmax, finite = TRUE, any.missing = FALSE,
                min.len = 1, add = checkArgs)
  assertNumeric(S0, lower = 0.000001, finite = TRUE, any.missing = FALSE,
                min.len = 1, add = checkArgs)
  if (length(nu0) == 1) nu0 <- rep(nu0, length(S0))
  if (length(S0) == 1)  S0  <- rep(S0,  length(nu0))
  if (length(nu0) != length(S0))
    checkArgs$push('nu0 and S0 must be of same length')
  assertFlag(HW.prior, add = checkArgs)
  assertCount(nrep, positive = TRUE, add = checkArgs)
  assertFlag(verbose, add = checkArgs)
  reportAssertions(checkArgs)

  Rmat <- list()
  par  <- unique(cbind(nu0, S0))
  for (i in 1:nrow(par))
  {
    nu0i <- par[i, 1]
    S0i  <- par[i, 2]
    cat('simulating prior for nu0 =', nu0i, 'and S0 =', S0i, '\n')
    omg <- rWishart(nrep, df = nu0i, Sigma = diag(Kmax))
    if (HW.prior) {
      nus <- nu0i - Kmax + 1
      Sk  <- replicate(Kmax, rgamma(nrep, shape = .5, rate = 1/(2 * nus * S0i)))
      Sk  <- sqrt(1/Sk)
      omg <- lapply(seq(nrep), function(x)
               diag(Sk[x,]) %*% solve(omg[,,x]) %*% diag(Sk[x,]))
    } else {
      S0d <- sqrt(1/S0i) * diag(Kmax)
      omg <- lapply(seq(nrep), function(x) S0d %*% solve(omg[,,x]) %*% S0d)
    }
    omg <- lapply(omg, cov2cor)
    lab <- paste0('nu0 = ', nu0i, ', S0 = ', S0i)
    Rmat[[lab]] <- simplify2array(omg)
  }

  attr(Rmat, 'call') <- match.call()
  attr(Rmat, 'Kmax') <- Kmax
  attr(Rmat, 'nrep') <- nrep
  class(Rmat) <- 'simul.R.prior'
  return(Rmat)

}

module mda_class

  use global
  use covmat_block_invwishart_class
  use probability, only : rgamma, rinvgamma
  implicit none

  private
  public :: workpar

  type :: workpar
    integer               :: nfac
    integer               :: nmeas
    integer               :: nobs
    real(r8), allocatable :: D(:)
  contains
    procedure, public :: init       => init_workpar
    procedure, public :: expand     => expand_workpar
    procedure, public :: transfback => transform_back_workpar
  end type workpar


contains


  !-----------------------------------------------------------------------------

  subroutine init_workpar(this, nfac, nmeas, nobs)
    implicit none
    class(workpar)       :: this
    integer,  intent(in) :: nfac
    integer,  intent(in) :: nmeas
    integer,  intent(in) :: nobs

    allocate(this%D(nfac))

    this%nfac  = nfac
    this%nmeas = nmeas
    this%nobs  = nobs

  end subroutine init_workpar


  !-----------------------------------------------------------------------------

  subroutine expand_workpar(this, dedic, alpha, covmat)
    implicit none
    class(workpar)                                :: this
    integer,                        intent(in)    :: dedic(this%nmeas)
    real(r8),                       intent(inout) :: alpha(this%nmeas)
    class(covmat_block_invwishart), intent(inout) :: covmat
    integer                                       :: k, l

    select type (Rmat => covmat)
      type is (covmat_block_HuangWand)
        ! sample scale parameters from prior for Huang-Wand prior
        do k = 1, this%nfac
          Rmat%S0(k) = rgamma(.5_r8, 2._r8*Rmat%nus*Rmat%A2k(k))
        end do
    end select

    ! sample working parameters from conditional prior
    do k = 1, this%nfac
      this%D(k) = rinvgamma(.5_r8*covmat%nu0, &
                            .5_r8*covmat%S0(k)*covmat%prec(k,k))
    end do
    this%D = sqrt(this%D)

    ! transform model parameters
    ! (note: factors not tansformed, as they are updated right in the next step)
    do k = 1, this%nfac
      where(dedic == k) alpha = alpha / this%D(k)
    end do
    forall(k = 1:this%nfac, l = 1:this%nfac, k <= l)
      covmat%var(k,l)  = covmat%var(k,l)  * this%D(k) * this%D(l)
      covmat%prec(k,l) = covmat%prec(k,l) / this%D(k) / this%D(l)
      covmat%var(l,k)  = covmat%var(k,l)
      covmat%prec(l,k) = covmat%prec(k,l)
    end forall

  end subroutine expand_workpar



  !-----------------------------------------------------------------------------

  subroutine transform_back_workpar(this, dedic, alpha, covmat, fac)
    implicit none
    class(workpar)                                :: this
    integer,                        intent(in)    :: dedic(this%nmeas)
    real(r8),                       intent(inout) :: alpha(this%nmeas)
    class(covmat_block_invwishart), intent(inout) :: covmat
    real(r8),                       intent(inout) :: fac(this%nobs,this%nfac)
    integer                                       :: k, l

    ! retrieve updated working parameters
    forall(k = 1:this%nfac) this%D(k) = sqrt(covmat%var(k,k))

    ! transform back
    do k = 1, this%nfac
      fac(:,k) = fac(:,k) / this%D(k)
      where(dedic == k) alpha = alpha * this%D(k)
    end do
    forall(k = 1:this%nfac, l = 1:this%nfac, k <= l)
      covmat%var(k,l)  = covmat%var(k,l)  / this%D(k) / this%D(l)
      covmat%prec(k,l) = covmat%prec(k,l) * this%D(k) * this%D(l)
      covmat%var(l,k)  = covmat%var(k,l)
      covmat%prec(l,k) = covmat%prec(k,l)
    end forall

  end subroutine transform_back_workpar


end module mda_class

module covmat_block_invwishart_class

  use global
  use indicators_dedic_class
  use matrix,      only : crossprod, matinv, chol
  use probability, only : rnorm, rgamma, rinvwishart
  implicit none

  private
  public :: covmat_block_invwishart
  public :: covmat_block_HuangWand


  type :: covmat_block_invwishart
    integer               :: nfac
    integer               :: npar
    real(r8), allocatable :: prec(:,:)
    real(r8), allocatable :: var(:,:)
    logical,  allocatable :: var_mask(:,:)  ! mask for lower triangular elements
    real(r8)              :: nu0            ! prior in expanded model:
    real(r8), allocatable :: S0(:)          !   var ~ IWish(nu0, diag(S0))
    real(r8)              :: df_post
    real(r8), allocatable :: L221(:,:)      ! ingredients used to sample
    real(r8), allocatable :: G1112(:,:)     ! inactive factors later
    integer,  allocatable :: ind(:)
    ! back up
    real(r8), allocatable :: var_bak(:,:)
    real(r8), allocatable :: prec_bak(:,:)
  contains
    procedure, public :: init    => init_covmat_block_invwishart
    procedure, public :: update  => update_covmat_block_invwishart
    procedure, public :: get     => get_covmat_block_invwishart
    procedure, public :: backup  => backup_covmat_block_invwishart
    procedure, public :: restore => restore_covmat_block_invwishart
  end type covmat_block_invwishart


  type, extends(covmat_block_invwishart) :: covmat_block_HuangWand
    real(r8), allocatable :: A2k(:)
    real(r8)              :: nus
    real(r8)              :: Rmat_b_shape
  end type covmat_block_HuangWand


contains


  !-----------------------------------------------------------------------------

  subroutine init_covmat_block_invwishart(this, nobs, nfac, prior, start)
    implicit none
    class(covmat_block_invwishart) :: this
    integer,  intent(in)           :: nobs
    integer,  intent(in)           :: nfac
    real(r8), intent(in)           :: prior(1:nfac+1)
    real(r8), intent(in)           :: start(nfac, nfac)
    integer                        :: i, j

    allocate(this%prec(nfac, nfac))
    allocate(this%var(nfac, nfac))
    allocate(this%var_mask(nfac, nfac))
    allocate(this%S0(nfac))
    allocate(this%var_bak(nfac, nfac))
    allocate(this%prec_bak(nfac, nfac))
    this%nfac = nfac
    this%npar = nfac*(nfac-1)/2

    ! mask for lower triangular elements
    this%var_mask = .false.
    forall(i = 1:nfac, j = 1:nfac, j < i)
      this%var_mask(i,j) = .true.
    end forall

    ! starting values
    this%var = start
    this%prec = matinv(this%var)
    this%var_bak = start
    this%prec_bak = this%prec

    ! prior
    this%nu0 = prior(1)
    this%S0  = prior(2:)
    this%df_post = this%nu0 + dble(nobs)

    ! additional parameters for Huang-wand prior
    select type (this)
      type is (covmat_block_HuangWand)
        allocate(this%A2k(nfac))
        this%A2k = prior(2:)
        this%nus = this%nu0 - this%nfac + 1._r8
        this%Rmat_b_shape = .5_r8*(this%nu0 + 1._r8)
    end select

    ! factor indicators 1, ..., nfac
    allocate(this%ind(nfac))
    this%ind = (/ (i, i=1, nfac) /)

  end subroutine init_covmat_block_invwishart


  !-----------------------------------------------------------------------------

  subroutine update_covmat_block_invwishart(this, fac, dedic)
    implicit none
    class(covmat_block_invwishart)        :: this
    real(r8), dimension(:,:), intent(in)  :: fac
    class(indic_dedic),       intent(in)  :: dedic
    integer,  dimension(:),   allocatable :: acti, inacti
    real(r8), dimension(:,:), allocatable :: S_post, G221, G12
    real(r8)                              :: df
    real(r8)                              :: Rmat_b_rate
    integer                               :: K1, K2, k, l

    K1 = dedic%K1       ! active factors
    K2 = dedic%K2       ! inactive factors
    allocate(acti(K1))
    acti = pack(this%ind, dedic%active)

    !----- update scale parameters for Huang-Wand prior

    select type (this)
      type is (covmat_block_HuangWand)
        do k = 1, this%nfac
          Rmat_b_rate = .5_r8*(this%prec(k,k) + 1._r8/(this%nus*this%A2k(k)))
          this%S0(k) = rgamma(this%Rmat_b_shape, 1._r8/Rmat_b_rate)
        end do
    end select

    !----- sample submatrix for active factors

    if(K1 > 0) then

      allocate(S_post(K1, K1))

      S_post = crossprod(fac(:,acti))
      forall(k = 1:K1) S_post(k,k) = S_post(k,k) + this%S0(acti(k))
      df = this%df_post - dble(K2)
      this%var(acti,acti) = rinvwishart(df, S_post)

      deallocate(S_post)

    end if

    !----- sample remaining block corresponding to inactive factors

    if(allocated(this%L221)) deallocate(this%L221)

    if(K1 == 0) then      ! only inactive factors, sample matrix from prior

      allocate(this%L221(K2,K2), S_post(K2,K2))
      S_post = 0._r8
      forall(k = 1:K2) S_post(k,k) = this%S0(k)
      this%var = rinvwishart(this%nu0, S_post)
      this%L221 = chol(this%var)

    else if(K2 > 0) then

      if(allocated(this%G1112)) deallocate(this%G1112)
      allocate(this%L221(K2,K2), this%G1112(K1,K2))
      allocate(inacti(K2), S_post(K2,K2), G221(K2,K2), G12(K1,K2))

      inacti = pack(this%ind, mask=.not.dedic%active)

      S_post = 0._r8
      forall(k = 1:K2) S_post(k,k) = this%S0(inacti(k))
      G221 = rinvwishart(this%nu0, S_post)    ! G221 = Omega_22.1
      this%L221  = chol(G221)
      do k = 1, K1                            ! G1112 = inv(Omega_11) * Omega_12
        do l = 1, K2
          this%G1112(k,l) = rnorm()
        end do
      end do
      this%G1112 = matmul(this%G1112, transpose(this%L221))
      forall(k = 1:K1) this%G1112(k,:) = this%G1112(k,:)/sqrt(this%S0(acti(k)))
      G12 = matmul(this%var(acti,acti), this%G1112)      ! G12 = Omega_12

      this%var(acti, inacti) = G12
      this%var(inacti, acti) = transpose(G12)
      this%var(inacti, inacti) = G221 + matmul(transpose(G12), this%G1112)

    end if

    this%prec = matinv(this%var)

  end subroutine update_covmat_block_invwishart


  !-----------------------------------------------------------------------------

  function get_covmat_block_invwishart(this) result(par)
    implicit none
    class(covmat_block_invwishart) :: this
    real(r8)                       :: par(this%npar)

    par = pack(this%var, this%var_mask)

  end function get_covmat_block_invwishart


  !-----------------------------------------------------------------------------

  subroutine backup_covmat_block_invwishart(this)
    implicit none
    class(covmat_block_invwishart) :: this

    this%var_bak  = this%var
    this%prec_bak = this%prec

  end subroutine backup_covmat_block_invwishart


  !-----------------------------------------------------------------------------

  subroutine restore_covmat_block_invwishart(this)
    implicit none
    class(covmat_block_invwishart) :: this

    this%var  = this%var_bak
    this%prec = this%prec_bak

  end subroutine restore_covmat_block_invwishart


end module covmat_block_invwishart_class

module factor_normal_block_class

  use global
  use factor_normal_class
  use indicators_dedic_class
  use covmat_block_invwishart_class
  use probability, only : rnorm
  use matrix,      only : matinv, chol
  implicit none

  private
  public :: factor_normal_block


  type, extends(factor_normal) :: factor_normal_block
    integer,  allocatable :: indi(:)
    integer,  allocatable :: mi(:)
  contains
    procedure, public :: init          => init_factor_normal
    procedure, public :: update_acti   => update_factor_block_acti
    procedure, public :: update_inacti => update_factor_block_inacti
  end type factor_normal_block


contains


  !-----------------------------------------------------------------------------

  subroutine init_factor_normal(this, nobs, nmeas, nfac, start)
    implicit none
    class(factor_normal_block) :: this
    integer,  intent(in) :: nobs
    integer,  intent(in) :: nmeas
    integer,  intent(in) :: nfac
    real(r8), intent(in) :: start(nobs,nfac)
    integer              :: i

    call this%factor_normal%init(nobs, nmeas, nfac, start)

    allocate(this%indi(nfac))
    allocate(this%mi(nmeas))
    this%indi = (/(i, i=1, this%nfac)/)
    this%mi = (/(i, i=1, this%nmeas)/)

  end subroutine init_factor_normal


  !-----------------------------------------------------------------------------

  subroutine update_factor_block_acti(this, Y, alpha, dedic, idioprec, fdist)
    implicit none
    class(factor_normal_block)                 :: this
    real(r8),                       intent(in) :: Y(this%nobs,this%nmeas)
    real(r8),                       intent(in) :: idioprec(this%nmeas)
    real(r8),                       intent(in) :: alpha(this%nmeas)
    class(indic_dedic),             intent(in) :: dedic
    class(covmat_block_invwishart), intent(in) :: fdist
    logical                                    :: acti(this%nfac)
    integer                                    :: actind(dedic%K1)
    real(r8)                                   :: mean_post(this%nobs,dedic%K1)
    real(r8)                                   :: var_post(dedic%K1,dedic%K1)
    integer                                    :: i, k

    if(dedic%K1 == 0) return    ! no active factors to sample

    acti   = dedic%active
    actind = pack(this%indi, acti)

    ! compute posterior moments for active factors
    var_post = matinv(fdist%var(actind,actind))
    forall(k = 1:dedic%K1)
      var_post(k,k) = var_post(k,k) &
                    + sum(idioprec*(alpha**2), mask=(dedic%group==actind(k)))
      mean_post(:,k) = matmul(Y(:, pack(this%mi, dedic%group==actind(k))), &
                              pack(idioprec*alpha, dedic%group==actind(k)))
    end forall
    var_post = matinv(var_post)

    ! sample active factors factors
    do k = 1, this%nfac
      if(.not.acti(k)) cycle
      do i = 1, this%nobs
        this%theta(i,k) = rnorm()
      end do
    end do
    this%theta(:,actind) = matmul(mean_post, var_post) &
                         + matmul(this%theta(:,actind), &
                                  transpose(chol(var_post)))

  end subroutine update_factor_block_acti


  !-----------------------------------------------------------------------------

  subroutine update_factor_block_inacti(this, fdist, dedic)
    implicit none
    class(factor_normal_block)                 :: this
    class(covmat_block_invwishart), intent(in) :: fdist
    class(indic_dedic),             intent(in) :: dedic
    integer                                    :: actind(dedic%K1)
    integer                                    :: inactind(dedic%K2)
    integer                                    :: i, k

    if(dedic%K2 == 0) return   ! no inactive factors to sample

    actind   = pack(this%indi, dedic%active)
    inactind = pack(this%indi, .not.dedic%active)

    do k = 1, dedic%K2
      do i = 1, this%nobs
        this%theta(i, inactind(k)) = rnorm()
      end do
    end do

    if(dedic%K1 == 0) then

      this%theta = matmul(this%theta, transpose(fdist%L221))

    else

      this%theta(:,inactind) = matmul(this%theta(:,inactind), &
                                      transpose(fdist%L221)) &
                             + matmul(this%theta(:,actind), fdist%G1112)

    end if

  end subroutine update_factor_block_inacti


end module factor_normal_block_class

#'
#' Perform column switchting on posterior MCMC sample
#'
#' This function reorders the columns of the factor loading matrix for each MCMC
#' draw, as well as the rows and columns of the correlation matrix of the
#' factors, to restore the identification of the model \emph{a posteriori} with
#' respect to the column switching problem.
#'
#' @param mcmc
#'        Object of class '\code{befa}'.
#'
#' @details The reordering of the columns of the factor loading matrix is based
#' on the top elements of the columns (i.e., the first row containing a nonzero
#' factor loading in each nonzero column of \eqn{\alpha}, starting from the top
#' of the matrix). At each MCMC iteration, the nonzero columns of \eqn{\alpha}
#' are reordered such that the top elements appear in increasing order.
#' The rows and columns of the correlation matrix \eqn{R} of the factors are
#' switched accordingly. See section 4.3 of CFSHP (p.42) for more details.
#'
#' @return Same '\code{befa}' object as the one passed to the function, where
#' the indicators in the matrix \code{dedic}, as well as the rows and columns of
#' the correlation matrix of the factors saved in \code{draws}, have been
#' switched appropriately to restore the identification of the factor model with
#' respect to column switching.
#'
#' @author Rémi Piatek \email{remi.piatek@@gmail.com}
#'
#' @references
#' G. Conti, S. Frühwirth-Schnatter, J.J. Heckman,
#' R. Piatek (2014): ``Bayesian Exploratory Factor Analysis'',
#' \emph{Journal of Econometrics}, 183(1), pages 31-57,
#' \url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.
#'
#' @seealso \code{\link{post.sign.switch}} to restore identification a
#' posteriori with respect to the sign switching problem.
#'
#' @examples
#' set.seed(6)
#' Y <- simul.dedic.facmod(N = 100, dedic = rep(1:3, each = 5))
#' mcmc <- befa(Y, Kmax = 5, iter = 1000)
#' mcmc <- post.column.switch(mcmc)
#'
#' @export post.column.switch
#' @import checkmate

post.column.switch <- function(mcmc)
{

  assertClass(mcmc, classes = "befa")
  if (attr(mcmc, "post.column.switch")) {
    warning("column switching already performed, nothing done.")
    return(mcmc)
  }

  # issue warning if M-H acceptance rate too low
  if (mean(mcmc$MHacc) < 0.2) {
    warning(paste("M-H acceptance rate of sampler is low (< 0.20).",
                  "Check convergence and mixing!"))
  }

  Kmax   <- attr(mcmc, "Kmax")
  nvar   <- ncol(mcmc$dedic)
  iter   <- nrow(mcmc$dedic)
  R.npar <- Kmax * (Kmax - 1)/2

  # index matrix used to reconstruct matrix from lower triangular elements
  R.mat <- diag(Kmax) * (R.npar + 1)
  R.mat[lower.tri(R.mat)] <- 1:R.npar
  R.mat[upper.tri(R.mat)] <- t(R.mat)[upper.tri(R.mat)]

  v <- 1:Kmax
  for (i in 1:iter) {
    d <- mcmc$dedic[i, ]

    # relabel indicators
    mcmc$dedic[i, ] <- relabel.dedic(d)

    # reorder rows and columns of correlation matrix
    u <- unique(d[d != 0])
    r <- c(u, v[!v %in% u])
    R <- c(mcmc$R[i, ], 1)
    R <- matrix(R[R.mat], nrow = Kmax)
    R <- R[r, r]
    mcmc$R[i, ] <- R[lower.tri(R)]
  }

  attr(mcmc, "post.column.switch") <- TRUE
  return(mcmc)

}

module matrix

  use global
  implicit none

  private
  public :: trace
  public :: chol
  public :: det
  public :: matinv
  public :: solvl
  public :: solvu
  public :: crossprod
  public :: outerprod


  interface outerprod
    module procedure outerprod1
    module procedure outerprod2
  end interface outerprod


contains


  !-----------------------------------------------------------------------------
  ! returns the trace of the matrix A

  real(r8) function trace(A) result(t)
    implicit none
    real(r8), intent(in) :: A(:,:)
    integer :: i, n

    n = size(A, 1)
    if(n /= size(A, 2)) then
      call rexit('### ERROR: trace only for square matrices')
    end if

    t = 0._r8
    do i = 1, n
      t = t + A(i,i)
    end do

  end function trace


  !-----------------------------------------------------------------------------
  ! Cholesky decomposition of a positive-definite symmetric matrix A:
  !     A = L * L'
  ! Input:
  !     A        positive-definite symmetric matrix to be decomposed
  !              (only upper triangular part of A is used)
  ! Output:
  !     Achol    Cholesky factor L is returned

  function chol(A) result(Achol)
    implicit none
    real(r8), intent(in) :: A(:,:)
    real(r8) :: Achol(size(A,1), size(A,2))
    real(r8) :: p(size(A,1))
    real(r8) :: asum
    integer  :: i, j, n

    n = size(A,1)
    if(n .ne. size(A,2)) then
      call rexit('*** ERROR: matrix is not square (chol) ***')
    end if

    Achol = A
    do i = 1, n
      asum = Achol(i,i) - sum(Achol(i,1:i-1)**2)
      if(asum <= 0.0_r8) call rexit('*** ERROR: chol failed')
      p(i) = sqrt(asum)
      Achol(i,i) = p(i)
      Achol(i+1:n,i) = (Achol(i,i+1:n) - matmul(Achol(i+1:n,1:i-1), &
                                                Achol(i,1:i-1))) / p(i)
    end do

    forall(i = 1:n, j = 1:n, i < j) Achol(i,j) = 0._r8

  end function chol


  !------------------------------------------------------------------------------
  ! solves the set of linear equations R * x = b
  ! where R is a triangular matrix that is either upper triangular (solvu)
  ! or lower triangular (solvl)

  function solvu(U, b) result(x)
    ! U is upper triangular
    implicit none
    real(r8), intent(in) :: U(:,:)
    real(r8), intent(in) :: b(:)
    real(r8) :: x(size(b))
    integer  :: i, n

    ! check diagonal elements are not zero
    n = size(b)
    do i = 1, n
      if(abs(U(i,i)) > 0._r8) cycle
      call rexit('*** ERROR: zero diagonal element(s) (solvu) ***')
    end do

    ! solve equations
    x(n) = b(n) / U(n,n)
    do i = n-1, 1, -1
      x(i) = ( b(i) - dot_product(U(i,i+1:n), x(i+1:n)) ) / U(i,i)
    end do

  end function solvu

  function solvl(L, b) result(x)
    ! L is lower triangular
    implicit none
    real(r8), intent(in) :: L(:,:)
    real(r8), intent(in) :: b(:)
    real(r8)             :: x(size(b))
    integer              :: i, n

    ! check diagonal elements are not zero
    n = size(b)
    do i = 1, n
      if(abs(L(i,i)) > 0._r8) cycle
      call rexit('*** ERROR: zero diagonal element(s) (solvl) ***')
    end do

    ! solve equations
    x(1) = b(1) / L(1,1)
    do i = 2, n
      x(i) = ( b(i) - dot_product(L(i,1:i-1), x(1:i-1)) ) / L(i,i)
    end do

  end function solvl


  !-----------------------------------------------------------------------------
  ! computes the inverse of the matrix A using the LU decomposition
  ! depends on LAPACK
  ! see http://fortranwiki.org/fortran/show/Matrix+inversion

  function matinv(A) result(Ainv)
    real(r8), intent(in) :: A(:,:)
    real(r8) :: Ainv(size(A,1), size(A,2))
    real(r8) :: work(size(A,1))
    integer  :: piv(size(A,1))
    integer  :: n, info

    ! external LAPACK procedures
    external DGETRF
    external DGETRI

    ! check matrix is square
    n = size(A,1)
    if(n .ne. size(A,2)) then
      call rexit('*** ERROR: matrix is not square (matinv) ***')
    end if

    ! compute LU factorization with LAPACK subroutine DGETRF
    ! using partial pivoting with interchange rows
    Ainv = A
    call DGETRF(n, n, Ainv, n, piv, info)

    if(info /= 0) then
      call rexit('*** ERROR: singular matrix (matinv) ***')
    end if

    ! compute the inverse of the matrix using LAPACK subroutine DGETRI,
    ! using LU factorization computed by DGETRF
    call DGETRI(n, Ainv, n, piv, work, n, info)

    if(info /= 0) then
      call rexit('*** ERROR: matrix inversion failed (matinv) ***')
    end if

  end function matinv


  !-----------------------------------------------------------------------------
  ! computes the determinant of the matrix A using the LU decomposition

  real(r8) function det(A)
    implicit none
    real(r8), intent(in) :: A(:,:)
    real(r8) :: Ain(size(A,1), size(A,2))
    integer  :: piv(size(A,1))
    integer  :: i, n, info

    ! external LAPACK procedures
    external DGETRF

    ! check matrix is square
    n = size(A,1)
    if(n .ne. size(A,2)) then
      call rexit('*** ERROR: matrix is not square (matinv) ***')
    end if

    ! DGETRF computes an LU factorization of a general M-by-N matrix A
    ! using partial pivoting with row interchanges.
    Ain = A
    call DGETRF(n, n, Ain, n, piv, info)

    if(info /= 0) then
      call rexit('*** ERROR: LU decomposition failed (det) ***')
    end if

    ! compute determinant
    det = 1._r8
    do i = 1, n
      if(piv(i) /= i) then
        det = -det * Ain(i,i)
      else
        det = det * Ain(i,i)
      end if
    end do

  end function det


  !-----------------------------------------------------------------------------
  ! returns the cross-product A' * A of the matrix A

  function crossprod(A) result(AA)
    implicit none
    real(r8), intent(in) :: A(:,:)
    real(r8) :: AA(size(A,2), size(A,2))
    integer  :: i, j, n

    n = size(A,2)
    forall(i = 1:n, j = 1:n, i <= j)
      AA(i,j) = dot_product(A(:,i), A(:,j))
      AA(j,i) = AA(i,j)
    end forall

  end function crossprod


  !-----------------------------------------------------------------------------
  ! compute outer product p = a * a' or p = a * b'

  function outerprod1(a) result(p)
    real(r8), intent(in) :: a(:)
    real(r8)             :: p(size(a), size(a))

    p = spread(a, dim=2, ncopies=size(a))
    p = p * transpose(p)

  end function outerprod1

  function outerprod2(a, b) result(p)
    real(r8), intent(in) :: a(:)
    real(r8), intent(in) :: b(:)
    real(r8)             :: p(size(a), size(b))

    p = spread(a, dim=2, ncopies=size(b)) * &
        spread(b, dim=1, ncopies=size(a))

  end function outerprod2


end module matrix

#'
#' Summarize 'befa' object
#'
#' Generic function summarizing the posterior results of a 'befa' object.
#' Optional arguments can be specified to customize the summary.
#'
#' @param object
#'        Object of class 'befa'.
#' @param ...
#'        The following extra arguments can be specified:
#' \itemize{
#'   \item \code{what}: How to summarize the posterior distribution?
#'         \itemize{
#'           \item \code{what = 'maxp'} (default): Only factor loadings with
#'                 highest posterior probability of being different from zero or
#'                 discarded from the model (if \code{dedic = 0}) are
#'                 summarized.
#'           \item \code{what = 'all'}: All factor loadings with corresponding
#'                 posterior probability to be allocated to a given factor (or
#'                 to be discarded from the model) larger than \code{min.prob}
#'                 are summarized.
#'           \item \code{what = 'hppm'}: Highest posterior probability models
#'                 with probability larger than \code{min.prob} are summarized.
#'         }
#'   \item \code{byfac}: Sort factor loadings by factors if \code{TRUE},
#'         otherwise by manifest variables if \code{FALSE} (default).
#'   \item \code{hpd.prob}: Probability used to compute the highest posterior
#'         density intervals of the posterior distribution of the model
#'         parameters (default: 0.95).
#'   \item \code{min.prob}: If \code{what = 'all'}, only factor loadings with
#'         posterior probability of being dedicated to a given factor (or
#'         discarded from the model) larger than this value are displayed.
#'         If \code{what = 'hppm'}, only highest posterior probability models
#'         with probability larger than this value are displayed. (default:
#'         0.20)
#' }
#'
#' @details
#' This function summarizes the posterior distribution of the parameters.
#' The algorithm may visit different configurations of the indicator matrix
#' \eqn{\Delta} during sampling, where the manifest variables are allocated to
#' different latent factors. When the posterior distribution of the factor
#' loadings is summarized separately for each manifest variable
#' (\code{what = 'maxp'} or \code{what = 'all'}), the function provides the
#' latent factor each manifest variable is allocated to (\code{dedic}), and the
#' corresponding posterior probability (\code{prob}). If \code{dedic = 0}, then
#' \code{prob} corresponds to the posterior probability that the manifest
#' variable is discarded. Discarded variables are listed last if
#' \code{byfac = TRUE}. Low probability cases can be discarded by setting
#' \code{min.prob} appropriately (default is 0.20).
#'
#' Idiosyncratic variances, factor correlation matrix and regression
#' coefficients (if any) are summarized across all MCMC iterations if
#' \code{what = 'all'} or \code{what = 'maxp'}, and within each HPP model if
#' \code{what = 'hppm'}.
#'
#' \strong{Highest posterior probability model.}
#' The HPP model is the model with a given allocation of the measurements to the
#' latent factors (i.e., a given indicator matrix \eqn{\Delta}) that is visited
#' most often by the algorithm.
#'
#' When specifying \code{what = 'hppm'}, the function sorts the models according
#' to the posterior frequencies of their indicator matrices in decreasing order.
#' Therefore, the first model returned (labeled 'm1') corresponds to the HPP
#' model.
#' Low probability models can be discarded by setting \code{min.prob}
#' appropriately(default is 0.20, implying that only models with a posterior
#' probability larger than 0.20 are displayed).
#'
#' HPP models can only be found if identification with respect to column
#' switching has been restored \emph{a posteriori}. An error message is returned
#' if this is not the case.
#'
#' @return If called directly, the summary is formatted and displayed on the
#' standard output. Otherwise if saved in an object, a list of the following
#' elements is returned:
#' \itemize{
#'  \item \code{MHacc}: Metropolis-Hastings acceptance rate.
#'  \item \code{alpha}: Data frame (or list of data frames if
#'        \code{what = 'hppm'}) containing posterior summary statistics for the
#'        factor loadings.
#'  \item \code{sigma}: Data frame (or list of matrices if \code{what = 'hppm'})
#'        containing posterior summary statistics for the idiosyncratic
#'        variances.
#'  \item \code{R}: Data frame (or list of data frames if \code{what = 'hppm'})
#'        containing posterior summary statistics for the factor correlations.
#'  \item \code{beta}: Data frame (or list of data frames if
#'        \code{what = 'hppm'}) containing posterior summary statistics for the
#'        regression coefficients (if any).
#'  \item \code{nfac} (only if \code{what = 'maxp'} or \code{what = 'all'}):
#'        Table of posterior frequencies of numbers of factors.
#'  \item \code{hppm} (only if \code{what = 'hppm'}): List of the following
#'        elements summarizing the different HPP models, sorted in decreasing
#'        order of their posterior probabilities:
#'        \itemize{
#'          \item \code{prob}: Vector of posterior probabilities.
#'          \item \code{nfac}: Vector of numbers of factors.
#'          \item \code{dedic}: Data frame of factor indicators.
#'        }
#' }
#' Data frames of posterior summary statistics include the means (\code{mean}),
#' standard deviations (\code{sd}) and highest posterior density intervals
#' (\code{hpd.lo} and \code{hpd.up}, for the probability specified in
#' \code{hpd.prob}) of the corresponding parameters.
#'
#' For the factor loadings, the matrix may also include a column labeled
#' '\code{dedic}' indicating to which factors the corresponding manifest
#' variables are dedicated (a zero value means that the manifest variable does
#' not load on any factor), as well as a column labeled '\code{prob}' showing
#' the corresponding posterior probabilities that the manifest variables load on
#' these factors.
#'
#' Summary results are returned as lists of data frames for HPP models, where
#' the elements of the list are labeled as '\code{m1}, '\code{m2}', etc.
#'
#' @author Rémi Piatek \email{remi.piatek@@gmail.com}
#'
#' @seealso \code{\link{plot.befa}} to plot posterior results.
#'
#' @examples
#' set.seed(6)
#'
#' # generate fake data with 15 manifest variables and 3 factors
#' Y <- simul.dedic.facmod(N = 100, dedic = rep(1:3, each = 5))
#'
#' # run MCMC sampler and post process output
#' # notice: 1000 MCMC iterations for illustration purposes only,
#' #  increase this number to obtain reliable posterior results!
#' mcmc <- befa(Y, Kmax = 5, iter = 1000)
#' mcmc <- post.column.switch(mcmc)
#' mcmc <- post.sign.switch(mcmc)
#'
#' # summarize posterior results
#' summary(mcmc)
#'
#' # summarize highest posterior probability (HPP) model
#' hppm.sum <- summary(mcmc, what = 'hppm')
#'
#' # print summary with 6-digit precision
#' print(hppm.sum, digits = 6)
#'
#' # extract posterior means of the factor loadings in HPP model
#' alpha.mean <- hppm.sum$alpha$m1$mean
#' print(alpha.mean)
#'
#' @examples \dontshow{
#' summary(mcmc, what = 'maxp', byfac = TRUE)
#' summary(mcmc, what = 'all')
#' summary(mcmc, what = 'all', byfac = TRUE)
#' summary(mcmc, what = 'all', min.prob = 0)
#' summary(mcmc, what = 'all', min.prob = 0, byfac = TRUE)
#' summary(mcmc, what = 'hppm', byfac = TRUE)
#' summary(mcmc, what = 'hppm', min.prob = 0)
#' summary(mcmc, what = 'hppm', min.prob = 0, byfac = TRUE)
#' }
#'
#' @export
#' @import checkmate
#' @importFrom stats sd
#' @importFrom coda as.mcmc HPDinterval
#' @importFrom plyr count

summary.befa <- function(object, ...)
{

  if (class(object) != 'befa')
    stop('object passed to print.befa should be of class befa')

  # extra arguments
  args <- list(...)
  min.prob <- ifelse (is.null(args$min.prob), .20, args$min.prob)
  hpd.prob <- ifelse (is.null(args$hpd.prob), .95, args$hpd.prob)
  byfac    <- ifelse (is.null(args$byfac), FALSE, args$byfac)
  assertNumber(min.prob, lower = 0, upper = 1)
  assertNumber(hpd.prob, lower = 0, upper = 1)
  assertFlag(byfac)
  what <- match.arg(args$what, choices = c('maxp', 'all', 'hppm'))

  # container for summary object
  output <- list()

  ### column-switching and/or sign-switching problems
  if (!attr(object, "post.column.switch")) {
    warning("MCMC output not processed by function 'post.column.switch'. ",
            "Posterior results for factor loadings and factor correlations ",
            "may not be interpretable! Make sure column-switching problem has ",
            "been fixed before summarizing.\n",
            immediate. = TRUE)
    if (what == 'hppm')
      stop("Highest posterior probability model (hppm) can only be searched ",
           " for if columns have be reordered a posteriori. Use function ",
           "'post.column.switch' first.\n")
  }

  if (!attr(object, "post.sign.switch")) {
    warning("MCMC output not processed by function 'post.sign.switch'. ",
            "Posterior results for factor loadings and factor correlations ",
            "may not be interpretable! Make sure sign-switching problem has ",
            "been fixed before summarizing.\n",
            immediate. = TRUE)
  }

  ### Metropolis-Hastings acceptance rate
  output$MHacc <- mean(object$MHacc)
  if (output$MHacc < .2)
    warning('Metropolis-Hastings acceptance rate is low (< 0.20). ',
            'Check convergence and mixing before summarizing.',
            immediate. = TRUE)

  ### function summarizing draws from posterior
  iter <- attr(object, 'iter')
  summarize.mcmc <- function(z, prob = FALSE) {
    if (length(z) == 1) {
      sd  <- 0
      hpd <- c(NA, NA)
    } else {
      sd  <- sd(z)
      hpd <- coda::HPDinterval(coda::as.mcmc(z), prob = hpd.prob)
    }
    res <- c(mean = mean(z), sd = sd, hpd.lo = hpd[1], hpd.up = hpd[2])
    if (prob) res <- c(prob = length(z)/iter, res)
    return(res)
  }

  ### function sorting manifest variables according to the factors they load on
  ### (variables loading on no factors are listed last)
  sort.byfac <- function(a) {
    a[a[, 'dedic'] == 0, 'dedic'] <- NA             # make 0 -> NA
    a <- a[order(a[, 'dedic'], na.last = TRUE), ]   # sort, putting NAs last
    a[is.na(a[, 'dedic']), 'dedic'] <- 0            # make NA -> 0
    return(a)
  }

  ### highest posterior probability models
  if (what == 'hppm') {

    alpha <- object$alpha
    dedic <- object$dedic

    dedic.tab <- plyr::count(as.data.frame(dedic))
    dedic.tab <- dedic.tab[order(dedic.tab$freq, decreasing = TRUE),]
    if (dedic.tab$freq[1] < iter*min.prob)
      stop('Probability of HPP model lower than min.prob. ',
           'Try to decrease the value of min.prob')
    dedic.tab  <- dedic.tab[dedic.tab$freq >= iter*min.prob,]

    hppm.freq  <- dedic.tab$freq/iter
    hppm.dedic <- dedic.tab[-length(dedic.tab)]
    hppm.dedic <- as.data.frame(t(hppm.dedic))
    colnames(hppm.dedic) <- paste0('m', 1:ncol(hppm.dedic))
    hppm.nfac  <- apply(hppm.dedic, 2, count.unique.nonzero)

    output$hppm <- list(prob  = hppm.freq,
                        nfac  = hppm.nfac,
                        dedic = hppm.dedic)

    output$alpha <- list()
    output$sigma <- list()
    output$R     <- list()
    if (!is.null(object$beta)) output$beta <- list()

    for (i in seq_along(hppm.freq)) {

      if (hppm.freq[i] < min.prob) break

      hppm.id <- apply(t(dedic) == hppm.dedic[,i], 2, all)

      a <- apply(alpha[hppm.id, , drop = FALSE], 2, summarize.mcmc)
      a <- cbind(dedic = hppm.dedic[,i], t(a))
      if (byfac) a <- sort.byfac(a)
      rownames(a) <- colnames(alpha)

      lab <- paste0('m', i)
      output$alpha[[lab]] <- a
      output$sigma[[lab]] <- t(apply(object$sigma[hppm.id, , drop = FALSE], 2, 
                                     summarize.mcmc))
      output$R[[lab]]     <- t(apply(object$R[hppm.id, , drop = FALSE], 2, 
                                     summarize.mcmc))
      if (!is.null(object$beta))
        output$beta[[lab]] <- t(apply(object$beta[hppm.id, , drop = FALSE], 2, 
                                      summarize.mcmc))

    }

    if (length(output$alpha) == 0)
      stop("No hpp model found! Try to decrease value of 'min.prob'.")

    # factor loadings: insert NA values when not dedicated to any factor
    for (i in 1:length(output$alpha))
      for (j in 1:nrow(output$alpha[[i]])) {
        if (output$alpha[[i]][j, 'dedic'] == 0)
          output$alpha[[i]][j, c('mean', 'sd', 'hpd.lo', 'hpd.up')] <- NA
      }

  ### general case (not HPP models)
  } else {

    alpha <- mat2list(object$alpha)
    dedic <- mat2list(object$dedic)

    # posterior number of latent factors
    iter <- nrow(object$dedic)
    nfac <- table(object$nfac)/iter
    output$nfac <- nfac[nfac >= .05]   # discard if prob < 0.05

    # summarize posterior results for loadings
    if (what == 'all') {

      bysum <- function(x, d) by(x, d, summarize.mcmc, prob = TRUE)
      a <- mapply(bysum, alpha, dedic, SIMPLIFY = FALSE)
      a <- lapply(a, function(x) do.call(rbind, x))
      a <- lapply(a, function(x) cbind(dedic = as.integer(rownames(x)), x))
      for(i in 1:length(a)) rownames(a[[i]]) <- rep(names(a)[i], nrow(a[[i]]))
      a <- do.call(rbind, a)
      a <- a[a[, 'prob'] >= min.prob,]   # drop rows for which prob < min.prob

    } else if (what == 'maxp') {

      get.max <- function(x) as.integer(names(which.max(table(x))))
      dhp     <- sapply(dedic, get.max)
      hpsum   <- function(x, d, dhp) summarize.mcmc(x[d == dhp], prob = TRUE)
      a <- t(mapply(hpsum, alpha, dedic, dhp))
      a <- cbind(dedic = dhp, a)
      rownames(a) <- names(alpha)

    }
    if (byfac) a <- sort.byfac(a)
    output$alpha <- a

    # summarize posterior results for remaining parameters
    output$sigma <- t(apply(object$sigma, 2, summarize.mcmc))
    output$R     <- t(apply(object$R, 2, summarize.mcmc))
    if (!is.null(object$beta))
      output$beta <- t(apply(object$beta, 2, summarize.mcmc))

    # factor loadings: insert NA values when not dedicated to any factor
    for (i in 1:nrow(output$alpha))
      if (output$alpha[i, 'dedic'] == 0)
        output$alpha[i, c('mean', 'sd', 'hpd.lo', 'hpd.up')] <- NA

  }

  # convert parameter summaries to data frames,
  # rename duplicate row names if necessary (esp. for factor loadings)
  mdf <- function(x) as.data.frame(x, row.names = make.unique(rownames(x)))
  if (what == 'hppm') {
    output$alpha <- lapply(output$alpha, mdf)
    output$sigma <- lapply(output$sigma, mdf)
    output$R     <- lapply(output$R,     mdf)
    if (!is.null(output$beta)) output$beta <- lapply(output$beta, mdf)
  } else {
    output$alpha <- mdf(output$alpha)
    output$sigma <- mdf(output$sigma)
    output$R     <- mdf(output$R)
    if (!is.null(output$beta)) output$beta <- mdf(output$beta)
  }

  ### save attributes and return object
  attr(output, 'Kmax')     <- attr(object, 'Kmax')
  attr(output, 'Nid')      <- attr(object, 'Nid')
  attr(output, 'iter')     <- attr(object, 'iter')
  attr(output, 'burnin')   <- attr(object, 'burnin')
  attr(output, 'what')     <- what
  attr(output, 'min.prob') <- min.prob
  attr(output, 'hpd.prob') <- hpd.prob
  attr(output, 'byfac')    <- byfac

  class(output) <- "summary.befa"
  return(output)

}

### check if object is a formula

is.formula <- function(x) return(class(x) == "formula")


### check if matrix is positive semidefinite

is.pos.semidefinite.matrix <- function(x) {
  if (!is.matrix(x))
    return(FALSE)
  if (!isSymmetric(x, tol = 10^-9))
    return(FALSE)
  return(all(eigen(x, only.values = TRUE)$values >= 0))
}


### check if matrix can be inverted

is.invertible.matrix <- function(x) {
  return(class(try(solve(x), silent = TRUE))[1L] == "matrix")
}


### relabel indicators for dedicated factor model
### example:
###   > d <- c(6, 6, 6, 2, 2, 2, 0, 0, 4, 4, 4)
###   > relabel(d) [1] 1 1 1 2 2 2 0 0 3 3 3

relabel.dedic <- function(d) {
  u <- unique(d[d != 0])
  t <- rep(0, max(d))
  t[u] <- 1:length(u)
  d[d != 0] <- t[d]
  return(d)
}


### count number of unique nonzero elements in x

count.unique.nonzero <- function(x) return(length(unique(x[x != 0])))


### convert matrix to list

mat2list <- function(x, byrow = FALSE) {
  if (byrow) {
    y <- split(x, c(row(x)))
    names(y) <- rownames(x)
  } else {
    y <- split(x, c(col(x)))
    names(y) <- colnames(x)
  }
  return(y)
}


### get maximum correlation (in absolute value) from correlation matrix

get.maxcor <- function(x) max(abs(x[lower.tri(x)]))


### get minimum eigenvalue of symmetric matrix

get.mineig <- function(x) min(eigen(x, symmetric = TRUE,
                                    only.values = TRUE)$values)

module indicators_dedic_class

  use global
  use probability, only : runif
  implicit none

  private
  public :: indic_dedic


  !-----------------------------------------------------------------------------

  type :: ratio_marglik_cat
    integer  :: nfac
    real(r8) :: alpha_prec0
  contains
    procedure :: init => init_ratio_marglik
    procedure :: get  => get_ratio_marglik
  end type ratio_marglik_cat


  type, extends(ratio_marglik_cat) :: ratio_marglik_cont
    real(r8) :: prec_scale0
    real(r8) :: prec_shape
  end type ratio_marglik_cont


  type :: ratio_marglik
    class(ratio_marglik_cat), allocatable :: p
  end type ratio_marglik


  !-----------------------------------------------------------------------------

  type :: param_tau
    integer               :: nmeas
    integer               :: nfac
    real(r8)              :: xi0
    real(r8), allocatable :: kappa0(:)
    real(r8)              :: skappa0
    real(r8), allocatable :: atau(:,:)
    real(r8), allocatable :: btau(:)
    real(r8), allocatable :: stau(:)
  contains
    procedure :: init => init_param_tau
    procedure :: get  => get_param_tau
  end type param_tau

  type, extends(param_tau) :: param_tau_alt
    real(r8) :: kappa0_xi0
  end type param_tau_alt


  !-----------------------------------------------------------------------------

  type :: indic_dedic
    integer                          :: nmeas
    integer                          :: nobs
    integer                          :: nfac
    integer,             allocatable :: group(:)
    integer,             allocatable :: ngroup(:)
    logical,             allocatable :: active(:)   ! TRUE for active factors
    integer                          :: K1          ! number of active factors
    integer                          :: K2          ! number of inactive factors
    type(ratio_marglik), allocatable :: mlik(:)
    class(param_tau),    allocatable :: ltau
    ! back up
    integer,             allocatable :: group_bak(:)
    integer,             allocatable :: ngroup_bak(:)
  contains
    procedure, public :: init    => init_indic_dedic
    procedure, public :: update  => update_indic_dedic
    procedure, public :: backup  => backup_indic_dedic
    procedure, public :: restore => restore_indic_dedic
  end type indic_dedic


contains


  !-----------------------------------------------------------------------------

  subroutine init_ratio_marglik(this, nobs, nfac, prior)
    implicit none
    class(ratio_marglik_cat) :: this
    integer,  intent(in)     :: nobs, nfac
    real(r8), intent(in)     :: prior(3)

    this%nfac = nfac
    this%alpha_prec0 = prior(1)

    select type (this)
      type is (ratio_marglik_cont)
        this%prec_scale0 = prior(3)
        this%prec_shape = prior(2) + .5_r8*dble(nobs)
    end select

  end subroutine init_ratio_marglik


  !-----------------------------------------------------------------------------

  function get_ratio_marglik(this, Y2, fac2, Yfac2) result(lmlik)
    implicit none
    class(ratio_marglik_cat) :: this
    real(r8), intent(in)     :: Y2, fac2(this%nfac), Yfac2(this%nfac)
    real(r8)                 :: Pm(this%nfac), Qm(this%nfac), Rm(this%nfac)
    real(r8)                 :: CNn, lmlik(0:this%nfac, 0:this%nfac)

    select type (this)

      type is (ratio_marglik_cat)

        Pm = 1._r8 + fac2/this%alpha_prec0
        Qm = .5_r8*Yfac2/(this%alpha_prec0+fac2)
        Rm = -.5_r8*log(Pm) + Qm

        lmlik(0, 1:) = Rm

      type is (ratio_marglik_cont)

        Pm = 1._r8 + fac2/this%alpha_prec0
        Qm = .5_r8*Yfac2/(this%alpha_prec0+fac2)

        CNn = this%prec_scale0 + .5_r8*Y2
        Rm = -.5_r8*log(Pm) - this%prec_shape*log(CNn - Qm)

        lmlik(0, 1:) = Rm + this%prec_shape*log(CNn)

    end select

    lmlik(0,0) = 0._r8
    lmlik(1:, 0) = -lmlik(0, 1:)
    lmlik(1:,1:) = spread(Rm, dim=1, ncopies=this%nfac) &
                 - spread(Rm, dim=2, ncopies=this%nfac)

  end function get_ratio_marglik


  !-----------------------------------------------------------------------------

  subroutine init_param_tau(this, nmeas, nfac, prior)
    implicit none
    class(param_tau)       :: this
    integer,  intent(in) :: nmeas, nfac
    real(r8), intent(in) :: prior(0:nfac+1)
    integer              :: j, k

    this%nmeas = nmeas
    this%nfac = nfac

    allocate(this%kappa0(0:this%nfac))
    this%xi0 = prior(0)
    this%kappa0 = prior(1:nfac+1)
    this%skappa0 = sum(this%kappa0(1:))

    select type (this)
      type is (param_tau_alt)
        this%kappa0_xi0 = log(this%kappa0(0)) - log(this%xi0)
    end select

    ! tables to avoid recomputing log function at each MCMC iteration
    allocate(this%atau(0:nmeas, 0:nfac))
    allocate(this%btau(0:nmeas))
    allocate(this%stau(0:nmeas))
    forall(j = 0:nmeas, k = 0:nfac)
      this%atau(j,k) = log(dble(j) + this%kappa0(k) )
    end forall
    forall(j = 0:nmeas)
      this%btau(j) = log(dble(j) + this%xi0)
      this%stau(j) = log(dble(j) + this%skappa0)
    end forall

  end subroutine init_param_tau


  !-----------------------------------------------------------------------------

  function get_param_tau(this, kold, ngroup) result(ltau)
    implicit none
    class(param_tau)    :: this
    integer, intent(in) :: kold, ngroup(this%nfac)
    real(r8)            :: ltau(0:this%nfac, 0:this%nfac)
    real(r8)            :: lngk(this%nfac)
    integer             :: ng(this%nfac)
    integer             :: k, sng

    ng = ngroup
    if(kold > 0) ng(kold) = ng(kold) - 1
    sng = sum(ng)

    forall(k = 1:this%nfac) lngk(k) = this%atau(ng(k), k)
    ltau(0, 0) = 0._r8
    ltau(1:, 1:) = spread(lngk, dim=1, ncopies=this%nfac) &
                 - spread(lngk, dim=2, ncopies=this%nfac)

    select type (this)

      type is (param_tau)
        ltau(1:, 0) = this%atau(this%nmeas-sng-1, 0) &
                    + this%stau(sng) &
                    - this%btau(sng) &
                    - lngk

      type is (param_tau_alt)
        ltau(1:, 0) = this%stau(sng) - lngk + this%kappa0_xi0

    end select

    ltau(0, 1:) = -ltau(1:, 0)

  end function get_param_tau


  !-----------------------------------------------------------------------------

  subroutine init_indic_dedic(this, nobs, nmeas, nfac, Ycat, prior, start)
    implicit none
    class(indic_dedic)   :: this
    integer,  intent(in) :: nobs
    integer,  intent(in) :: nmeas
    integer,  intent(in) :: nfac
    integer,  intent(in) :: Ycat(nmeas)
    real(r8), intent(in) :: prior(0:3*nmeas+nfac+2)
    integer,  intent(in) :: start(nmeas)
    integer              :: j, k
    real(r8)             :: prior_marglik(nmeas, 3)

    allocate(this%group(nmeas))
    allocate(this%ngroup(nfac))
    allocate(this%active(nfac))
    allocate(this%group_bak(nmeas))
    allocate(this%ngroup_bak(nfac))

    this%nmeas = nmeas
    this%nobs = nobs
    this%nfac = nfac
    this%group = start
    this%group_bak = start

    forall(k = 1:this%nfac) this%ngroup(k) = count(this%group == k)
    this%ngroup_bak = this%ngroup
    this%active = this%ngroup > 0
    this%K1 = count(this%active)
    this%K2 = nfac - this%K1

    prior_marglik = reshape(prior(1:3*nmeas), shape=[nmeas,3])
    allocate(this%mlik(nmeas))
    do j = 1, nmeas
      select case (Ycat(j))
        case(0);      allocate(ratio_marglik_cont :: this%mlik(j)%p)
        case default; allocate(ratio_marglik_cat  :: this%mlik(j)%p)
      end select
      call this%mlik(j)%p%init(nobs, nfac, prior_marglik(j,:))
    end do

    select case(int(prior(0)))
      case(0);      allocate(param_tau     :: this%ltau)
      case default; allocate(param_tau_alt :: this%ltau)
    end select
    call this%ltau%init(nmeas, nfac, prior(3*nmeas+1:))

  end subroutine init_indic_dedic


  !-----------------------------------------------------------------------------

  subroutine update_indic_dedic(this, Yaux, Fac)
    implicit none
    class(indic_dedic)   :: this
    real(r8), intent(in) :: Yaux(this%nobs, this%nmeas)
    real(r8), intent(in) :: Fac(this%nobs, this%nfac)
    real(r8)             :: Fac2(this%nfac)
    real(r8)             :: FacYXb2(this%nfac, this%nmeas)
    real(r8)             :: YXb2(this%nmeas)
    real(r8)             :: prob_sub(0:this%nfac, 0:this%nfac)
    real(r8)             :: prob(0:this%nfac)
    real(r8)             :: sprob, e
    integer              :: j, k, kold

    ! cross-products used to compute posterior log odds
    Fac2 = sum(Fac**2, dim=1)
    YXb2 = sum(Yaux**2, dim=1)
    FacYXb2 = matmul(transpose(Fac), Yaux)**2

    do j = 1, this%nmeas

      ! compute posterior probabilities
      kold = this%group(j)
      prob_sub = this%mlik(j)%p%get(YXb2(j), Fac2, FacYXb2(:,j)) &
               + this%ltau%get(kold, this%ngroup)
      prob = 1._r8/sum(exp(prob_sub), dim=2)

      ! sample indicator
      e = runif()
      sprob = 0._r8
      do k = 0, this%nfac
        sprob = sprob + prob(k)
        if(e <= sprob) then
          this%group(j) = k
          if(kold > 0) this%ngroup(kold) = this%ngroup(kold) - 1
          if(k > 0)    this%ngroup(k)    = this%ngroup(k)    + 1
          exit
        end if
      end do

    end do

    ! update indicators for active factors
    this%active = this%ngroup > 0
    this%K1 = count(this%active)
    this%K2 = this%nfac - this%K1

  end subroutine update_indic_dedic


  !-----------------------------------------------------------------------------

  subroutine backup_indic_dedic(this)
    implicit none
    class(indic_dedic) :: this

    this%group_bak  = this%group
    this%ngroup_bak = this%ngroup

  end subroutine backup_indic_dedic


  !-----------------------------------------------------------------------------

  subroutine restore_indic_dedic(this)
    implicit none
    class(indic_dedic) :: this

    this%group  = this%group_bak
    this%ngroup = this%ngroup_bak

  end subroutine restore_indic_dedic


end module indicators_dedic_class

#' @export

print.simul.nfac.prior <- function(x, ...)
{
  cat('sample from prior distribution of number of latent factors\n')
  cat('\ncall:\n  ')
  print(attr(x, 'call'))
  cat('\n')
  cat('nvar =',  attr(x, 'nvar'), '\n')
  cat('Kmax  =', attr(x, 'Kmax'), '\n')
  cat('Nid   =', attr(x, 'Nid'), '\n')
  cat('kappa =', attr(x, 'kappa'), '\n')
  cat('nrep  =', attr(x, 'nrep'), '\n\n')
  cat('use summary() and plot() functions to summarize and plot',
      'prior distribution of number of latent factors', sep = '\n')
}

#' BayesFM: Package for Bayesian Factor Modeling
#'
#' The long-term goal of this package is to provide a collection of procedures
#' to perform Bayesian inference on a variety of factor models.
#'
#' @details Currently, this package includes: Bayesian Exploratory Factor
#' Analysis (\code{befa}), as developed in Conti et al. (2014), an approach to
#' dedicated factor analysis with stochastic search on the structure of the
#' factor loading matrix. The number of latent factors, as well as the
#' allocation of the observed variables to the factors, are not fixed a priori
#' but determined during MCMC sampling. More approaches will be included in
#' future releases of this package.
#'
#' @note This package is under development. You are very welcome to send me any
#' comments or suggestions for improvements, and to share with me any problems
#' you may encounter with the use of this package.
#'
#' @author Rémi Piatek \email{remi.piatek@@gmail.com}
#'
#' @references G. Conti, S. Frühwirth-Schnatter, J.J. Heckman, R. Piatek (2014):
#' ``Bayesian Exploratory Factor Analysis'', \emph{Journal of Econometrics},
#' 183(1), pages 31-57, \url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.
#'
#' @docType package
#' @name BayesFM
NULL

.onAttach <- function(libname, pkgname) {
  if (interactive() || getOption("verbose")) {
    msg <- sprintf(paste(
      "###",
      "### Package %s (%s) loaded",
      "###",
      "### Please report any bugs, and send suggestions or feedback",
      "### to %s",
      "###", sep = "\n"),
      pkgname,
      utils::packageDescription(pkgname)$Version,
      utils::maintainer(pkgname))
    packageStartupMessage(msg)
  }
}

.onUnload <- function(libpath) {
  library.dynam.unload("BayesFM", libpath)
}

module probability

  use global
  use matrix, only : chol, matinv
  implicit none

  private

  public :: set_seed
  public :: runif
  public :: rexpon
  public :: rnorm
  public :: rtnorm
  public :: rgamma
  public :: rinvgamma
  public :: rdirich
  public :: rwishart
  public :: rinvwishart
  public :: rpoisson

  real(r8),    parameter :: pi    = 3.141592653589793238462643383276_r8
  real(r8),    parameter :: half  = .5_r8
  real(r8),    parameter :: third = 1._r8/3._r8


  !----- variables for 64-bit Mersenne Twister algorithm
  integer(i8), parameter :: nn      = 312_i8
  integer(i8), parameter :: mm      = 156_i8
  integer(i8), parameter :: matx_a  = -5403634167711393303_i8
  integer(i8), parameter :: um      = -2147483648_i8 ! most significant 33 bits
  integer(i8), parameter :: lm      =  2147483647_i8 ! least significant 31 bits
  real(r8),    parameter :: pi253_1 = 1._r8/(2._r8**53 - 1._r8)

  integer(i8) :: mt(nn)       ! array for the state vector
  integer     :: mti = nn+1   ! mti==nn+1 means mt(nn) is not initialized


  interface runif
   module procedure runif_01
   module procedure runif_ab
  end interface runif

 interface rnorm
   module procedure rnorm_01
   module procedure rnorm_mu_var
  end interface rnorm


contains


  !-----------------------------------------------------------------------------
  ! 64-bit Mersenne Twister algorithm
  !
  ! Fortran translation from C-program for MT19937-64 (2004/9/29 version)
  ! originally coded by Takuji Nishimura and Makoto Matsumoto
  ! see http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt64.html
  ! translation by Rémi Piatek, University of Copenhagen

  !----- initializes mt(nn) with a seed

  subroutine set_seed(seed)
    implicit none
    integer, intent(in) :: seed
    integer :: i

    mt(1) = int(seed, kind=i8)
    do i = 1, nn-1
      mt(i+1) = 6364136223846793005_i8 * ieor(mt(i), ishft(mt(i), -62)) + i
    end do
    mti = nn

  end subroutine set_seed


  !----- generates a random number on [-2^63, 2^63-1]-interval

  integer(r8) function genrand64_int64()
    implicit none
    integer(i8) :: mag01(0:1) = (/0_i8, matx_a/)
    integer(i8) :: x
    integer     :: i

    if(mti >= nn) then ! generate nn words at one time

      ! if set_seed() has not been called, a default initial seed is used
      if(mti == nn+1) call set_seed(5489)

      do i = 1, nn-mm
        x = ior(iand(mt(i),um), iand(mt(i+1), lm))
        mt(i) = ieor(ieor(mt(i+mm), ishft(x, -1)), mag01(iand(x, 1_i8)))
      end do

      do i = nn-mm+1, nn-1
        x = ior(iand(mt(i), um), iand(mt(i+1), lm))
        mt(i) = ieor(ieor(mt(i+mm-nn), ishft(x, -1)), mag01(iand(x, 1_i8)))
      end do

      x = ior(iand(mt(nn), um), iand(mt(1), lm))
      mt(nn) = ieor(ieor(mt(mm), ishft(x, -1)), mag01(iand(x, 1_i8)))

      mti = 0

    end if

    mti = mti + 1
    x = mt(mti)

    x = ieor(x, iand(ishft(x,-29), 6148914691236517205_i8))
    x = ieor(x, iand(ishft(x, 17), 8202884508482404352_i8))
    x = ieor(x, iand(ishft(x, 37),   -2270628950310912_i8))
    x = ieor(x, ishft(x, -43))

    genrand64_int64 = x

  end function genrand64_int64


  !-----------------------------------------------------------------------------
  ! generates random number from uniform distribution (genrand64_real1)

  real(r8) function runif_01()
    implicit none

    runif_01 = real(ishft(genrand64_int64(), -11), kind=r8) * pi253_1

  end function runif_01


  real(r8) function runif_ab(a, b)
    implicit none
    real(r8), intent(in) :: a, b

    if(b <= a) call rexit('*** ERROR: a should be < b (runif) ***')

    runif_ab = runif_01() * (b-a) + a

  end function runif_ab


  !-----------------------------------------------------------------------------
  ! generates random number from exponential distribution
  ! with scale parameter b such that mean = b and variance = b^2

  real(r8) function rexpon(b)
    implicit none
    real(r8), intent(in) :: b

    if(b <= 0._r8) then
      call rexit('*** ERROR: rate parameter should be > 0 (rexpon) ***')
    end if

    rexpon = -log(runif()) / b

  end function rexpon


  !-----------------------------------------------------------------------------
  ! generates random number from normal distribution
  !
  ! source:
  !   Joseph L. Leva
  !   ``A Fast Normal Random Number Generator''
  !   ACM Transactions on Mathematical Software
  !   Vol. 18, No. 4, December 1992, pages 449-453

  real(r8) function rnorm_01()
    implicit none
    real(r8) :: u, v, x, y, q

    do
      u = runif()
      v = 1.7156_r8*(runif() - 0.5_r8)
      x = u - 0.449871_r8
      y = abs(v) + 0.386595_r8
      q = x**2 + y*(0.19600_r8*y - 0.25472_r8*x)
      if(q < 0.27597_r8) exit
      if(q > 0.27846_r8) cycle
      if(v**2 <= -4._r8*(u**2)*log(u)) exit    ! rarely evaluated
    end do
    rnorm_01 = v/u

  end function rnorm_01


  real(r8) function rnorm_mu_var(mu, var)
    implicit none
    real(r8), intent(in) :: mu, var

    if(var <= 0._r8) then
      call rexit('*** ERROR: var should be positive (rnorm) ***')
    end if

    rnorm_mu_var = mu + sqrt(var)*rnorm_01()

  end function rnorm_mu_var


  !-----------------------------------------------------------------------------
  ! generates random number from truncated normal distribution
  ! with mean mu and standard deviation sd with truncation
  !   (a, +infty)    if left = true
  !   (-infty, a)    if left = false
  !
  ! source:
  !   John Geweke (1991)
  !   ``Efficient Simulation from the Multivariate Normaland Student-t
  !     Distributions Subject to Linear Constraintsand the Evaluation
  !     of Constraint Probabilities''
  !   Computing Science and Statistics:
  !   Proceedings of the 23rd Symposium on the Interface
  !   Ed. E. Keramidas and S. Kaufman, pages 571-578
  !   Fairfax Station, VA: InterfaceFoundation of North America.

  real(r8) function rtnorm(mu, var, a, left)
    implicit none
    real(r8), intent(in) :: mu, var, a
    logical,  intent(in) :: left
    real(r8) :: sd, c, u, z

    if(var <= 0._r8) then
      call rexit('*** ERROR: var should be positive (rtnorm) ***')
    end if
    sd = sqrt(var)

    c = (a - mu)/sd
    if(.not.left) c = -c

    if(c <= .45_r8) then   ! normal rejection sampling
      do
        z = rnorm()
        if(z > c) exit
      end do
    else                   ! exponential rejection sampling
      do
        z = rexpon(c)
        u = runif()
        if(u < exp(-.5_r8*(z**2))) exit
      end do
      z = z + c
    end if

    if(left) then
       rtnorm = mu + z*sd
    else
       rtnorm = mu - z*sd
    end if

  end function rtnorm


  !-----------------------------------------------------------------------------
  ! generates random number from gamma distribution
  ! with shape parameter a and scale parameter b
  ! such that mean=a*b and var=a*b^2
  !
  ! source:
  !   George Marsaglia and Wai Wan Tsang
  !   ``A Simple Method for Generating Gamma Variables''
  !   ACM Transactions on Mathematical Software
  !   Vol. 26, No. 3, September 2000, pages 363-372

  real(r8) function rgamma(a, b)
    implicit none
    real(r8), intent(in) :: a, b
    real(r8) :: a1, c, d, u, v, x

    if(a <= 0._r8) call rexit('*** ERROR: a should be positive (rgamma) ***')
    if(b <= 0._r8) call rexit('*** ERROR: b should be positive (rgamma) ***')

    a1 = a
    if(a < 1._r8) a1 = a + 1._r8
    d = a1 - third
    c = 1._r8/sqrt(9._r8*d)

    do
      do
        x = rnorm()
        v = (1._r8 + c*x)
        if(v > 0._r8) exit
      end do
      v = v**3
      u = runif()
      if(u < 1._r8 - 0.0331_r8*(x**4)) exit
      if(log(u) < .5_r8*(x**2) + d*(1._r8-v+log(v))) exit
    end do
    rgamma = d*v*b

    ! case where a < 1 (see note p.371)
    if(a < 1._r8) then
      do
        u = runif()
        if(u > 0._r8) then   ! cycle if u = 0
          rgamma = rgamma * u**(1._r8/a)
          exit
        end if
      end do
    end if

  end function rgamma


  !-----------------------------------------------------------------------------
  ! generates random number from inverse-gamma distribution
  ! with shape parameter a and rate parameter b

  real(r8) function rinvgamma(a, b)
    implicit none
    real(r8), intent(in) :: a, b

    if(a <= 0._r8) call rexit('*** ERROR: a should be positive (rinvgamma) ***')
    if(b <= 0._r8) call rexit('*** ERROR: b should be positive (rinvgamma) ***')

    rinvgamma = 1._r8/rgamma(a, 1._r8/b)

  end function rinvgamma


  !-----------------------------------------------------------------------------
  ! generates vector from Dirichlet distribution
  ! with concentration parameters alpha

  function rdirich(alpha)
    implicit none
    real(r8), intent(in) :: alpha(:)
    real(r8)             :: rdirich(size(alpha))
    integer              :: i

    if(any(alpha <= 0._r8)) then
      call rexit('*** ERROR: alpha should be strictly positive (rdirich) ***')
    end if

    do i = 1, size(alpha)
      rdirich(i) = rgamma(alpha(i), 1._r8)
    end do
    rdirich = rdirich / sum(rdirich)

  end function rdirich


  !-----------------------------------------------------------------------------
  ! generates random matrix from Wishart distribution
  ! with df degrees of freedom and scale matrix S:
  !
  !   X ~ Wishart(df, S)
  !   p(X) \propto |X|^{(df-p-1)/2} exp{ -tr(X * S^-1)/2 }
  !
  ! reference:
  !   A. K. Gupta & D. K. Nagar (2000)
  !   ``Matrix Variate Distributions''
  !   Chapman & Hall/CRC
  !   Monographs and Surveys in Pure and Applied Mathematics, No. 104
  !   Theorems 3.3.1 and 3.3.4, pp.90-91

  function rwishart(df, S)
    implicit none
    real(r8), intent(in) :: df, S(:,:)
    real(r8), dimension(size(S,1),size(S,2)) :: rwishart, B, BB, L
    integer :: i, j, p

    p = size(S,1)
    if(df < dble(p)) then
      call rexit('*** ERROR: degrees of freedom should be > p-1 (rwishart) ***')
    end if

    B = 0._r8
    do i = 1, p
      do j = 1, i
        if (i == j) then
          B(i,i) = sqrt(rgamma(.5_r8*(df-dble(i)+1._r8), 2._r8))
        else
          B(i,j) = rnorm()
        end if
      end do
    end do

    BB = matmul(B, transpose(B))
    L = chol(S)
    rwishart = matmul(L, matmul(BB, transpose(L)))

  end function rwishart


  !-----------------------------------------------------------------------------
  ! generates random matrix from inverse-Wishart distribution
  !
  !   X ~ inv-Wishart(df, W)
  !   p(X) \propto |X|^{-(df+p+1)/2} exp{ -tr(W * X^-1)/2 }

  function rinvwishart(df, W)
    implicit none
    real(r8), intent(in) :: df, W(:,:)
    real(r8) :: rinvwishart(size(W,1), size(W,1))

    rinvwishart = matinv(rwishart(df, matinv(W)))

  end function rinvwishart


  !-----------------------------------------------------------------------------
  ! generates random number from Poisson distribution

  integer function rpoisson(b)
    implicit none
    real(r8), intent(in) :: b
    real(r8)             :: em, t
    real(r8), save       :: g, oldb=-1._r8

    if(b <= 0._r8) then
      call rexit('*** ERROR: b should be > 0 (rpoisson) ***')
    end if

    ! Knuth's algorithm
    if(abs(b - oldb) > 0._r8) then
      oldb = b
      g = exp(-b)
    end if
    em = -1._r8
    t = 1._r8
    do
      em = em + 1._r8
      t = t * runif()
      if(t <= g) exit
    end do

    rpoisson = int(em)

  end function rpoisson


end module probability

#' @export
#' @importFrom ggplot2 ggplot aes_string geom_density theme labs element_blank
#' @importFrom ggplot2 ggplot_gtable ggplot_build
#' @importFrom gridExtra grid.arrange arrangeGrob

plot.simul.R.prior <- function(x, ...)
{

  if (!is.list(x)) x <- as.list(x)
  nsim <- length(x)

  # function making individual plot
  plot.dens <- function(Rsim, FUN, xlab) {

    val <- lapply(Rsim, function(z) apply(z, 3, FUN))
    val <- do.call(cbind, val)
    dat <- data.frame(val = c(val),
                      lab = rep(colnames(val), each = nrow(val)))

    # make plot
    p <- ggplot(dat, aes_string(x = 'val', color = 'lab')) +
           geom_density(kernel = 'gaussian') +
           labs(x = xlab, y = 'prior density') +
           theme(legend.position  = 'bottom',
                 legend.direction = 'horizontal',
                 legend.title     = element_blank())
    return(p)

  }

  # make plots
  plots <- list()
  plots$maxcor <- plot.dens(x, FUN = get.maxcor, xlab = 'max(|R|)')
  plots$mineig <- plot.dens(x, FUN = get.mineig, xlab = 'min(eigen(R))')

  # get common legend
  legend <- ggplot_gtable(ggplot_build(plots[[1]]))
  legid  <- which(sapply(legend$grobs, function(p) p$name) == 'guide-box')
  legend <- legend$grobs[[legid]]

  # make plot
  grid.arrange(arrangeGrob(plots[[1]] + theme(legend.position = 'none'),
                           plots[[2]] + theme(legend.position = 'none'),
                           nrow = 1),
               legend, nrow = 2, heights = c(10, 1),
               top = 'correlation matrix of the latent factors')

}

#'
#' Perform sign switchting on posterior MCMC sample
#'
#' This function performs a sign switch on the MCMC draws to restore the
#' consistency of the signs of the factors loadings and of the correlations of
#' the latent factors \emph{a posteriori}.
#'
#' @param mcmc
#'        Object of class '\code{befa}'.
#' @param benchmark
#'        Vector of integers of length equal to the maximum number of latent
#'        factors. Each element indicates which factor loading is used as a
#'        benchmark for the sign switch. If \code{NULL}, the factor loadings
#'        with the highest posterior probabilities of being different from zero
#'        in each column of the factor loading matrix are used as benchmarks.
#' @param benchmark.threshold
#'        Minimum posterior probability for a factor loading to be considered
#'        as a benchmark.
#'
#' @details The signs of the factor loadings, as well as of the corresponding
#' correlations of the latent factors, are switched for each MCMC iteration such
#' that the factor loadings defined as \code{benchmark}s are positive. The sign
#' switch can only be performed if \code{\link{post.column.switch}} has been run
#' before. See section 4.3 (p.42) of CFSHP for more details.
#'
#' If a latent factor has no benchmarks, or if its benchmark is equal to zero at
#' some MCMC iteration, then no sign switch is performed on the corresponding
#' loadings and correlations for this particular factor or MCMC iteration.
#'
#' Note that in complicated models where the sampler visits several models with
#' different numbers of latent factors, it may not be relevant to use the
#' default value of \code{benchmark}, as the posterior probabilities that the
#' factor loadings are different from zero would be computed across models.
#' Instead, the user might consider finding the highest posterior probability
#' model first, and use its top elements in each column of the factor loading
#' matrix as benchmarks to perform the sign switch.
#'
#' @return This function returns the same '\code{befa}' object, where the signs
#' of the factor loadings and of the factor correlations have been switched
#' appropriately to restore the identification of the factor model with respect
#' to sign switching.
#'
#' @author Rémi Piatek \email{remi.piatek@@gmail.com}
#'
#' @references
#' G. Conti, S. Frühwirth-Schnatter, J.J. Heckman,
#' R. Piatek (2014): ``Bayesian Exploratory Factor Analysis'',
#' \emph{Journal of Econometrics}, 183(1), pages 31-57,
#' \url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.
#'
#' @seealso \code{\link{post.column.switch}} for column switching of the factor
#' loading matrix and of the correlation matrix of the latent factors to restore
#' identification \emph{a posteriori}.
#'
#' @examples
#' set.seed(6)
#' Y <- simul.dedic.facmod(N = 100, dedic = rep(1:3, each = 5))
#' mcmc <- befa(Y, Kmax = 5, iter = 1000)
#' mcmc <- post.column.switch(mcmc)
#'
#' # factor loadings corresponding to variables 1, 6, 11, 12 and 13 are
#' # used as benchmarks:
#' mcmc1 <- post.sign.switch(mcmc, benchmark = c(1, 6, 11, 12, 13))
#'
#' # factor loadings with the highest posterior probability of being different
#' # from zero in each column are used as benchmark:
#' mcmc2 <- post.sign.switch(mcmc)
#'
#' @export post.sign.switch
#' @import checkmate

post.sign.switch <- function(mcmc, benchmark = NULL, benchmark.threshold = 0.5)
{

  assertClass(mcmc, classes = "befa")
  if (attr(mcmc, "post.sign.switch")) {
    warning("sign switching already performed, nothing done.")
    return(mcmc)
  }
  if (!attr(mcmc, "post.column.switch")) {
    stop("post.column.switch should be run first.")
  }
  assertNumber(benchmark.threshold, lower = 0, upper = 1)

  Kmax   <- attr(mcmc, "Kmax")
  nvar   <- ncol(mcmc$dedic)
  iter   <- nrow(mcmc$dedic)
  R.npar <- Kmax * (Kmax - 1)/2

  # factor loadings used as benchmarks
  if (is.null(benchmark)) {
    alpha.post.prob <- matrix(0, Kmax, nvar)
    for (k in 1:Kmax) {
      alpha.post.prob[k, ] <- colMeans(mcmc$dedic == k)
    }
    benchmark <- max.col(alpha.post.prob, ties.method = "first")
    for (k in 1:Kmax) {
      if (alpha.post.prob[k, benchmark[k]] < benchmark.threshold) {
        benchmark[k] <- NA  # not a benchmark if post prob < threshold
      }
    }
  } else {
    assertIntegerish(benchmark, lower = 0, upper = nvar, any.missing = FALSE,
                     len = Kmax)
  }

  # index matrix used to reconstruct matrix from lower triangular elements
  R.mat <- diag(Kmax) * (R.npar + 1)
  R.mat[lower.tri(R.mat)] <- 1:R.npar
  R.mat[upper.tri(R.mat)] <- t(R.mat)[upper.tri(R.mat)]

  for (i in 1:iter) {
    # switch signs of factor loadings
    dedic <- mcmc$dedic[i, ]
    switch <- sign(mcmc$alpha[i, benchmark])
    switch[is.na(switch)] <- 1  # no sign switch for factors with no benchmark
    switch.meas <- rep(0, nvar)
    for (j in 1:nvar) {
      if (dedic[j] == 0) next
      switch.meas[j] <- switch[dedic[j]]
    }
    mcmc$alpha[i, ] <- mcmc$alpha[i, ] * switch.meas

    # switch signs of rows and columns of correlation matrix
    R <- c(mcmc$R[i, ], 1)
    R <- matrix(R[R.mat], nrow = Kmax)
    R <- diag(switch) %*% R %*% diag(switch)
    mcmc$R[i, ] <- R[lower.tri(R)]
  }

  attr(mcmc, "post.sign.switch") <- TRUE
  return(mcmc)

}

#' @export

print.summary.befa <- function(x, ...)
{

  cat('BEFA - Bayesian Exploratory Factor Analysis',
      'Summary of posterior results\n', sep = '\n')

  args <- list(...)
  digits <- args$digits
  if (is.null(digits)) digits <- 3

  cat('Maximum number of factors (Kmax) =', attr(x, 'Kmax'), '\n')
  cat('Identification restriction (Nid) =', attr(x, 'Nid'), '\n\n')

  cat('MCMC iterations =', attr(x, 'iter'), '\n')
  cat('burn-in period  =', attr(x, 'burnin'), '\n\n')

  cat('Metropolis-Hastings acceptance rate =', round(x$MHacc, digits = 3))
  cat('\n\n')

  what     <- attr(x, 'what')
  hpd.prob <- attr(x, 'hpd.prob')

  if (what == 'hppm') {

    cat('-----------------------------------------------------------------',
        'Indicator matrix\n',
        'Dedicated structures visited by the sampler, with corresponding ',
        'posterior probabilities and number of factors (K)\n', sep = '\n')
    cat(paste0('prob(', colnames(x$hppm$dedic), ') = ',
               sprintf('%.3f', x$hppm$prob),
               ',  K = ', x$hppm$nfac),
        sep = '\n')
    cat('\n')
    print(x$hppm$dedic, print.gap = 3)
    cat('\n')

  } else {

    cat('Posterior frequency of number of latent factors:\n')
    freq <- sprintf('%3.2f%%', 100*x$nfac)
    nf   <- sprintf('%2i', as.integer(names(x$nfac)))
    cat(paste('  K =', nf, '   ', freq), sep='\n')
    cat('\n')

  }

  # print MCMC summary results for model parameters
  cat('-----------------------------------------------------------------',
      'Model parameters\n',
      sep = '\n')

  print.mcmc <- function(z, title) {
    res <- round(z, digits = digits)
    collab <- c('mean', 'sd',
                sprintf(paste0('[%', digits, '.0f%%'), 100*hpd.prob),
                sprintf(paste0('%', -(digits+1), 's]'), 'hpd'))
    if (ncol(z) == 5) collab <- c('dedic', collab)
    if (ncol(z) == 6) collab <- c('dedic', 'prob', collab)
    colnames(res) <- collab
    if (!missing(title)) cat(title, ':\n\n', sep='')
    print(as.matrix(res), print.gap = 3, na.print = ' ')
    cat('\n')
  }

  print.mcmc.all <- function(alpha, sigma, R, beta) {
    print.mcmc(alpha, title = 'Factor loadings')
    print.mcmc(sigma, title = 'Idiosyncratic variances')
    print.mcmc(R,     title = 'Factor correlation matrix')
    if (!is.null(beta)) print.mcmc(beta, title = 'Regression coefficients')
  }

  if (what == 'hppm') {

    for (i in seq_along(x$hppm$prob)) {
      cat('-----------------------------------------------------------------\n')
      cat('Highest posterior probability model ', i, '\n\n',
          'prob = ', round(x$hppm$prob[i], digits = 3), '\n',
          '   K = ', x$hppm$nfac[i], '\n\n',
          sep = '')
      print.mcmc.all(x$alpha[[i]], x$sigma[[i]], x$R[[i]], x$beta[[i]])
    }

  } else {

    print.mcmc.all(x$alpha, x$sigma, x$R, x$beta)

  }

}

#' @importFrom stats complete.cases cor terms

extract.data <- function(model, data)
{

  errmsg <- warnmsg <- NULL

  # get equation terms
  neq <- length(model)
  all.terms <- lapply(model, terms)

  # which equations have a manifest variable?
  resp <- sapply(all.terms, attr, 'response')
  resp <- as.logical(resp)

  # manifest variables
  allY <- sapply(lapply(model, all.vars), '[', 1)
  Ylab <- unique(allY[resp])
  nY   <- length(Ylab)
  allY[!resp] <- NA

  # intercept terms
  const <- sapply(all.terms, attr, 'intercept')
  const <- as.logical(const)

  # covariates
  allX <- lapply(all.terms, attr, 'term.labels')
  Xlab <- unique(unlist(allX))
  Xlab <- c('(Intercept)', Xlab)
  nX   <- length(Xlab)

  YXloc <- matrix(FALSE, nY, nX, dimnames = list(Ylab, Xlab))
  for (i in 1:neq) {
    if (is.na(allY[i])) {
      YXloc[, unlist(allX[i])] <- TRUE
      YXloc[, '(Intercept)'] <- const[i]
    } else {
      YXloc[allY[i], unlist(allX[i])] <- TRUE
      YXloc[allY[i], '(Intercept)'] <- const[i]
    }
  }

  # check that all variables are in data frame
  YXlab <- unique(unlist(c(Ylab, Xlab)))
  YXlab <- YXlab[YXlab != '(Intercept)']   # no error message for intercept term
  indat <- YXlab %in% names(data)
  if (any(!indat)) {
    errmsg <- paste('following variables not in data:\n   ',
                    paste(YXlab[!indat], collapse = ', '))
    return(list(errmsg))
  }
  YXdata <- data[YXlab]
  nobs   <- nrow(YXdata)
  YXdata[["(Intercept)"]] <- 1    # add vector of ones for intercept terms

  # type of manifest variables
  Ytype <- sapply(YXdata[Ylab], typeof)

  # discard missing values in covariates
  nomiss <- complete.cases(YXdata[Xlab])
  YXdata <- YXdata[nomiss, ]
  if (!all(nomiss)) {
    warnmsg <- paste(sum(!nomiss), 'observations discarded because of NAs',
                     'in at least one covariate')
    nobs <- nobs - sum(!nomiss)
  }

  # check for multicollinearity in covariates specified in each equation
  Xcor <- cor(data[Xlab[Xlab != '(Intercept)']])  # exclude intercept
  for (i in 1:nY) {
    Xcori <- Xcor[YXloc[i, -1], YXloc[i, -1]]
    Xcori <- Xcori[lower.tri(Xcori)]
    if (any(abs(Xcori - 1) < 1e-12)) {
      errmsg <- c(errmsg, paste('perfect multicollinearity between covariates',
                                 'of manifest variable', Ylab[i]))
    } else if (any(abs(Xcori) > 0.95)) {
      warnmsg <- c(warnmsg, paste('possible multicollinearity problem between',
                                  'covariates of manifest variable', Ylab[i]))
    }
  }

  # return
  return(list(Ytype   = Ytype,
              Yobs    = as.matrix(YXdata[Ylab]),
              Xobs    = as.matrix(YXdata[Xlab]),
              YXloc   = YXloc,
              errmsg  = errmsg,
              warnmsg = warnmsg))

}

#'
#' Plot object of class 'befa'
#'
#' This function makes different plots that are useful to assess the posterior
#' results: a trace plot of the number of latent factors (also showing
#' Metropolis-Hastings acceptance across MCMC replications), a histogram
#' of the posterior probabilities of the number of factors, heatmaps for the
#' inficator probabilities, the factor loading matrix, and the correlation
#' matrix of the latent factors.
#'
#' @inheritParams summary.befa
#' @param x Object of class 'befa'.
#'
#' @details This function makes graphs based on the summary results returned by
#' \code{\link{summary.befa}}. It therefore accepts the same optional arguments
#' as this function.
#'
#' @author Rémi Piatek \email{remi.piatek@@gmail.com}
#'
#' @seealso \code{\link{summary.befa}} to summarize posterior results.
#'
#' @examples
#' set.seed(6)
#'
#' # generate fake data with 15 manifest variables and 3 factors
#' Y <- simul.dedic.facmod(N = 100, dedic = rep(1:3, each = 5))
#'
#' # run MCMC sampler and post process output
#' # notice: 1000 MCMC iterations for illustration purposes only,
#' #  increase this number to obtain reliable posterior results!
#' mcmc <- befa(Y, Kmax = 5, iter = 1000)
#' mcmc <- post.column.switch(mcmc)
#' mcmc <- post.sign.switch(mcmc)
#'
#' # plot results for highest posterior probability model
#' plot(mcmc, what = 'hppm')
#'
#' @export
#' @importFrom ggplot2 ggplot aes_string theme labs element_text
#' @importFrom ggplot2 geom_line geom_tile geom_text geom_rug geom_bar
#' @importFrom ggplot2 scale_fill_gradient2 guide_colorbar guides unit

plot.befa <- function(x, ...)
{

  args <- list(...)
  show.val <- ifelse(is.null(args$show.val), TRUE, args$show.val)
  what <- ifelse(is.null(args$what), 'maxp', args$what)
  assertFlag(show.val)
  if (!what %in% c('maxp', 'hppm'))
    stop('plot.befa() only implemented for what = "maxp" and what = "hppm"')

  Kmax <- attr(x, 'Kmax')

  ##############################################################################
  ### trace of number of factors

  dat <- data.frame(nfac  = factor(x$nfac, levels = 0:Kmax),
                    iter  = as.numeric(names(x$nfac)),
                    MHacc = as.numeric(x$MHacc))

  p.nfac <- ggplot(dat, aes_string(x = 'iter', y = 'nfac')) +
              geom_line(colour = 'steelblue')

  p.nfac <- p.nfac + labs(title = paste0('trace plot of number of factors\n',
                             '(accepted Metropolis-Hastings draws at bottom)'),
                         x = 'MCMC iterations',
                         y = 'number of factors')

  # add Metropolis-Hastings acceptance
  p.nfac <- p.nfac + geom_rug(aes_string(y = 'MHacc'), sides = 'b',
                              colour = 'darkcyan')


  # posterior probabilities of number of factors
  nft <- table(factor(x$nfac, levels = 0:Kmax))
  dat <- data.frame(nfac = as.factor(0:Kmax),
                    freq = as.numeric(nft / length(x$nfac)))
  p.hnfac <- ggplot(dat, aes_string(x = 'nfac')) +
               geom_bar(aes_string(weight = 'freq'), fill = 'steelblue') +
               labs(title = 'posterior probabilities of number of factors',
                    x = 'number of factors',
                    y = 'frequencies')


  ##############################################################################
  # summarize and plot

  x <- summary(x, ...)
  if (what == 'hppm') {
    alpha <- x$alpha$m1
    dedic <- x$alpha$m1$dedic
    R     <- x$R$m1
  } else {
    alpha <- x$alpha
    dedic <- x$alpha$dedic
    R     <- x$R
  }
  nvar  <- length(dedic)

  ### matrix of indicator probabilities

  if (what != 'hppm') {     # skip for HPP model

    pind <- matrix(NA, nvar, Kmax)
    rownames(pind) <- sapply(strsplit(rownames(alpha), ':'), '[', 2)
    colnames(pind) <- paste0('f', 1:Kmax)
    for (i in 1:nvar)
      pind[i, dedic[i]] <- alpha$prob[i]

    # which factors are loaded by at least one measurement?
    acti <- apply(!is.na(pind), 2, any)

    # heatmap for active factors only
    p.indic <- make.heatmap(pind[, acti],
                            title = 'indicator probabilities of being nonzero',
                            xlab = 'latent factors (active factors only)',
                            ylab = 'manifest variables',
                            show.val)

  }

  ### matrix of factor loadings

  # construct matrix from factor loadings and indicators
  # (remove the 'alpha:' part from variable names to simplify plot)
  alpha.post <- matrix(NA, nvar, Kmax)
  rownames(alpha.post) <- sapply(strsplit(rownames(alpha), ':'), '[', 2)
  colnames(alpha.post) <- paste0('f', 1:Kmax)
  for (i in 1:nvar)
    alpha.post[i, dedic[i]] <- alpha$mean[i]

  # which factors are loaded by at least one measurement?
  acti <- !apply(is.na(alpha.post), 2, all)

  # heatmap for active factors only
  p.alpha <- make.heatmap(alpha.post[, acti],
                          title = 'factor loading matrix',
                          xlab = 'latent factors (active factors only)',
                          ylab = 'manifest variables',
                          show.val)

  ### correlation matrix of the factors

  # construct matrix from lower diagonal elements
  Rmat <- .5 * diag(Kmax)
  Rmat[lower.tri(Rmat)] <- R$mean
  Rmat <- Rmat + t(Rmat)
  rownames(Rmat) <- colnames(Rmat) <- paste0('f', 1:Kmax)

  # heatmap for active factors only
  p.Rmat <- make.heatmap(Rmat[acti, acti],
                         title = 'correlation matrix of the factors',
                         xlab = 'latent factors (active factors only)',
                         ylab = '',
                         show.val)

  ##############################################################################

  print(p.nfac)
  invisible(readline(prompt = "Press <Enter> to show next graph..."))
  print(p.hnfac)
  invisible(readline(prompt = "Press <Enter> to show next graph..."))
  if (what != 'hppm') {
    print(p.indic)
    invisible(readline(prompt = "Press <Enter> to show next graph..."))
  }
  print(p.alpha)
  invisible(readline(prompt = "Press <Enter> to show next graph..."))
  print(p.Rmat)

}


################################################################################

make.heatmap <- function(x, title, xlab, ylab, show.val) {

  # prepare data
  xcol <- colnames(x)
  xrow <- rownames(x)
  dat <- data.frame(xvar = factor(rep(xcol, each = nrow(x)), levels = xcol),
                    yvar = factor(rep(xrow, ncol(x)), levels = rev(xrow)),
                    val  = c(round(x, digits = 2)))

  # make heatmap
  p <- ggplot(dat, aes_string(x = 'xvar', y = 'yvar')) +
    geom_tile(aes_string(fill = 'val')) +
    scale_fill_gradient2(low  = 'steelblue',
                         mid  = 'white',
                         high = 'red',
                         na.value = NA)

  # add title and labels
  p <- p + labs(title = title, x = xlab, y = ylab, fill = 'posterior\nmean')

  # adjust font sizes
  # p <- p + theme(axis.title  = element_text(size=12),
  #                axis.text   = element_text(size=12),
  #                plot.title  = element_text(size=16),
  #                legend.text = element_text(size=10))

  # legend
  p <- p + guides(fill = guide_colorbar(barheight = unit(6, 'cm')))

  # add values to heatmap?
  if (show.val)
    p <- p + geom_text(aes_string(label = 'val'),
                       colour = 'white',
                       na.rm = TRUE)

  return(p)

}

module global

  use, intrinsic :: iso_fortran_env
  implicit none

  ! Fortran kinds for single and double precisions
  integer, parameter :: i4 = INT32
  integer, parameter :: i8 = INT64
  integer, parameter :: r4 = REAL32
  integer, parameter :: r8 = REAL64

end module global

#' @export

print.befa <- function(x, ...)
{
  if (class(x) != 'befa')
    stop('object passed to print.befa should be of class befa')

  cat('BEFA output - Bayesian Exploratory Factor Analysis\n\n')
  cat('call:\n  ')
  print(attr(x, 'call'))
  cat('\n')
  cat('posterior column switch:', attr(x, "post.column.switch"), '\n')
  cat('posterior sign switch:  ', attr(x, "post.sign.switch"), '\n\n')
}

#' @export

print.simul.R.prior <- function(x, ...)
{
  cat('sample from prior distribution of factor correlation matrix\n')
  cat('\ncall:\n  ')
  print(attr(x, 'call'))
  cat('\nKmax =', attr(x, 'Kmax'), '\n')
  cat('\nparameter values:\n  ')
  cat(names(x), sep = '\n  ')
  cat('\nnrep =', attr(x, 'nrep'), '\n\n')
  cat('use summary() and plot() functions to summarize and plot prior density',
      'of maximum correlation and/or of minimum eigenvalue', sep = '\n')
}

module loading_idioprec_class

  use global
  use probability, only : rnorm, rgamma
  implicit none

  private
  public :: loading_idioprec


  type :: loading_idioprec
    logical  :: cont            ! TRUE if continuous case, FALSE if categorical
    real(r8) :: alpha           ! factor loading
    real(r8) :: alpha_mu0       ! prior:
    real(r8) :: alpha_prec0     !   alpha ~ N(alpha_mu0, sig2/alpha_prec0)
    real(r8) :: var             ! idiosyncratic variance
    real(r8) :: prec            ! idiosyncratic precision
    real(r8) :: prec_a0         ! prior:
    real(r8) :: prec_b0         !   sig2  ~ IGamma(prec_a0, prec_b0)
    real(r8) :: prec_a_post     ! posterior shape for prec
    ! back up values
    real(r8) :: alpha_bak
    real(r8) :: var_bak
    real(r8) :: prec_bak
  contains
    procedure, public :: init    => init_loading_idioprec
    procedure, public :: update  => update_loading_idioprec
    procedure, public :: backup  => backup_loading_idioprec
    procedure, public :: restore => restore_loading_idioprec
  end type loading_idioprec


contains


  !-----------------------------------------------------------------------------

  subroutine init_loading_idioprec(this, nobs, cont, prior, start)
    implicit none
    class(loading_idioprec) :: this
    integer,  intent(in)    :: nobs
    logical,  intent(in)    :: cont
    real(r8), intent(in)    :: prior(3)
    real(r8), intent(in)    :: start(2)

    this%cont        = cont
    this%alpha_mu0   = 0._r8
    this%alpha_prec0 = prior(1)
    this%prec_a0     = prior(2)
    this%prec_b0     = prior(3)

    this%prec_a_post = this%prec_a0 + .5_r8*nobs

    this%alpha = start(1)
    this%var   = start(2)
    this%prec  = 1._r8/this%var

    this%alpha_bak = this%alpha
    this%var_bak   = this%var
    this%prec_bak  = this%prec

  end subroutine init_loading_idioprec


  !-----------------------------------------------------------------------------

  subroutine update_loading_idioprec(this, Yaux, dedic, fac)
    implicit none
    class(loading_idioprec)       :: this
    real(r8), intent(in)          :: Yaux(:)
    integer,  intent(in)          :: dedic
    real(r8), intent(in), target  :: fac(:,:)
    real(r8),             pointer :: fp(:) => null()
    real(r8)                      :: aN, AAN, prec_b_post

    !----- 'null model'
    if(dedic == 0) then

      if(.not.this%cont) return  ! nothing to do in categorical case

      ! sample idiosyncratic precision in continuous case
      prec_b_post = this%prec_b0 + .5_r8*sum(Yaux**2)
      this%prec = rgamma(this%prec_a_post, 1._r8/prec_b_post)
      this%var = 1._r8/this%prec

    !----- general case
    else

      fp => fac(:, dedic)

      aN  = dot_product(Yaux, fp)
      AAN = 1._r8/(sum(fp**2) + this%alpha_prec0)

      if(this%cont) then    ! sample precision in continuous case
        prec_b_post = this%prec_b0 + .5_r8*(sum(Yaux**2) - AAN*(aN**2))
        this%prec = rgamma(this%prec_a_post, 1._r8/prec_b_post)
        this%var = 1._r8/this%prec
      end if

      this%alpha = rnorm(AAn*aN, AAN*this%var)

    end if

  end subroutine update_loading_idioprec


  !-----------------------------------------------------------------------------

  subroutine backup_loading_idioprec(this)
    implicit none
    class(loading_idioprec) :: this

    this%alpha_bak = this%alpha
    this%var_bak   = this%var
    this%prec_bak  = this%prec

  end subroutine backup_loading_idioprec


  !-----------------------------------------------------------------------------

  subroutine restore_loading_idioprec(this)
    implicit none
    class(loading_idioprec) :: this

    this%alpha = this%alpha_bak
    this%var   = this%var_bak
    this%prec  = this%prec_bak

  end subroutine restore_loading_idioprec


end module loading_idioprec_class

#' @export
#' @import checkmate
#' @importFrom stats median sd quantile

summary.simul.R.prior <- function(object, ...)
{

  args <- list(...)
  if (is.null(args$probs)) {
    probs <- c(.05, .1, .25, .75, .9, .95)
  } else {
    probs <- args$probs
  }
  assertNumeric(probs, lower = 0, upper = 1, any.missing = FALSE, min.len = 1)

  sum.prior <- function(x, FUN) {
    val <- lapply(x, function(z) apply(z, 3, FUN))
    stat <- lapply(val, function(z) return(c(mean   = mean(z),
                                            median = median(z),
                                            sd     = sd(z),
                                            min    = min(z),
                                            max    = max(z))))
    quant <- lapply(val, quantile, probs)
    res <- list(stat  = do.call(rbind, stat),
                quant = do.call(rbind, quant))
    res <- lapply(res, round, digits = 3)
    return(res)
  }

  out <- list(maxcor = sum.prior(object, get.maxcor),
              mineig = sum.prior(object, get.mineig))
  class(out) <- 'summary.simul.R.prior'
  return(out)

}

subroutine befa &
    (nmeas, nobs, kmax, nid, Yobs, Ycat, Ymiss, nX, Xobs, Xloc, iter, burnin, &
     search_delay, Rmat_delay, step_rnd, step_lambda, seed, prior_loadprec, &
     prior_beta, prior_dedic, prior_facdist, start_loadprec, start_beta, &
     start_dedic, start_factor, start_facdist, verbose, npar, MCMCdraws, &
     MCMCdedic, MHacc)

  use global
  use probability, only : set_seed, rpoisson
  use measurement_class
  use loading_idioprec_class
  use covariates_class
  use indicators_dedic_class
  use factor_normal_block_class
  use covmat_block_invwishart_class
  use mda_class
  use mcmc_progress_class
  implicit none

  !----- input
  integer,  intent(in) :: nmeas
  integer,  intent(in) :: nobs
  integer,  intent(in) :: kmax
  integer,  intent(in) :: nid
  real(r8), intent(in) :: Yobs(nobs,nmeas)
  integer,  intent(in) :: Ycat(nmeas)
  logical,  intent(in) :: Ymiss(nobs,nmeas)
  integer,  intent(in) :: nX
  real(r8), intent(in) :: Xobs(nobs,nX)
  logical,  intent(in) :: Xloc(nmeas,nX)
  integer,  intent(in) :: iter
  integer,  intent(in) :: burnin
  integer,  intent(in) :: search_delay
  integer,  intent(in) :: Rmat_delay
  logical,  intent(in) :: step_rnd
  real(r8), intent(in) :: step_lambda
  integer,  intent(in) :: seed
  real(r8), intent(in) :: prior_loadprec(nmeas,3)
  real(r8), intent(in) :: prior_beta(nmeas)
  real(r8), intent(in) :: prior_dedic(0:3*nmeas+kmax+2)
  real(r8), intent(in) :: prior_facdist(0:kmax+1)
  real(r8), intent(in) :: start_loadprec(nmeas,2)
  real(r8), intent(in) :: start_beta(nmeas,nX)
  integer,  intent(in) :: start_dedic(nmeas)
  real(r8), intent(in) :: start_factor(nobs,kmax)
  real(r8), intent(in) :: start_facdist(kmax,kmax)
  logical,  intent(in) :: verbose

  !----- output
  integer,  intent(in)  :: npar
  real(r8), intent(out) :: MCMCdraws(burnin+1:iter,npar)
  integer,  intent(out) :: MCMCdedic(burnin+1:iter,nmeas)
  logical,  intent(out) :: MHacc(burnin+1:iter)

  !----- model ingredients
  type(measurement)                           :: Ylat(nmeas)
  type(covariates)                            :: Xcov(nmeas)
  type(loading_idioprec)                      :: alpha_prec(nmeas)
  type(indic_dedic)                           :: dedic
  type(factor_normal_block)                   :: factors
  class(covmat_block_invwishart), allocatable :: facdist
  type(workpar)                               :: mda

  logical :: norestr
  integer :: i, ii, j, k, step

  type(mcmc_progress) :: mcmc_prog
  call mcmc_prog%init(burnin, iter, verbose)


  !=============================================================================
  ! initialization

  call set_seed(seed)

  do j = 1, nmeas

    ! latent variables underlying measurements
    select case (Ycat(j))
      case(0); allocate(measurement_cont :: Ylat(j)%p)
      case(2); allocate(measurement_bin  :: Ylat(j)%p)
    end select
    call Ylat(j)%p%init(nobs, Yobs(:,j), Ymiss(:,j))

    ! slope parameters for covariates
    call Xcov(j)%init(nobs, nX, Xobs, prior_beta(j), start_beta(j,:), Xloc(j,:))

    ! factor loadings and idiosyncratic precisions
    call alpha_prec(j)%init(nobs, Ycat(j)==0, prior_loadprec(j,:), &
                            start_loadprec(j,:))

  end do

  ! indicators
  call dedic%init(nobs, nmeas, kmax, Ycat, prior_dedic, start_dedic)
  where(dedic%group == 0) alpha_prec%alpha = 0._r8

  ! latent factors
  call factors%init(nobs, nmeas, kmax, start_factor)
  select case (int(prior_facdist(0)))
    case(0); allocate(covmat_block_invwishart :: facdist)
    case(1); allocate(covmat_block_HuangWand  :: facdist)
  end select
  call facdist%init(nobs, kmax, prior_facdist(1:), start_facdist)

  ! working parameters for MDA
  call mda%init(kmax, nmeas, nobs)

  norestr = nid==1
  step = int(step_lambda)
  MHacc = .false.


  !=============================================================================
  ! MCMC sampling

  do i = 1, iter

    if(i <= search_delay) then

      call gibbs_sweep_forward(search=.false., sample_Rmat = i>Rmat_delay)
      if(i == search_delay) call backup_draws()  ! backup before starting search

    else if(norestr) then

      call gibbs_sweep_forward(search=.true., sample_Rmat = i>Rmat_delay)

    else

      ! number of intermediate steps in expanded model
      if(step_rnd) step = 1 + rpoisson(step_lambda)

      ! intermediate steps forward
      do ii = 1, step
        call gibbs_sweep_forward(search=.true., sample_Rmat = i>Rmat_delay)
      end do

      ! intermediate steps backward
      do ii = 1, step
        call gibbs_sweep_backward(sample_Rmat = i>Rmat_delay)
      end do

      ! M-H step: check identification restrictions
      if(any(dedic%ngroup<nid .and. dedic%ngroup>0)) then
        ! model is not identified, restore previous draws
        call restore_draws()
      else
        ! model is identified, back up current draws for next iteration
        call backup_draws()
        if(i > burnin) MHacc(i) = .true.
      end if

    end if

    ! save current draws after burn-in
    if(i > burnin) then
      MCMCdedic(i,:) = dedic%group
      MCMCdraws(i,:) = [ alpha_prec%alpha, &
                         alpha_prec%var, &
                         facdist%get(), &
                         get_all_covariates(Xcov) ]
    end if

    ! show MCMC progress
    call mcmc_prog%show(i)

  end do


contains


  !=============================================================================

  subroutine gibbs_sweep_forward(search, sample_Rmat)
    implicit none
    logical, intent(in) :: search, sample_Rmat
    real(r8)            :: Yaux(nobs, nmeas)

    forall(j = 1:nmeas) Yaux(:,j) = Ylat(j)%p%Y - Xcov(j)%Xbeta

    ! sample indicators
    if(search) then
      call dedic%update(Yaux, factors%theta)
      where(dedic%group == 0) alpha_prec%alpha = 0._r8
    end if

    do j = 1, nmeas

      k = dedic%group(j)

      ! sample factor loading and idiosyncratic precision
      call alpha_prec(j)%update(Yaux(:,j), k, factors%theta)

      ! sample slope parameters for covariates
      Yaux(:,j) = Ylat(j)%p%Y
      if(k > 0) Yaux(:,j) = Yaux(:,j) - alpha_prec(j)%alpha*factors%theta(:,k)
      call Xcov(j)%update(Yaux(:,j), alpha_prec(j)%prec)

      ! sample latent variable underlying measurement
      if(Ycat(j)>0 .or. any(Ymiss(:,j))) then
        Yaux(:,j) = Xcov(j)%Xbeta
        if(k > 0) Yaux(:,j) = Yaux(:,j) + alpha_prec(j)%alpha*factors%theta(:,k)
        call Ylat(j)%p%update(Yaux(:,j), alpha_prec(j)%prec)
      end if

    end do

    if(sample_Rmat) then

      ! MDA: expand model
      call mda%expand(dedic%group, alpha_prec%alpha, facdist)

      ! sample active factors
      forall(j = 1:nmeas) Yaux(:,j) = Ylat(j)%p%Y - Xcov(j)%Xbeta
      call factors%update_acti &
             (Yaux, alpha_prec%alpha, dedic, alpha_prec%prec, facdist)

      ! sample factor covariance matrix
      call facdist%update(factors%theta, dedic)

      ! sample inactive factors
      call factors%update_inacti(facdist, dedic)

      ! MDA: transform back to identified model
      call mda%transfback(dedic%group, alpha_prec%alpha, facdist, factors%theta)

    else

      forall(j = 1:nmeas) Yaux(:,j) = Ylat(j)%p%Y - Xcov(j)%Xbeta
      call factors%update &
             (Yaux, alpha_prec%alpha, dedic%group, alpha_prec%prec, facdist)

    end if

  end subroutine gibbs_sweep_forward


  !=============================================================================

  subroutine gibbs_sweep_backward(sample_Rmat)
    implicit none
    logical, intent(in) :: sample_Rmat
    real(r8)            :: Yaux(nobs, nmeas)

    if(sample_Rmat) then

      ! MDA: expand model
      call mda%expand(dedic%group, alpha_prec%alpha, facdist)

      ! sample active factors
      forall(j = 1:nmeas) Yaux(:,j) = Ylat(j)%p%Y - Xcov(j)%Xbeta
      call factors%update_acti &
             (Yaux, alpha_prec%alpha, dedic, alpha_prec%prec, facdist)

      ! sample factor covariance matrix
      call facdist%update(factors%theta, dedic)

      ! sample inactive factors
      call factors%update_inacti(facdist, dedic)

      ! MDA: transform back to identified model
      call mda%transfback(dedic%group, alpha_prec%alpha, facdist, factors%theta)

    else

      forall(j = 1:nmeas) Yaux(:,j) = Ylat(j)%p%Y - Xcov(j)%Xbeta
      call factors%update &
             (Yaux, alpha_prec%alpha, dedic%group, alpha_prec%prec, facdist)

    end if

    do j = 1, nmeas

      k = dedic%group(j)

      ! sample latent variable underlying measurement
      if(Ycat(j)>0 .or. any(Ymiss(:,j))) then
        Yaux(:,j) = Xcov(j)%Xbeta
        if(k > 0) Yaux(:,j) = Yaux(:,j) + alpha_prec(j)%alpha*factors%theta(:,k)
        call Ylat(j)%p%update(Yaux(:,j), alpha_prec(j)%prec)
      end if

      ! sample slope parameters for covariates
      Yaux(:,j) = Ylat(j)%p%Y
      if(k > 0) Yaux(:,j) = Yaux(:,j) - alpha_prec(j)%alpha*factors%theta(:,k)
      call Xcov(j)%update(Yaux(:,j), alpha_prec(j)%prec)

    end do

    ! sample indicators
    forall(j = 1:nmeas) Yaux(:,j) = Ylat(j)%p%Y - Xcov(j)%Xbeta
    call dedic%update(Yaux, factors%theta)
    where(dedic%group == 0) alpha_prec%alpha = 0._r8

    ! sample factor loading and idiosyncratic precision
    do j = 1, nmeas
      call alpha_prec(j)%update(Yaux(:,j), dedic%group(j), factors%theta)
    end do

  end subroutine gibbs_sweep_backward


  !=============================================================================

  subroutine backup_draws()
    implicit none
    integer :: j

    call dedic%backup()
    call factors%backup()
    call facdist%backup()
    do j = 1, nmeas
      call alpha_prec(j)%backup()
      call Xcov(j)%backup()
      call Ylat(j)%p%backup()
    end do

  end subroutine backup_draws


  !=============================================================================

  subroutine restore_draws()
    implicit none
    integer :: j

    call dedic%restore()
    call factors%restore()
    call facdist%restore()
    do j = 1, nmeas
      call alpha_prec(j)%restore()
      call Xcov(j)%restore()
      call Ylat(j)%p%restore()
    end do

  end subroutine restore_draws


end subroutine befa

module factor_normal_class

  use global
  use covmat_block_invwishart_class
  use probability, only : rnorm
  use matrix,      only : matinv, chol
  implicit none

  private
  public :: factor_normal


  type :: factor_normal
    integer               :: nobs
    integer               :: nmeas
    integer               :: nfac
    real(r8), allocatable :: theta(:,:)
    ! back up
    real(r8), allocatable :: theta_bak(:,:)
  contains
    procedure, public :: init    => init_factor_normal
    procedure, public :: update  => update_factor_normal
    procedure, public :: backup  => backup_factor_normal
    procedure, public :: restore => restore_factor_normal
  end type factor_normal


contains


  !-----------------------------------------------------------------------------

  subroutine init_factor_normal(this, nobs, nmeas, nfac, start)
    implicit none
    class(factor_normal) :: this
    integer,  intent(in) :: nobs
    integer,  intent(in) :: nmeas
    integer,  intent(in) :: nfac
    real(r8), intent(in) :: start(nobs,nfac)

    this%nobs  = nobs
    this%nmeas = nmeas
    this%nfac  = nfac

    allocate(this%theta(nobs, nfac))
    allocate(this%theta_bak(nobs, nfac))
    this%theta = start
    this%theta_bak = start

  end subroutine init_factor_normal


  !-----------------------------------------------------------------------------

  subroutine update_factor_normal(this, Y, alpha, dedic, idioprec, fdist)
    implicit none
    class(factor_normal)                       :: this
    real(r8),                       intent(in) :: Y(this%nobs, this%nmeas)
    real(r8),                       intent(in) :: idioprec(this%nmeas)
    real(r8),                       intent(in) :: alpha(this%nmeas)
    integer,                        intent(in) :: dedic(this%nmeas)
    class(covmat_block_invwishart), intent(in) :: fdist
    real(r8) :: ap2(this%nmeas)
    real(r8) :: ing(this%nmeas, this%nfac)
    real(r8) :: mean_post(this%nobs, this%nfac)
    real(r8) :: var_post(this%nfac, this%nfac)
    integer  :: i, j, k

    ! posterior covariance matrix
    var_post = fdist%prec
    ap2 = idioprec * (alpha**2)
    forall(k = 1:this%nfac) var_post(k,k) = var_post(k,k) + sum(ap2, dedic==k)
    var_post = matinv(var_post)

    ! posterior mean
    do j = 1, this%nmeas
      if(dedic(j) == 0) then
        ing(j,:) = 0._r8
      else
        forall(k = 1:this%nfac)
          ing(j,k) = var_post(dedic(j), k) * alpha(j) * idioprec(j)
        end forall
      end if
    end do
    mean_post = matmul(Y, ing)

    ! sample factors
    do k = 1, this%nfac
      do i = 1, this%nobs
        this%theta(i,k) = rnorm()
      end do
    end do
    this%theta = mean_post + matmul(this%theta, transpose(chol(var_post)))

  end subroutine update_factor_normal


  !-----------------------------------------------------------------------------

  subroutine backup_factor_normal(this)
    implicit none
    class(factor_normal) :: this

    this%theta_bak = this%theta

  end subroutine backup_factor_normal


  !-----------------------------------------------------------------------------

  subroutine restore_factor_normal(this)
    implicit none
    class(factor_normal) :: this

    this%theta = this%theta_bak

  end subroutine restore_factor_normal


end module factor_normal_class

module mcmc_progress_class

  implicit none

  type :: mcmc_progress
    logical      :: verbose
    integer      :: burnin
    integer      :: i
    integer      :: steps(20)
    character(6) :: perct(20)
  contains
    procedure, public :: init => init_mcmc_progress
    procedure, public :: show => show_mcmc_progress
  end type mcmc_progress


contains


  !-----------------------------------------------------------------------------

  subroutine init_mcmc_progress(this, burnin, iter, verbose)
    implicit none
    class(mcmc_progress) :: this
    integer, intent(in)  :: iter, burnin
    logical, intent(in)  :: verbose
    integer              :: i

    this%verbose = verbose
    this%burnin = burnin
    this%i = 1

    this%steps = (/(i, i=iter/20, iter, iter/20)/)
    this%steps(20) = iter   ! make sure "100%" displayed after last iteration

    this%perct = ["    5%", "   10%", "   15%", "   20%", "   25%", "   30%", &
                  "   35%", "   40%", "   45%", "   50%", "   55%", "   60%", &
                  "   65%", "   70%", "   75%", "   80%", "   85%", "   90%", &
                  "   95%", "  100%"]

  end subroutine init_mcmc_progress


  !-----------------------------------------------------------------------------

  subroutine show_mcmc_progress(this, rep)
    implicit none
    class(mcmc_progress) :: this
    integer, intent(in)  :: rep

    if(modulo(rep, 100) == 0) call rchkusr()

    if(this%verbose) then
      if(rep == this%burnin) then
        call intpr("done with burn-in period", 24, 0, 0)
      end if
      if(rep == this%steps(this%i)) then
        call intpr(this%perct(this%i), 6, 0, 0)
        this%i = this%i+1
      end if
    end if

  end subroutine show_mcmc_progress


end module mcmc_progress_class

#'
#' Generate synthetic data from a dedicated factor model
#'
#' This function simulates data from a dedicated factor model. The parameters of
#' the model are either passed by the user or simulated by the function.
#'
#' @param N
#'        Number of observations in data set.
#' @param dedic
#'        Vector of indicators. The number of manifest variables is equal to the
#'        length of this vector, and the number of factors is equal to the
#'        number of unique nonzero elements. Each integer element indicates on
#'        which latent factor the corresponding variable loads uniquely.
#' @param alpha
#'        Vector of factor loadings, should be of same length as \code{dedic}.
#'        If missing, values are simulated (see details below).
#' @param sigma
#'        Idiosyncratic variances, should be of same length as \code{dedic}.
#'        If missing, values are simulated (see details below).
#' @param R
#'        Covariance matrix of the latent factors.
#'        If missing, values are simulated (see details below).
#' @param R.corr
#'        If TRUE, covariance matrix \code{R} is rescaled to be a correlation
#'        matrix.
#' @param max.corr
#'        Maximum correlation allowed between the latent factors.
#' @param R.max.trial
#'        Maximum number of trials allowed to sample from the truncated
#'        distribution of the covariance matrix of the latent factors
#'        (accept/reject sampling scheme, to make sure \code{max.corr} is not
#'        exceeded).
#'
#' @details The function simulates data from the following dedicated factor
#' model, for \eqn{i = 1, ..., N}:
#'   \deqn{Y_i = \alpha \theta_i + \epsilon_i}
#'   \deqn{\theta_i \sim \mathcal{N}(0, R)}{\theta_i ~ N(0, R)}
#'   \deqn{\epsilon_i \sim \mathcal{N}(0, \Sigma)}{\epsilon_i ~ N(0, \Sigma)}
#' where the \eqn{K}-vector \eqn{\theta_i} contains the latent factors, and
#' \eqn{\alpha} is the \eqn{(M \times K)}{(M*K)}-matrix of factor loadings. Each
#' row \eqn{m} of \eqn{\alpha} contains only zeros, besides its element
#' indicated by the \eqn{m}th element of \code{dedic} that is equal to the
#' \eqn{m}th element of \code{alpha} (denoted \eqn{\alpha_m^\Delta} below).
#' The \eqn{M}-vector \eqn{\epsilon_i} is the vector of error terms, with
#' \eqn{\Sigma = diag(}\code{sigma}\eqn{)}. \eqn{M} is equal to the length of
#' the vector \code{dedic}, and \eqn{K} is equal to the maximum value of this
#' vector.
#'
#' Only \code{N} and \code{dedic} are required, all the other parameters can be
#' missing, completely or partially. Missing values (\code{NA}) are
#' independently sampled from the following distributions, for each manifest
#' variable \eqn{m = 1, ..., M}:
#'
#' Factor loadings:
#' \deqn{\alpha_m^\Delta = (-1)^{\phi_m}\sqrt{a_m}}{
#'       \alpha_m^\Delta = (-1)^\phi_m\sqrt(a_m)}
#' \deqn{\phi_m \sim \mathcal{B}er(0.5)}{\phi_m ~ Ber(0.5)}
#' \deqn{a_m \sim \mathcal{U}nif (0.04, 0.64)}{a_m ~ Unif (0.04, 0.64)}
#'
#' Idiosyncratic variances:
#' \deqn{\sigma^2_m \sim \mathcal{U}nif (0.2, 0.8)}{
#'       \sigma^2_m ~ Unif (0.2, 0.8)}
#'
#' For the variables that do not load on any factors (i.e., for which the
#' corresponding elements of \code{dedic} are equal to 0), it is specified that
#' \eqn{\alpha_m^\Delta = 0} and \eqn{\sigma^2_m = 1}.
#'
#' Covariance matrix of the latent factors:
#' \deqn{\Omega \sim \mathcal{I}nv-\mathcal{W}ishart(K+5, I_K)}{
#'       \Omega ~ Inv-Wishart(K+5, I_K)}
#' which is rescaled to be a correlation matrix if \code{R.corr = TRUE}:
#' \deqn{R = \Lambda^{-1/2} \Omega \Lambda^{-1/2}}{
#'       R = \Lambda^-1/2 \Omega \Lambda^-1/2}
#' \deqn{\Lambda = diag(\Omega)}
#'
#' Note that the distribution of the covariance matrix is truncated such that
#' all the off-diagonal elements of the implied correlation matrix \eqn{R} are
#' below \code{max.corr} in absolute value. The truncation is also applied if
#' the covariance matrix is used instead of the correlation matrix (i.e., if
#' \code{R.corr = FALSE}).
#'
#' The distributions and the corresponding default values used to simulate the
#' model parameters are specified as in the Monte Carlo study of CFSHP, see
#' section 4.1 (p.43).
#'
#' @return The function returns a data frame with \code{N} observations
#' simulated from the corresponding dedicated factor model.
#' The parameters used to generate the data are saved as attributes:
#' \code{dedic}, \code{alpha}, \code{sigma} and \code{R}.
#'
#' @author Rémi Piatek \email{remi.piatek@@gmail.com}
#'
#' @references
#' G. Conti, S. Frühwirth-Schnatter, J.J. Heckman,
#' R. Piatek (2014): ``Bayesian Exploratory Factor Analysis'',
#' \emph{Journal of Econometrics}, 183(1), pages 31-57,
#' \url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.
#'
#' @examples
#' # generate 1000 observations from model with 4 factors and 20 variables
#' # (5 variables loading on each factor)
#' dat <- simul.dedic.facmod(N = 1000, dedic = rep(1:4, each = 5))
#'
#' # generate data set with 5000 observations from the following model:
#' dedic <- rep(1:3, each = 4)        # 3 factors and 12 manifest variables
#' alpha <- rep(c(1, NA, NA, NA), 3)  # set first loading to 1 for each factor,
#'                                    #   sample remaining loadings from default
#' sigma <- rep(0.5, 12)              # idiosyncratic variances all set to 0.5
#' R <- toeplitz(c(1, .6, .3))        # Toeplitz matrix
#' dat <- simul.dedic.facmod(N = 5000, dedic, alpha, sigma, R)
#'
#' @export simul.dedic.facmod
#' @import checkmate
#' @importFrom stats rWishart rnorm runif cov2cor

simul.dedic.facmod <- function(N, dedic, alpha, sigma, R, R.corr = TRUE,
                               max.corr = 0.85, R.max.trial = 1000)
{

  # sanity checks for N and dedic, relabel appropriately
  assertInt(N, lower = 1)
  assertIntegerish(dedic, lower = 0, any.missing = FALSE, min.len = 3)
  dedic <- relabel.dedic(dedic)
  nvar  <- length(dedic)
  nfac  <- max(dedic)

  # factor loading matrix
  if (missing(alpha)) {
    alpha <- rep(NA, nvar)
  }
  assertNumeric(alpha, finite = TRUE, len = nvar)
  nna <- sum(is.na(alpha))
  if (nna > 0) {
    alpha[is.na(alpha)] <- sample(c(-1, 1), nna, replace = TRUE) *
                             sqrt(runif(nna, min = 0.04, max = 0.64))
  }
  alpha.mat <- matrix(0, nvar, nfac)
  for (i in 1:nvar) {
    if (dedic[i] > 0)
      alpha.mat[i, dedic[i]] <- alpha[i]
  }

  # error terms
  if (missing(sigma)) sigma <- rep(NA, nvar)
  assertNumeric(sigma, lower = 0.001, finite = TRUE, len = nvar)
  nna <- sum(is.na(sigma))
  if (nna > 0) sigma[is.na(sigma)] <- runif(nna, min = 0.2, max = 0.8)
  sigma[dedic == 0] <- 1
  eps <- matrix(rnorm(N * nvar), N, nvar) %*% diag(sqrt(sigma))

  # latent factors
  assertLogical(R.corr, len = 1, any.missing = FALSE)
  assertNumber(max.corr, lower = 0, upper = 1)
  assertInt(R.max.trial, lower = 1, upper = 10^6)
  if (missing(R)) {
    for (i in 1:R.max.trial) {
      Omega <- rWishart(n = 1, df = nfac + 5, Sigma = diag(nfac))
      R <- cov2cor(Omega[, , 1])
      rho <- max(abs(R[lower.tri(R)]))
      if (rho <= max.corr)
        break
    }
    if (rho > max.corr)
      stop("R.max.trial exceeded, could not sample R")
    if (!R.corr)
      R <- Omega
  } else {
    if (!is.pos.semidefinite.matrix(R))
      stop("R is not positive semidefinite.")
    if (R.corr & !all(diag(R) == 1))
      stop("R is not a correlation matrix")
  }
  theta <- matrix(rnorm(N * nfac), N, nfac) %*% chol(R)

  # manifest variables
  Y <- theta %*% t(alpha.mat) + eps
  colnames(Y) <- paste0("Y", 1:nvar)
  rownames(Y) <- 1:N

  # label parameters
  names(dedic) <- paste0("Y", 1:nvar)
  names(alpha) <- paste0("alpha:", 1:nvar)
  names(sigma) <- paste0("sigma:", 1:nvar)
  rownames(R)  <- colnames(R) <- paste0("R:", 1:nfac)

  # return simulated data
  output <- as.data.frame(Y)
  attr(output, "dedic") <- dedic
  attr(output, "alpha") <- alpha
  attr(output, "sigma") <- sigma
  attr(output, "R")     <- R
  return(output)

}

#'
#' Simulate prior distribution of number of latent factors
#'
#' This function produces a sample from the prior distribution of the number of
#' latent factors. It depends on the prior parameters used for the distribution
#' of the indicators, on the size of the model (number of manifest variables
#' and maximum number of latent factors), and on the identification restriction
#' (minimum number of manifest variables dedicated to each factor).
#'
#' @inheritParams befa
#' @param nvar Number of manifest variables.
#' @param Kmax Maximum number of latent factors.
#' @param kappa Concentration parameter of the Dirichlet prior distribution on
#'        the indicators.
#' @param nrep  Number of Monte Carlo replications.
#'
#' @details This function simulates the prior distribution of the number of
#' latent factors for models that fulfill the identification restriction
#' restriction that at least \code{Nid} manifest variables (or no variables) are
#' loading on each latent factor. Several (scalar) parameters \code{kappa} can
#' be passed to the function to simulate the prior for different prior parameter
#' values and compare the results.
#'
#' An accept/reject sampling scheme is used: a vector of probabilities is drawn
#' from a Dirichlet distribution with concentration parameter \code{kappa}, and
#' the \code{nvar} manifest variables are randomly allocated to the \code{Kmax}
#' latent factors. If each latent factor has at least \code{Nid} dedicated
#' variables or no variables at all, the identification requirement is fulfilled
#' and the draw is accepted. The number of factors loaded by at least \code{Nid}
#' manifest variables is returned as a draw from the prior distribution.
#'
#' Note that this function does not use the two-level prior distribution
#' implemented in CFSHP, where manifest variables can be discarded from the
#' model according to a given probability. Therefore, this function only help
#' understand the prior distribution conditional on all the manifest variables
#' being included into the model.
#'
#' @return A list of length equal to the number of parameters specified in
#' \code{kappa} is returned, where each element of the list contains:
#' \itemize{
#'   \item \code{nfac}: Vector of integers of length equal to the number of
#'   accepted draws.
#'   \item \code{acc}: Acceptance rate of the accept/reject sampling scheme.
#' }
#'
#' @author Rémi Piatek \email{remi.piatek@@gmail.com}
#'
#' @references G. Conti, S. Frühwirth-Schnatter, J.J. Heckman, R. Piatek (2014):
#' ``Bayesian Exploratory Factor Analysis'', \emph{Journal of Econometrics},
#' 183(1), pages 31-57, \url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.
#'
#' @examples
#' # replicate first row of table 2 in CFSHP (p.44)
#' # note: use larger number of replications nrep to improve accuracy
#' prior.nfac <- simul.nfac.prior(nvar = 15, Kmax = 5, kappa = c(.3, .7, 1), 
#'                                nrep = 10000)
#' summary(prior.nfac)
#' plot(prior.nfac)
#'
#' @export simul.nfac.prior
#' @import checkmate
#' @useDynLib BayesFM, .registration = TRUE, .fixes = "F_"

simul.nfac.prior <- function(nvar, Kmax, Nid = 3, kappa = 1/Kmax, nrep = 10^6)
{

  # sanity checks
  checkArgs <- makeAssertCollection()
  assertInt(nvar, lower = 1, add = checkArgs)
  assertInt(Kmax,  lower = 1, upper = nvar, add = checkArgs)
  assertInt(Nid,   lower = 1, upper = floor(nvar/Kmax), add = checkArgs)
  assertNumeric(kappa, lower = 10^-7, finite = TRUE, any.missing = FALSE,
                min.len = 1, unique = TRUE, add = checkArgs)
  assertCount(nrep, positive = TRUE, add = checkArgs)
  reportAssertions(checkArgs)

  out <- list()
  for (kap in kappa) {

    seed <- round(runif(1) * 10^9)
    sim  <- .Fortran(F_simnfacprior,
                     as.integer (nvar),
                     as.integer (Kmax),
                     as.integer (Nid),
                     as.double  (rep(kap, Kmax)),
                     as.integer (nrep),
                     as.integer (seed),
                     nfac    = integer(nrep),
                     restrid = logical(nrep),
                     PACKAGE = 'BayesFM')

    lab <- paste('kappa =', kap)
    out[[lab]] <- list(nfac = sim$nfac[sim$restrid],
                       acc  = sum(sim$restrid) / nrep)

  }

  attr(out, 'call')  <- match.call()
  attr(out, 'nvar')  <- nvar
  attr(out, 'Kmax')  <- Kmax
  attr(out, 'Nid')   <- Nid
  attr(out, 'kappa') <- kappa
  attr(out, 'nrep')  <- nrep
  class(out) <- 'simul.nfac.prior'
  return(out)

}

#' @export

summary.simul.nfac.prior <- function(object, ...)
{
  Kmax <- attr(object, 'Kmax')

  comp.freq <- function(x) table(factor(x, levels = 1:Kmax))/length(x)

  acc  <- lapply(object, '[[', 'acc')
  nfac <- lapply(object, '[[', 'nfac')
  nfac <- lapply(nfac, comp.freq)

  out <- list(nfac = nfac, acc = acc)
  class(out) <- 'summary.simul.nfac.prior'
  return(out)
}

subroutine simnfacprior(nmeas, Kmax, Nid, kappa, nrep, seed, nfac, restrid)

  use global
  use probability, only : set_seed, rdirich, runif
  implicit none

  integer,  intent(in)  :: nmeas
  integer,  intent(in)  :: Kmax
  integer,  intent(in)  :: Nid
  real(r8), intent(in)  :: kappa(Kmax)
  integer,  intent(in)  :: nrep
  integer,  intent(in)  :: seed
  integer,  intent(out) :: nfac(nrep)
  logical,  intent(out) :: restrid(nrep)

  real(r8) :: prob(Kmax)
  integer  :: dedic(nmeas)
  integer  :: ndedic(Kmax)
  real(r8) :: e, csum
  logical  :: checkid
  integer  :: i, j, k

  checkid = Nid > 1
  restrid = .true.

  call set_seed(seed)

  do i = 1, nrep

    ! sample indicator probabilities
    prob = rdirich(kappa)

    ! sample indicators
    ndedic = 0
    do j = 1, nmeas
      e = runif()
      csum = 0._r8
      do k = 1, Kmax
        csum = csum + prob(k)
        if(e <= csum) exit
      end do
      dedic(j) = k
      ndedic(k) = ndedic(k) + 1
    end do

    ! count number of factors
    nfac(i) = count(ndedic > 0)

    ! check identification restriction
    if(checkid) restrid(i) = all(ndedic >= Nid .or. ndedic == 0)

  end do

end subroutine simnfacprior

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/befa.R
\name{befa}
\alias{befa}
\title{Bayesian Exploratory Factor Analysis}
\usage{
befa(model, data, burnin = 1000, iter = 10000, Nid = 3, Kmax, A0 = 10,
  B0 = 10, c0 = 2, C0 = 1, HW.prior = TRUE, nu0 = Kmax + 1, S0 = 1,
  kappa0 = 2, xi0 = 1, kappa = 1/Kmax, indp.tau0 = TRUE,
  rnd.step = TRUE, n.step = 5, search.delay = min(burnin, 10),
  R.delay = min(burnin, 100), dedic.start, alpha.start, sigma.start,
  beta.start, R.start, verbose = TRUE)
}
\arguments{
\item{model}{This argument specifies the manifest variables and the covariates used
in the model (if any). It can be specified in two different ways:
\itemize{
\item A numeric matrix or a data frame containing the manifest
  variables. This corresponds to a model without covariates,
  and the argument \code{data} is not required.
\item A list of model formulas. Each element of the list specifies
  a manifest variable and its corresponding control variables (e.g.,
  '\code{Y1 ~ X1 + X2}' to use \code{X1} and \code{X2} as control
  variables for \code{Y1}).
  If a formula has no left-hand side variable, covariates on the
  right-hand side are included in all equations (e.g., '\code{~ X3}'
  means that \code{X3} is used as a control variable for all the
  manifest variables). Argument \code{data} can be passed to the
  function in that case, otherwise parent data frame is used.
}
Binary manifest variables should be specified as logical vectors in
the data frame to be treated as dichotomous. \code{NA} values are
accepted in manifest variables only.}

\item{data}{Data frame. If missing, parent data frame if used.}

\item{burnin}{Burn-in period of the MCMC sampler.}

\item{iter}{Number of MCMC iterations saved for posterior inference (after
burn-in).}

\item{Nid}{Minimum number of manifest variables dedicated to each latent factor
for identification.}

\item{Kmax}{Maximum number of latent factors. If missing, the maximum number of
factors that satisfies the identification condition determined by
\code{Nid} and the Ledermann bound is specified (see CFSHP,
section 2.2).}

\item{A0}{Scaling parameters of the variance of the Normal prior on the nonzero
factor loadings. Either a scalar or a numeric vector of length equal
to the number of manifest variables.}

\item{B0}{Variances of the Normal prior on the regression coefficients. Either a
scalar or a numeric vector of length equal to the number of
manifest variables.}

\item{c0}{Shape parameters of the Inverse-Gamma prior on the idiosyncratic
variances. Either a scalar or a numeric vector of length equal to the
number of manifest variables.}

\item{C0}{Scale parameters of the Inverse-Gamma prior on the idiosyncratic
variances. Either a scalar or a numeric vector of length equal to the
number of manifest variables.}

\item{HW.prior}{If \code{TRUE}, implement Huang-Wand (2013) prior on the covariance
matrix of the factors in the expanded model, otherwise use an
Inverse-Wishart prior if \code{FALSE}, see CFSHP section 2.3.5.}

\item{nu0}{Degrees of freedom of the Inverse-Wishart prior on the covariance
matrix of the latent factors in the expanded model.}

\item{S0}{Scale parameters of the Inverse-Wishart prior on the covariance matrix
of latent factors in the expanded model:
\itemize{
  \item if \code{HW.prior = TRUE}, scale parameter of the Gamma
  hyperprior distribution on the individual scales of the
  Inverse-Wishart prior.
  \item if \code{HW.prior = FALSE}, diagonal elements of the scale
  matrix of the Inverse-Wishart prior on the covariance matrix of
  the latent factors in the expanded model.
}
Either a scalar or a numeric vector of length equal to \code{Kmax}.}

\item{kappa0}{First shape parameter of the Beta prior distribution on the
probability \eqn{\tau_0} that a manifest variable does not load on any
factor.}

\item{xi0}{Second shape parameter of the Beta prior distribution on the
probability \eqn{\tau_0} that a manifest variable does not load on any
factor.}

\item{kappa}{Concentration parameters of the Dirichlet prior distribution on the
indicators. Either a scalar or a numeric vector of length equal to
\code{Kmax}.}

\item{indp.tau0}{If \code{TRUE}, specify the alternative prior specification with
independent parameters \eqn{\tau_{0m}}{\tau_0m} across manifest
variables \eqn{m = 1, ..., M}, otherwise use a common parameter
\eqn{\tau_0} if \code{FALSE}.}

\item{rnd.step}{If \code{TRUE}, select randomly the number of intermediate steps in
non-identified models at each MCMC iteration, otherwise use a fixed
number of steps if \code{FALSE}.}

\item{n.step}{Controls the number of intermediate steps in non-identified models:
\itemize{
  \item if \code{rnd.step = TRUE}, average number of steps. The number
  of steps is sampled at each MCMC iteration from
  1+Poisson(\code{n.step}-1).
  \item if \code{rnd.step = FALSE}, fixed number of steps.
}}

\item{search.delay}{Number of MCMC iterations run with fixed indicator matrix (specified
in \code{dedic.start}) at beginning of MCMC sampling.}

\item{R.delay}{Number of MCMC iterations run with fixed correlation matrix (specified
in \code{dedic.start}) at beginning of MCMC sampling.}

\item{dedic.start}{Starting values for the indicators. Vector of integers of length equal
to the number of manifest variables. Each element takes a value among
0, 1, ..., \code{Kmax}. If missing, random allocation of the manifest
variables to the maximum number of factors \code{Kmax}, with a minimum
of \code{Nid} manifest variables loading on each factor.}

\item{alpha.start}{Starting values for the factor loadings. Numeric vector of length
equal to the number of manifest variables. If missing, sampled from a
Normal distribution with zero mean and variance \code{A0}.}

\item{sigma.start}{Starting values for the idiosyncratic variances. Numeric vector of
length equal to the number of manifest variables. Sampled from prior
if missing.}

\item{beta.start}{Starting values for the regression coefficients. Numeric vector of
length equal to the total number of regression coefficients,
concatenated for all the manifest variables. Sampled from prior if
missing.}

\item{R.start}{Starting values for the correlation matrix of the latent factors.
Numeric matrix with \code{Kmax} rows and columns, and unit diagonal
elements. If missing, identity matrix is used.}

\item{verbose}{If \code{TRUE}, display information on the progress of the function.}
}
\value{
The function returns an object of class '\code{befa}' containing the
MCMC draws of the model parameters saved in the following matrices (each
matrix has '\code{iter}' rows):
\itemize{
  \item \code{alpha}: Factor loadings.
  \item \code{sigma}: Idiosyncratic variances.
  \item \code{R}: Correlation matrix of the latent factors (off-diagonal
        elements only).
  \item \code{beta}: regression coefficients (if any).
  \item \code{dedic}: indicators (integers indicating on which factors the
        manifest variable load).
  }
The returned object also contains:
\itemize{
  \item \code{nfac}: Vector of number of 'active' factors across MCMC
        iterations (i.e., factors loaded by at least \code{Nid} manifest
        variables).
  \item \code{MHacc}: Logical vector indicating accepted proposals of
        Metropolis-Hastings algorithm.
}
The parameters \code{Kmax} and \code{Nid} are saved as object attributes, as
well as the function call and the number of mcmc iterations (\code{burnin}
and \code{iter}), and two logical variables indicating if the returned object
has been post processed to address the column switching problem
(\code{post.column.switch}) and the sign switching problem
(\code{post.sign.switch}).
}
\description{
This function implements the Bayesian Exploratory Factor Analysis
(\code{befa}) approach developed in Conti et al. (CFSHP, 2014). It runs a
MCMC sampler for a factor model with dedicated factors, where each manifest
variable is allowed to load on at most one latent factor. The allocation of
the manifest variables to the latent factors is not fixed \emph{a priori} but
determined stochastically during sampling. The minimum number of variables
dedicated to each factor can be controlled by the user to achieve the desired
level of identification. The manifest variables can be continuous or
dichotomous, and control variables can be introduced as covariates.
}
\details{
\strong{Model specification.} The model is specified as follows, for
each observation \eqn{i = 1, ..., N}:
  \deqn{Y^*_i = \beta X_i + \alpha \theta_i + \epsilon_i}{
        Y*_i = \beta X_i + \alpha \theta_i + \epsilon_i}
  \deqn{\theta_i \sim \mathcal{N}(0, R)}{\theta_i ~ N(0, R)}
  \deqn{\epsilon_i \sim \mathcal{N}(0, \Sigma)}{\epsilon_i ~ N(0, \Sigma)}
  \deqn{\Sigma = diag(\sigma^2_1, ..., \sigma^2_M)}
where \eqn{Y^*_i}{Y*_i} is the \eqn{M}-vector containing the latent
variables underlying the corresponding \eqn{M} manifest variables
\eqn{Y_i}, which can be continuous such that
\eqn{Y_{im} = Y^*_{im}}{Y_im = Y*_im}, or binary with
\eqn{Y_{im} = 1[Y^*_{im} > 0]}{Y_im = 1[Y*_im > 0]}, for \eqn{m = 1, ..., M}.
The \eqn{K}-vector \eqn{\theta_i} contains the latent factors, and
\eqn{\alpha} is the \eqn{(M \times K)}{(M*K)}-matrix of factor loadings.
The \eqn{M}-vector \eqn{\epsilon_i} is the vector of error terms.
Covariates can be included in the \eqn{Q}-vector \eqn{X_i} and are related to
the manifest variables through the \eqn{(M \times Q)}{(M*Q)}-matrix of
regression coefficients \eqn{\beta}. Intercept terms are automatically
included, but can be omitted in some or all equations using the usual syntax
for R formulae (e.g., 'Y1 ~ X1 - 1' specifies that that Y1 is regressed on X1
and no intercept is included in the corresponding equation).

The number of latent factors \eqn{K} is specified as \code{Kmax}. However,
during MCMC sampling the stochastic search process on the matrix \eqn{\alpha}
may produce zero columns, thereby reducing the number of active factors.

The covariance matrix \eqn{R} of the latent factors is assumed to be a
correlation matrix for identification.

Each row of the factor loading matrix \eqn{\alpha} contains at most one
nonzero element (dedicated factor model). The allocation of the manifest
variables to the latent factors is indicated by the binary matrix
\eqn{\Delta} with same dimensions as \eqn{\alpha}, such that each row
\eqn{\Delta_m} indicates which factor loading is different from zero, e.g.:
\deqn{\Delta_m = (0, .., 0, 1, 0, ..., 0) \equiv e_k}{
      \Delta_m = (0, .., 0, 1, 0, ..., 0) = e_k}
indicates that variable \eqn{m} loads on the \eqn{k}th factor, where
\eqn{e_k} is a \eqn{K}-vector that contains only zeros, besides its \eqn{k}th
element that equals 1.

\strong{Identification.} The function verifies that the maximum number of
latent factors \code{Kmax} does not exceed the Ledermann bound. It also
checks that \code{Kmax} is consistent with the identification restriction
specified with \code{Nid} (enough variables should be available to load on
the factors when \code{Kmax} is reached). The default value for \code{Kmax}
is the minimum between the Ledermann bound and the maximum number of factors
that can be loaded by \code{Nid} variables. The user is free to select the
level of identification, see CFSHP section 2.2 (non-identified models are
allowed with \code{Nid = 1}).

Note that identification is achieved only with respect to the scale of the
latent factors. Non-identifiability problems may affect the posterior sample
because of column switching and sign switching of the factor loadings.
These issues can be addressed \emph{a posteriori} with the functions
\code{\link{post.column.switch}} and \code{\link{post.sign.switch}}.

\strong{Prior specification.}
The indicators are assumed to have the following probabilities,
for \eqn{k = 1, ..., K}:
\deqn{Prob(\Delta_m = e_k \mid \tau_k) = \tau_k}{
      Prob(\Delta_m = e_k | \tau_k) = \tau_k}
\deqn{\tau = (\tau_0, \tau_1, ..., \tau_K)}
If \code{indp.tau0 = FALSE}, the probabilities are specified as:
\deqn{\tau = [\tau_0, (1-\tau_0)\tau^*_1, ..., (1-\tau_0)\tau^*_K]}{
      \tau = [\tau_0, (1-\tau_0)\tau*_1, ..., (1-\tau_0)\tau*_K]}
\deqn{\tau_0 \sim \mathcal{B}eta(\kappa_0, \xi_0)}{
      \tau_0 ~ Beta(\kappa_0, \xi_0)}
\deqn{\tau^* = (\tau^*_1, ..., \tau^*_K) \sim \mathcal{D}ir(\kappa)}{
      \tau* = (\tau*_1, ..., \tau*_K) ~ Dir(\kappa)}
with \eqn{\kappa_0} = \code{kappa0}, \eqn{\xi_0} = \code{xi0} and
\eqn{\kappa} = \code{kappa}.
Alternatively, if \code{indp.tau0 = TRUE}, the probabilities are specified
as:
\deqn{\tau_m = [\tau_{0m}, (1-\tau_{0m})\tau^*_1, ...,
        (1-\tau_{0m})\tau^*_K]}{
      \tau_m = [\tau_0m, (1-\tau_0m)\tau*_1, ..., (1-\tau_0m)\tau*_K]}
\deqn{\tau_{0m} \sim \mathcal{B}eta(\kappa_0, \xi_0)}{
      \tau_0m ~ Beta(\kappa_0, \xi_0)}
for each manifest variable \eqn{m = 1, ..., M}.

A normal-inverse-Gamma prior distribution is assumed on the nonzero factor
loadings and on the idiosyncratic variances:
\deqn{\sigma^2_m \sim \mathcal{I}nv-\mathcal{G}amma(c_{0m}, C_{0m})}{
      \sigma^2_m ~ Inv-Gamma(c0_m, C0_m)}
\deqn{\alpha_m^\Delta \mid \sigma^2_m \sim \mathcal{N}(0, A_{0m}\sigma^2_m)}{
      \alpha_m^\Delta | \sigma^2_m ~ N(0, A0_m * \sigma^2_m)}
where \eqn{\alpha_m^\Delta} denotes the only nonzero loading in the \eqn{m}th
row of \eqn{\alpha}.

For the regression coefficients, a multivariate Normal prior distribution is
assumed on each row \eqn{m = 1, ..., M} of \eqn{\beta}:
\deqn{\beta_m \sim \mathcal{N}(0, B_0 I_Q)}{\beta_m ~ N(0, B_0 I_Q)}
The covariates can be different across manifest variables, implying zero
restrictions on the matrix \eqn{\beta}. To specify covariates, use a list
of formulas as \code{model} (see example below). Intercept terms can be
introduced using

To sample the correlation matrix \eqn{R} of the latent factors, marginal data
augmentation is implemented (van Dyk and Meng, 2001), see CFSHP section 2.2.
Using the transformation \eqn{\Omega = \Lambda^{1/2} R \Lambda^{1/2}}, the
parameters \eqn{\Lambda = diag(\lambda_1, ..., \lambda_K)} are used as
\emph{working parameters}. These parameters correspond to the variances of
the latent factors in an expanded version of the model where the factors do
not have unit variances. Two prior distributions can be specified on the
covariance matrix \eqn{\Omega} in the expanded model:
\itemize{
  \item If \code{HW.prior = FALSE}, inverse-Wishart distribution:
  \deqn{\Omega \sim \mathcal{I}nv-\mathcal{W}ishart(\nu_0, diag(S_0))}{
        \Omega ~ Inv-Wishart(\nu_0, diag(S0))}
  with \eqn{\nu_0} = \code{nu0} and \eqn{S_0} = \code{S0}.
  \item If \code{HW.prior = TRUE}, Huang-Wand (2013) prior:
  \deqn{\Omega \sim \mathcal{I}nv-\mathcal{W}ishart(\nu_0, W), \qquad
        W = diag(w_1, ..., w_K)}{
        \Omega ~ Inv-Wishart(nu0, W), W = diag(w_1, ..., w_K)}
  \deqn{w_k \sim \mathcal{G}amma\left(\frac{1}{2},
        \frac{1}{2\nu^*S_{0k}}\right)}{w_k ~ Gamma(1/2, 1/(2\nu*S0_k))}
  with \eqn{\nu^*}{\nu*} = \code{nu0} - \code{Kmax} + 1, and the shape and
  rate parameters are specified such that the mean of the gamma distribution
  is equal to \eqn{\nu^* S_{0k}}{\nu* S0_k}, for each \eqn{k = 1, ..., K}.
}

\strong{Missing values.} Missing values (\code{NA}) are allowed in the
manifest variables. They are drawn from their corresponding conditional
distributions during MCMC sampling. Control variables with missing values
can be passed to the function. However, all the observations with at least
one missing value in the covariates are discarded from the sample (a warning
message is issued in that case).
}
\examples{
#### model without covariates

set.seed(6)

# generate fake data with 15 manifest variables and 3 factors
N <- 100    # number of observations
Y <- simul.dedic.facmod(N, dedic = rep(1:3, each = 5))

# run MCMC sampler
# notice: 1000 MCMC iterations for illustration purposes only,
#  increase this number to obtain reliable posterior results!
mcmc <- befa(Y, Kmax = 5, iter = 1000)

# post process MCMC draws to restore identification
mcmc <- post.column.switch(mcmc)
mcmc <- post.sign.switch(mcmc)
\donttest{
summary(mcmc)  # summarize posterior results
plot(mcmc)     # plot posterior results

# summarize highest posterior probability (HPP) model
summary(mcmc, what = 'hppm')

#### model with covariates

# generate covariates and regression coefficients
Xcov <- cbind(1, matrix(rnorm(4*N), ncol = 4))
colnames(Xcov) <- c('(Intercept)', paste0('X', 1:4))
beta <- rbind(rnorm(15), rnorm(15), diag(3) \%x\% t(rnorm(5)))

# add covariates to previous model
Y <- Y + Xcov \%*\% beta

# specify model
model <- c('~ X1',                        # X1 covariate in all equations
           paste0('Y', 1:5,   ' ~ X2'),   # X2 covariate for Y1-Y5 only
           paste0('Y', 6:10,  ' ~ X3'),   # X3 covariate for Y6-Y10 only
           paste0('Y', 11:15, ' ~ X4'))   # X4 covariate for Y11-Y15 only
model <- lapply(model, as.formula)        # make list of formulas

# run MCMC sampler, post process and summarize
mcmc <- befa(model, data = data.frame(Y, Xcov), Kmax = 5, iter = 1000)
mcmc <- post.column.switch(mcmc)
mcmc <- post.sign.switch(mcmc)
mcmc.sum <- summary(mcmc)
mcmc.sum

# compare posterior mean of regression coefficients to true values
beta.comp <- cbind(beta[beta != 0], mcmc.sum$beta[, 'mean'])
colnames(beta.comp) <- c('true', 'mcmc')
print(beta.comp, digits = 3)
}

}
\references{
G. Conti, S. Frühwirth-Schnatter, J.J. Heckman, R. Piatek (2014):
``Bayesian Exploratory Factor Analysis'', \emph{Journal of Econometrics},
183(1), pages 31-57, \url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.

A. Huang, M.P. Wand (2013):
``Simple Marginally Noninformative Prior Distributions for Covariance
Matrices'', \emph{Bayesian Analysis}, 8(2), pages 439-452,
\url{http://dx.doi.org/10.1214/13-BA815}.

D.A. van Dyk, X.-L. Meng (2001):
``The Art of Data Augmentation'',
\emph{Journal of Computational and Graphical Statistics}, 10(1), pages 1-50,
\url{http://dx.doi.org/10.1198/10618600152418584}.
}
\seealso{
\code{\link{post.column.switch}} and \code{\link{post.sign.switch}}
for column switching and sign switching of the factor loading matrix and of
the correlation matrix of the latent factors to restore identification
\emph{a posteriori}.

\code{\link{summary.befa}} and \code{\link{plot.befa}} to summarize
and plot the posterior results.

\code{\link{simul.R.prior}} and \code{\link{simul.nfac.prior}} to
simulate the prior distribution of the correlation matrix of the factors and
the prior distribution of the indicator matrix, respectively. This is useful
to perform prior sensitivity analysis and to understand the role of the
corresponding parameters in the factor search.
}
\author{
Rémi Piatek \email{remi.piatek@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simul.nfac.prior.R
\name{simul.nfac.prior}
\alias{simul.nfac.prior}
\title{Simulate prior distribution of number of latent factors}
\usage{
simul.nfac.prior(nvar, Kmax, Nid = 3, kappa = 1/Kmax, nrep = 10^6)
}
\arguments{
\item{nvar}{Number of manifest variables.}

\item{Kmax}{Maximum number of latent factors.}

\item{Nid}{Minimum number of manifest variables dedicated to each latent factor
for identification.}

\item{kappa}{Concentration parameter of the Dirichlet prior distribution on
the indicators.}

\item{nrep}{Number of Monte Carlo replications.}
}
\value{
A list of length equal to the number of parameters specified in
\code{kappa} is returned, where each element of the list contains:
\itemize{
  \item \code{nfac}: Vector of integers of length equal to the number of
  accepted draws.
  \item \code{acc}: Acceptance rate of the accept/reject sampling scheme.
}
}
\description{
This function produces a sample from the prior distribution of the number of
latent factors. It depends on the prior parameters used for the distribution
of the indicators, on the size of the model (number of manifest variables
and maximum number of latent factors), and on the identification restriction
(minimum number of manifest variables dedicated to each factor).
}
\details{
This function simulates the prior distribution of the number of
latent factors for models that fulfill the identification restriction
restriction that at least \code{Nid} manifest variables (or no variables) are
loading on each latent factor. Several (scalar) parameters \code{kappa} can
be passed to the function to simulate the prior for different prior parameter
values and compare the results.

An accept/reject sampling scheme is used: a vector of probabilities is drawn
from a Dirichlet distribution with concentration parameter \code{kappa}, and
the \code{nvar} manifest variables are randomly allocated to the \code{Kmax}
latent factors. If each latent factor has at least \code{Nid} dedicated
variables or no variables at all, the identification requirement is fulfilled
and the draw is accepted. The number of factors loaded by at least \code{Nid}
manifest variables is returned as a draw from the prior distribution.

Note that this function does not use the two-level prior distribution
implemented in CFSHP, where manifest variables can be discarded from the
model according to a given probability. Therefore, this function only help
understand the prior distribution conditional on all the manifest variables
being included into the model.
}
\examples{
# replicate first row of table 2 in CFSHP (p.44)
# note: use larger number of replications nrep to improve accuracy
prior.nfac <- simul.nfac.prior(nvar = 15, Kmax = 5, kappa = c(.3, .7, 1), 
                               nrep = 10000)
summary(prior.nfac)
plot(prior.nfac)

}
\references{
G. Conti, S. Frühwirth-Schnatter, J.J. Heckman, R. Piatek (2014):
``Bayesian Exploratory Factor Analysis'', \emph{Journal of Econometrics},
183(1), pages 31-57, \url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.
}
\author{
Rémi Piatek \email{remi.piatek@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BayesFM.R
\docType{package}
\name{BayesFM}
\alias{BayesFM}
\title{BayesFM: Package for Bayesian Factor Modeling}
\description{
The long-term goal of this package is to provide a collection of procedures
to perform Bayesian inference on a variety of factor models.
}
\details{
Currently, this package includes: Bayesian Exploratory Factor
Analysis (\code{befa}), as developed in Conti et al. (2014), an approach to
dedicated factor analysis with stochastic search on the structure of the
factor loading matrix. The number of latent factors, as well as the
allocation of the observed variables to the factors, are not fixed a priori
but determined during MCMC sampling. More approaches will be included in
future releases of this package.
}
\note{
This package is under development. You are very welcome to send me any
comments or suggestions for improvements, and to share with me any problems
you may encounter with the use of this package.
}
\references{
G. Conti, S. Frühwirth-Schnatter, J.J. Heckman, R. Piatek (2014):
``Bayesian Exploratory Factor Analysis'', \emph{Journal of Econometrics},
183(1), pages 31-57, \url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.
}
\author{
Rémi Piatek \email{remi.piatek@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.befa.R
\name{plot.befa}
\alias{plot.befa}
\title{Plot object of class 'befa'}
\usage{
\method{plot}{befa}(x, ...)
}
\arguments{
\item{x}{Object of class 'befa'.}

\item{...}{The following extra arguments can be specified:
\itemize{
  \item \code{what}: How to summarize the posterior distribution?
        \itemize{
          \item \code{what = 'maxp'} (default): Only factor loadings with
                highest posterior probability of being different from zero or
                discarded from the model (if \code{dedic = 0}) are
                summarized.
          \item \code{what = 'all'}: All factor loadings with corresponding
                posterior probability to be allocated to a given factor (or
                to be discarded from the model) larger than \code{min.prob}
                are summarized.
          \item \code{what = 'hppm'}: Highest posterior probability models
                with probability larger than \code{min.prob} are summarized.
        }
  \item \code{byfac}: Sort factor loadings by factors if \code{TRUE},
        otherwise by manifest variables if \code{FALSE} (default).
  \item \code{hpd.prob}: Probability used to compute the highest posterior
        density intervals of the posterior distribution of the model
        parameters (default: 0.95).
  \item \code{min.prob}: If \code{what = 'all'}, only factor loadings with
        posterior probability of being dedicated to a given factor (or
        discarded from the model) larger than this value are displayed.
        If \code{what = 'hppm'}, only highest posterior probability models
        with probability larger than this value are displayed. (default:
        0.20)
}}
}
\description{
This function makes different plots that are useful to assess the posterior
results: a trace plot of the number of latent factors (also showing
Metropolis-Hastings acceptance across MCMC replications), a histogram
of the posterior probabilities of the number of factors, heatmaps for the
inficator probabilities, the factor loading matrix, and the correlation
matrix of the latent factors.
}
\details{
This function makes graphs based on the summary results returned by
\code{\link{summary.befa}}. It therefore accepts the same optional arguments
as this function.
}
\examples{
set.seed(6)

# generate fake data with 15 manifest variables and 3 factors
Y <- simul.dedic.facmod(N = 100, dedic = rep(1:3, each = 5))

# run MCMC sampler and post process output
# notice: 1000 MCMC iterations for illustration purposes only,
#  increase this number to obtain reliable posterior results!
mcmc <- befa(Y, Kmax = 5, iter = 1000)
mcmc <- post.column.switch(mcmc)
mcmc <- post.sign.switch(mcmc)

# plot results for highest posterior probability model
plot(mcmc, what = 'hppm')

}
\seealso{
\code{\link{summary.befa}} to summarize posterior results.
}
\author{
Rémi Piatek \email{remi.piatek@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.befa.R
\name{summary.befa}
\alias{summary.befa}
\title{Summarize 'befa' object}
\usage{
\method{summary}{befa}(object, ...)
}
\arguments{
\item{object}{Object of class 'befa'.}

\item{...}{The following extra arguments can be specified:
\itemize{
  \item \code{what}: How to summarize the posterior distribution?
        \itemize{
          \item \code{what = 'maxp'} (default): Only factor loadings with
                highest posterior probability of being different from zero or
                discarded from the model (if \code{dedic = 0}) are
                summarized.
          \item \code{what = 'all'}: All factor loadings with corresponding
                posterior probability to be allocated to a given factor (or
                to be discarded from the model) larger than \code{min.prob}
                are summarized.
          \item \code{what = 'hppm'}: Highest posterior probability models
                with probability larger than \code{min.prob} are summarized.
        }
  \item \code{byfac}: Sort factor loadings by factors if \code{TRUE},
        otherwise by manifest variables if \code{FALSE} (default).
  \item \code{hpd.prob}: Probability used to compute the highest posterior
        density intervals of the posterior distribution of the model
        parameters (default: 0.95).
  \item \code{min.prob}: If \code{what = 'all'}, only factor loadings with
        posterior probability of being dedicated to a given factor (or
        discarded from the model) larger than this value are displayed.
        If \code{what = 'hppm'}, only highest posterior probability models
        with probability larger than this value are displayed. (default:
        0.20)
}}
}
\value{
If called directly, the summary is formatted and displayed on the
standard output. Otherwise if saved in an object, a list of the following
elements is returned:
\itemize{
 \item \code{MHacc}: Metropolis-Hastings acceptance rate.
 \item \code{alpha}: Data frame (or list of data frames if
       \code{what = 'hppm'}) containing posterior summary statistics for the
       factor loadings.
 \item \code{sigma}: Data frame (or list of matrices if \code{what = 'hppm'})
       containing posterior summary statistics for the idiosyncratic
       variances.
 \item \code{R}: Data frame (or list of data frames if \code{what = 'hppm'})
       containing posterior summary statistics for the factor correlations.
 \item \code{beta}: Data frame (or list of data frames if
       \code{what = 'hppm'}) containing posterior summary statistics for the
       regression coefficients (if any).
 \item \code{nfac} (only if \code{what = 'maxp'} or \code{what = 'all'}):
       Table of posterior frequencies of numbers of factors.
 \item \code{hppm} (only if \code{what = 'hppm'}): List of the following
       elements summarizing the different HPP models, sorted in decreasing
       order of their posterior probabilities:
       \itemize{
         \item \code{prob}: Vector of posterior probabilities.
         \item \code{nfac}: Vector of numbers of factors.
         \item \code{dedic}: Data frame of factor indicators.
       }
}
Data frames of posterior summary statistics include the means (\code{mean}),
standard deviations (\code{sd}) and highest posterior density intervals
(\code{hpd.lo} and \code{hpd.up}, for the probability specified in
\code{hpd.prob}) of the corresponding parameters.

For the factor loadings, the matrix may also include a column labeled
'\code{dedic}' indicating to which factors the corresponding manifest
variables are dedicated (a zero value means that the manifest variable does
not load on any factor), as well as a column labeled '\code{prob}' showing
the corresponding posterior probabilities that the manifest variables load on
these factors.

Summary results are returned as lists of data frames for HPP models, where
the elements of the list are labeled as '\code{m1}, '\code{m2}', etc.
}
\description{
Generic function summarizing the posterior results of a 'befa' object.
Optional arguments can be specified to customize the summary.
}
\details{
This function summarizes the posterior distribution of the parameters.
The algorithm may visit different configurations of the indicator matrix
\eqn{\Delta} during sampling, where the manifest variables are allocated to
different latent factors. When the posterior distribution of the factor
loadings is summarized separately for each manifest variable
(\code{what = 'maxp'} or \code{what = 'all'}), the function provides the
latent factor each manifest variable is allocated to (\code{dedic}), and the
corresponding posterior probability (\code{prob}). If \code{dedic = 0}, then
\code{prob} corresponds to the posterior probability that the manifest
variable is discarded. Discarded variables are listed last if
\code{byfac = TRUE}. Low probability cases can be discarded by setting
\code{min.prob} appropriately (default is 0.20).

Idiosyncratic variances, factor correlation matrix and regression
coefficients (if any) are summarized across all MCMC iterations if
\code{what = 'all'} or \code{what = 'maxp'}, and within each HPP model if
\code{what = 'hppm'}.

\strong{Highest posterior probability model.}
The HPP model is the model with a given allocation of the measurements to the
latent factors (i.e., a given indicator matrix \eqn{\Delta}) that is visited
most often by the algorithm.

When specifying \code{what = 'hppm'}, the function sorts the models according
to the posterior frequencies of their indicator matrices in decreasing order.
Therefore, the first model returned (labeled 'm1') corresponds to the HPP
model.
Low probability models can be discarded by setting \code{min.prob}
appropriately(default is 0.20, implying that only models with a posterior
probability larger than 0.20 are displayed).

HPP models can only be found if identification with respect to column
switching has been restored \emph{a posteriori}. An error message is returned
if this is not the case.
}
\examples{
set.seed(6)

# generate fake data with 15 manifest variables and 3 factors
Y <- simul.dedic.facmod(N = 100, dedic = rep(1:3, each = 5))

# run MCMC sampler and post process output
# notice: 1000 MCMC iterations for illustration purposes only,
#  increase this number to obtain reliable posterior results!
mcmc <- befa(Y, Kmax = 5, iter = 1000)
mcmc <- post.column.switch(mcmc)
mcmc <- post.sign.switch(mcmc)

# summarize posterior results
summary(mcmc)

# summarize highest posterior probability (HPP) model
hppm.sum <- summary(mcmc, what = 'hppm')

# print summary with 6-digit precision
print(hppm.sum, digits = 6)

# extract posterior means of the factor loadings in HPP model
alpha.mean <- hppm.sum$alpha$m1$mean
print(alpha.mean)

\dontshow{
summary(mcmc, what = 'maxp', byfac = TRUE)
summary(mcmc, what = 'all')
summary(mcmc, what = 'all', byfac = TRUE)
summary(mcmc, what = 'all', min.prob = 0)
summary(mcmc, what = 'all', min.prob = 0, byfac = TRUE)
summary(mcmc, what = 'hppm', byfac = TRUE)
summary(mcmc, what = 'hppm', min.prob = 0)
summary(mcmc, what = 'hppm', min.prob = 0, byfac = TRUE)
}

}
\seealso{
\code{\link{plot.befa}} to plot posterior results.
}
\author{
Rémi Piatek \email{remi.piatek@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/post.sign.switch.R
\name{post.sign.switch}
\alias{post.sign.switch}
\title{Perform sign switchting on posterior MCMC sample}
\usage{
post.sign.switch(mcmc, benchmark = NULL, benchmark.threshold = 0.5)
}
\arguments{
\item{mcmc}{Object of class '\code{befa}'.}

\item{benchmark}{Vector of integers of length equal to the maximum number of latent
factors. Each element indicates which factor loading is used as a
benchmark for the sign switch. If \code{NULL}, the factor loadings
with the highest posterior probabilities of being different from zero
in each column of the factor loading matrix are used as benchmarks.}

\item{benchmark.threshold}{Minimum posterior probability for a factor loading to be considered
as a benchmark.}
}
\value{
This function returns the same '\code{befa}' object, where the signs
of the factor loadings and of the factor correlations have been switched
appropriately to restore the identification of the factor model with respect
to sign switching.
}
\description{
This function performs a sign switch on the MCMC draws to restore the
consistency of the signs of the factors loadings and of the correlations of
the latent factors \emph{a posteriori}.
}
\details{
The signs of the factor loadings, as well as of the corresponding
correlations of the latent factors, are switched for each MCMC iteration such
that the factor loadings defined as \code{benchmark}s are positive. The sign
switch can only be performed if \code{\link{post.column.switch}} has been run
before. See section 4.3 (p.42) of CFSHP for more details.

If a latent factor has no benchmarks, or if its benchmark is equal to zero at
some MCMC iteration, then no sign switch is performed on the corresponding
loadings and correlations for this particular factor or MCMC iteration.

Note that in complicated models where the sampler visits several models with
different numbers of latent factors, it may not be relevant to use the
default value of \code{benchmark}, as the posterior probabilities that the
factor loadings are different from zero would be computed across models.
Instead, the user might consider finding the highest posterior probability
model first, and use its top elements in each column of the factor loading
matrix as benchmarks to perform the sign switch.
}
\examples{
set.seed(6)
Y <- simul.dedic.facmod(N = 100, dedic = rep(1:3, each = 5))
mcmc <- befa(Y, Kmax = 5, iter = 1000)
mcmc <- post.column.switch(mcmc)

# factor loadings corresponding to variables 1, 6, 11, 12 and 13 are
# used as benchmarks:
mcmc1 <- post.sign.switch(mcmc, benchmark = c(1, 6, 11, 12, 13))

# factor loadings with the highest posterior probability of being different
# from zero in each column are used as benchmark:
mcmc2 <- post.sign.switch(mcmc)

}
\references{
G. Conti, S. Frühwirth-Schnatter, J.J. Heckman,
R. Piatek (2014): ``Bayesian Exploratory Factor Analysis'',
\emph{Journal of Econometrics}, 183(1), pages 31-57,
\url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.
}
\seealso{
\code{\link{post.column.switch}} for column switching of the factor
loading matrix and of the correlation matrix of the latent factors to restore
identification \emph{a posteriori}.
}
\author{
Rémi Piatek \email{remi.piatek@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simul.R.prior.R
\name{simul.R.prior}
\alias{simul.R.prior}
\title{Simulate prior distribution of factor correlation matrix}
\usage{
simul.R.prior(Kmax, nu0 = Kmax + 1, S0 = 1, HW.prior = TRUE,
  nrep = 10^5, verbose = TRUE)
}
\arguments{
\item{Kmax}{Maximum number of latent factors.}

\item{nu0}{Degrees of freedom of the Inverse-Wishart prior on the covariance
matrix of the latent factors in the expanded model.}

\item{S0}{Scale parameters of the Inverse-Wishart prior on the covariance matrix
of latent factors in the expanded model:
\itemize{
  \item if \code{HW.prior = TRUE}, scale parameter of the Gamma
  hyperprior distribution on the individual scales of the
  Inverse-Wishart prior.
  \item if \code{HW.prior = FALSE}, diagonal elements of the scale
  matrix of the Inverse-Wishart prior on the covariance matrix of
  the latent factors in the expanded model.
}
Either a scalar or a numeric vector of length equal to \code{Kmax}.}

\item{HW.prior}{If \code{TRUE}, implement Huang-Wand (2013) prior on the covariance
matrix of the factors in the expanded model, otherwise use an
Inverse-Wishart prior if \code{FALSE}, see CFSHP section 2.3.5.}

\item{nrep}{Number of Monte Carlo replications.}

\item{verbose}{If \code{TRUE}, display information on the progress of the function.}
}
\value{
A list of length equal to the number of pairs of parameters
\code{nu0} and \code{S0}, where each element of the list is an array of
dimension (\code{Kmax}, \code{Kmax}, \code{nrep}) that contains the
correlation matrices of the latent factors drawn from the prior.
}
\description{
This function produces a sample of correlation matrices drawn from their
prior distribution induced in the identified version of the factor model,
given the prior distribution specified on the corresponding covariance
matrices of the factors in the expanded model.
}
\details{
Covariance matrices are sampled from the prior distribution in the
expanded model, and transformed to produce the corresponding correlation
matrices. See section 2.3.5 of CFSHP (p.36-37), as well as the details of
the function \code{\link{befa}}.

To compare several prior specifications, different values of the parameters
\code{nu0} and \code{S0} can be specified. The function then simulates for
each pair of these parameters. \code{nu0} and \code{S0} should therefore be
scalars or vectors of same length.
}
\examples{
# partial reproduction of figure 1 in CFSHP (p.38)
# note: use larger number of replications nrep to increase smoothness
Kmax <- 10
Rsim <- simul.R.prior(Kmax, nu0 = Kmax + c(1, 2, 5), S0 = .5, nrep = 1000)
summary(Rsim)
plot(Rsim)

}
\references{
G. Conti, S. Frühwirth-Schnatter, J.J. Heckman, R. Piatek (2014):
``Bayesian Exploratory Factor Analysis'', \emph{Journal of Econometrics},
183(1), pages 31-57, \url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.
}
\author{
Rémi Piatek \email{remi.piatek@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simul.dedic.facmod.R
\name{simul.dedic.facmod}
\alias{simul.dedic.facmod}
\title{Generate synthetic data from a dedicated factor model}
\usage{
simul.dedic.facmod(N, dedic, alpha, sigma, R, R.corr = TRUE,
  max.corr = 0.85, R.max.trial = 1000)
}
\arguments{
\item{N}{Number of observations in data set.}

\item{dedic}{Vector of indicators. The number of manifest variables is equal to the
length of this vector, and the number of factors is equal to the
number of unique nonzero elements. Each integer element indicates on
which latent factor the corresponding variable loads uniquely.}

\item{alpha}{Vector of factor loadings, should be of same length as \code{dedic}.
If missing, values are simulated (see details below).}

\item{sigma}{Idiosyncratic variances, should be of same length as \code{dedic}.
If missing, values are simulated (see details below).}

\item{R}{Covariance matrix of the latent factors.
If missing, values are simulated (see details below).}

\item{R.corr}{If TRUE, covariance matrix \code{R} is rescaled to be a correlation
matrix.}

\item{max.corr}{Maximum correlation allowed between the latent factors.}

\item{R.max.trial}{Maximum number of trials allowed to sample from the truncated
distribution of the covariance matrix of the latent factors
(accept/reject sampling scheme, to make sure \code{max.corr} is not
exceeded).}
}
\value{
The function returns a data frame with \code{N} observations
simulated from the corresponding dedicated factor model.
The parameters used to generate the data are saved as attributes:
\code{dedic}, \code{alpha}, \code{sigma} and \code{R}.
}
\description{
This function simulates data from a dedicated factor model. The parameters of
the model are either passed by the user or simulated by the function.
}
\details{
The function simulates data from the following dedicated factor
model, for \eqn{i = 1, ..., N}:
  \deqn{Y_i = \alpha \theta_i + \epsilon_i}
  \deqn{\theta_i \sim \mathcal{N}(0, R)}{\theta_i ~ N(0, R)}
  \deqn{\epsilon_i \sim \mathcal{N}(0, \Sigma)}{\epsilon_i ~ N(0, \Sigma)}
where the \eqn{K}-vector \eqn{\theta_i} contains the latent factors, and
\eqn{\alpha} is the \eqn{(M \times K)}{(M*K)}-matrix of factor loadings. Each
row \eqn{m} of \eqn{\alpha} contains only zeros, besides its element
indicated by the \eqn{m}th element of \code{dedic} that is equal to the
\eqn{m}th element of \code{alpha} (denoted \eqn{\alpha_m^\Delta} below).
The \eqn{M}-vector \eqn{\epsilon_i} is the vector of error terms, with
\eqn{\Sigma = diag(}\code{sigma}\eqn{)}. \eqn{M} is equal to the length of
the vector \code{dedic}, and \eqn{K} is equal to the maximum value of this
vector.

Only \code{N} and \code{dedic} are required, all the other parameters can be
missing, completely or partially. Missing values (\code{NA}) are
independently sampled from the following distributions, for each manifest
variable \eqn{m = 1, ..., M}:

Factor loadings:
\deqn{\alpha_m^\Delta = (-1)^{\phi_m}\sqrt{a_m}}{
      \alpha_m^\Delta = (-1)^\phi_m\sqrt(a_m)}
\deqn{\phi_m \sim \mathcal{B}er(0.5)}{\phi_m ~ Ber(0.5)}
\deqn{a_m \sim \mathcal{U}nif (0.04, 0.64)}{a_m ~ Unif (0.04, 0.64)}

Idiosyncratic variances:
\deqn{\sigma^2_m \sim \mathcal{U}nif (0.2, 0.8)}{
      \sigma^2_m ~ Unif (0.2, 0.8)}

For the variables that do not load on any factors (i.e., for which the
corresponding elements of \code{dedic} are equal to 0), it is specified that
\eqn{\alpha_m^\Delta = 0} and \eqn{\sigma^2_m = 1}.

Covariance matrix of the latent factors:
\deqn{\Omega \sim \mathcal{I}nv-\mathcal{W}ishart(K+5, I_K)}{
      \Omega ~ Inv-Wishart(K+5, I_K)}
which is rescaled to be a correlation matrix if \code{R.corr = TRUE}:
\deqn{R = \Lambda^{-1/2} \Omega \Lambda^{-1/2}}{
      R = \Lambda^-1/2 \Omega \Lambda^-1/2}
\deqn{\Lambda = diag(\Omega)}

Note that the distribution of the covariance matrix is truncated such that
all the off-diagonal elements of the implied correlation matrix \eqn{R} are
below \code{max.corr} in absolute value. The truncation is also applied if
the covariance matrix is used instead of the correlation matrix (i.e., if
\code{R.corr = FALSE}).

The distributions and the corresponding default values used to simulate the
model parameters are specified as in the Monte Carlo study of CFSHP, see
section 4.1 (p.43).
}
\examples{
# generate 1000 observations from model with 4 factors and 20 variables
# (5 variables loading on each factor)
dat <- simul.dedic.facmod(N = 1000, dedic = rep(1:4, each = 5))

# generate data set with 5000 observations from the following model:
dedic <- rep(1:3, each = 4)        # 3 factors and 12 manifest variables
alpha <- rep(c(1, NA, NA, NA), 3)  # set first loading to 1 for each factor,
                                   #   sample remaining loadings from default
sigma <- rep(0.5, 12)              # idiosyncratic variances all set to 0.5
R <- toeplitz(c(1, .6, .3))        # Toeplitz matrix
dat <- simul.dedic.facmod(N = 5000, dedic, alpha, sigma, R)

}
\references{
G. Conti, S. Frühwirth-Schnatter, J.J. Heckman,
R. Piatek (2014): ``Bayesian Exploratory Factor Analysis'',
\emph{Journal of Econometrics}, 183(1), pages 31-57,
\url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.
}
\author{
Rémi Piatek \email{remi.piatek@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/post.column.switch.R
\name{post.column.switch}
\alias{post.column.switch}
\title{Perform column switchting on posterior MCMC sample}
\usage{
post.column.switch(mcmc)
}
\arguments{
\item{mcmc}{Object of class '\code{befa}'.}
}
\value{
Same '\code{befa}' object as the one passed to the function, where
the indicators in the matrix \code{dedic}, as well as the rows and columns of
the correlation matrix of the factors saved in \code{draws}, have been
switched appropriately to restore the identification of the factor model with
respect to column switching.
}
\description{
This function reorders the columns of the factor loading matrix for each MCMC
draw, as well as the rows and columns of the correlation matrix of the
factors, to restore the identification of the model \emph{a posteriori} with
respect to the column switching problem.
}
\details{
The reordering of the columns of the factor loading matrix is based
on the top elements of the columns (i.e., the first row containing a nonzero
factor loading in each nonzero column of \eqn{\alpha}, starting from the top
of the matrix). At each MCMC iteration, the nonzero columns of \eqn{\alpha}
are reordered such that the top elements appear in increasing order.
The rows and columns of the correlation matrix \eqn{R} of the factors are
switched accordingly. See section 4.3 of CFSHP (p.42) for more details.
}
\examples{
set.seed(6)
Y <- simul.dedic.facmod(N = 100, dedic = rep(1:3, each = 5))
mcmc <- befa(Y, Kmax = 5, iter = 1000)
mcmc <- post.column.switch(mcmc)

}
\references{
G. Conti, S. Frühwirth-Schnatter, J.J. Heckman,
R. Piatek (2014): ``Bayesian Exploratory Factor Analysis'',
\emph{Journal of Econometrics}, 183(1), pages 31-57,
\url{http://dx.doi.org/10.1016/j.jeconom.2014.06.008}.
}
\seealso{
\code{\link{post.sign.switch}} to restore identification a
posteriori with respect to the sign switching problem.
}
\author{
Rémi Piatek \email{remi.piatek@gmail.com}
}
